@@ -27,6 +27,7 @@
 
 #include "jsonkeys.h"
 #include "lsptypes.h"
+#include "semantictokens.h"
 
 namespace LanguageServerProtocol {
 
@@ -38,21 +39,94 @@ public:
     Utils::optional<bool> dynamicRegistration() const { return optionalValue<bool>(dynamicRegistrationKey); }
     void setDynamicRegistration(bool dynamicRegistration) { insert(dynamicRegistrationKey, dynamicRegistration); }
     void clearDynamicRegistration() { remove(dynamicRegistrationKey); }
+};
 
-    bool isValid(QStringList *error) const override
-    { return checkOptional<bool>(error, dynamicRegistrationKey); }
+class LANGUAGESERVERPROTOCOL_EXPORT FullSemanticTokenOptions : public JsonObject
+{
+public:
+    using JsonObject::JsonObject;
+
+    /**
+      * The client will send the `textDocument/semanticTokens/full/delta`
+      * request if the server provides a corresponding handler.
+      */
+    Utils::optional<bool> delta() const { return optionalValue<bool>(deltaKey); }
+    void setDelta(bool delta) { insert(deltaKey, delta); }
+    void clearDelta() { remove(deltaKey); }
+};
+
+class LANGUAGESERVERPROTOCOL_EXPORT SemanticTokensClientCapabilities : public DynamicRegistrationCapabilities
+{
+public:
+    using DynamicRegistrationCapabilities::DynamicRegistrationCapabilities;
+    class LANGUAGESERVERPROTOCOL_EXPORT Requests : public JsonObject
+    {
+        /**
+         * Which requests the client supports and might send to the server
+         * depending on the server's capability. Please note that clients might not
+         * show semantic tokens or degrade some of the user experience if a range
+         * or full request is advertised by the client but not provided by the
+         * server. If for example the client capability `requests.full` and
+         * `request.range` are both set to true but the server only provides a
+         * range provider the client might not render a minimap correctly or might
+         * even decide to not show any semantic tokens at all.
+         */
+    public:
+        using JsonObject::JsonObject;
+
+        /**
+         * The client will send the `textDocument/semanticTokens/range` request
+         * if the server provides a corresponding handler.
+         */
+        Utils::optional<Utils::variant<bool, QJsonObject>> range() const;
+        void setRange(const Utils::variant<bool, QJsonObject> &range);
+        void clearRange() { remove(rangeKey); }
+
+        /**
+         * The client will send the `textDocument/semanticTokens/full` request
+         * if the server provides a corresponding handler.
+         */
+        Utils::optional<Utils::variant<bool, FullSemanticTokenOptions>> full() const;
+        void setFull(const Utils::variant<bool, FullSemanticTokenOptions> &full);
+        void clearFull() { remove(fullKey); }
+    };
+
+    Requests requests() const { return typedValue<Requests>(requestsKey); }
+    void setRequests(const Requests &requests) { insert(requestsKey, requests); }
+
+    /// The token types that the client supports.
+    QList<QString> tokenTypes() const { return array<QString>(tokenTypesKey); }
+    void setTokenTypes(const QList<QString> &value) { insertArray(tokenTypesKey, value); }
+
+    /// The token modifiers that the client supports.
+    QList<QString> tokenModifiers() const { return array<QString>(tokenModifiersKey); }
+    void setTokenModifiers(const QList<QString> &value) { insertArray(tokenModifiersKey, value); }
+
+    /// The formats the clients supports.
+    QList<QString> formats() const { return array<QString>(formatsKey); }
+    void setFormats(const QList<QString> &value) { insertArray(formatsKey, value); }
+
+    /// Whether the client supports tokens that can overlap each other.
+    Utils::optional<bool> overlappingTokenSupport() const { return optionalValue<bool>(overlappingTokenSupportKey); }
+    void setOverlappingTokenSupport(bool overlappingTokenSupport) { insert(overlappingTokenSupportKey, overlappingTokenSupport); }
+    void clearOverlappingTokenSupport() { remove(overlappingTokenSupportKey); }
+
+    /// Whether the client supports tokens that can span multiple lines.
+    Utils::optional<bool> multiLineTokenSupport() const { return optionalValue<bool>(multiLineTokenSupportKey); }
+    void setMultiLineTokenSupport(bool multiLineTokenSupport) { insert(multiLineTokenSupportKey, multiLineTokenSupport); }
+    void clearMultiLineTokenSupport() { remove(multiLineTokenSupportKey); }
+
+    bool isValid() const override;
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT SymbolCapabilities : public DynamicRegistrationCapabilities
 {
 public:
-    SymbolCapabilities();
     using DynamicRegistrationCapabilities::DynamicRegistrationCapabilities;
 
-    class SymbolKindCapabilities : public JsonObject
+    class LANGUAGESERVERPROTOCOL_EXPORT SymbolKindCapabilities : public JsonObject
     {
     public:
-        SymbolKindCapabilities();
         using JsonObject::JsonObject;
 
         /*
@@ -68,9 +142,6 @@ public:
         Utils::optional<QList<SymbolKind>> valueSet() const;
         void setValueSet(const QList<SymbolKind> &valueSet);
         void clearValueSet() { remove(valueSetKey); }
-
-        bool isValid(QStringList *error) const override
-        { return checkOptionalArray<int>(error, valueSetKey); }
     };
 
     // Specific capabilities for the `SymbolKind` in the `workspace/symbol` request.
@@ -79,19 +150,21 @@ public:
     void setSymbolKind(const SymbolKindCapabilities &symbolKind) { insert(symbolKindKey, symbolKind); }
     void clearSymbolKind() { remove(symbolKindKey); }
 
-    bool isValid(QStringList *error) const override;
+    Utils::optional<bool> hierarchicalDocumentSymbolSupport() const
+    { return optionalValue<bool>(hierarchicalDocumentSymbolSupportKey); }
+    void setHierarchicalDocumentSymbolSupport(bool hierarchicalDocumentSymbolSupport)
+    { insert(hierarchicalDocumentSymbolSupportKey, hierarchicalDocumentSymbolSupport); }
+    void clearHierachicalDocumentSymbolSupport() { remove(hierarchicalDocumentSymbolSupportKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT TextDocumentClientCapabilities : public JsonObject
 {
 public:
-    TextDocumentClientCapabilities();
     using JsonObject::JsonObject;
 
-    class SynchronizationCapabilities : public DynamicRegistrationCapabilities
+    class LANGUAGESERVERPROTOCOL_EXPORT SynchronizationCapabilities : public DynamicRegistrationCapabilities
     {
     public:
-        SynchronizationCapabilities();
         using DynamicRegistrationCapabilities::DynamicRegistrationCapabilities;
 
         // The client supports sending will save notifications.
@@ -114,8 +187,6 @@ public:
         Utils::optional<bool> didSave() const { return optionalValue<bool>(didSaveKey); }
         void setDidSave(bool didSave) { insert(didSaveKey, didSave); }
         void clearDidSave() { remove(didSaveKey); }
-
-        bool isValid(QStringList *error) const override;
     };
 
     Utils::optional<SynchronizationCapabilities> synchronization() const
@@ -124,16 +195,33 @@ public:
     { insert(synchronizationKey, synchronization); }
     void clearSynchronization() { remove(synchronizationKey); }
 
-    class CompletionCapabilities : public DynamicRegistrationCapabilities
+    class LANGUAGESERVERPROTOCOL_EXPORT SemanticHighlightingCapabilities : public JsonObject
+    {
+    public:
+        using JsonObject::JsonObject;
+
+        bool semanticHighlighting() const { return typedValue<bool>(semanticHighlightingKey); }
+        void setSemanticHighlighting(bool semanticHighlighting)
+        { insert(semanticHighlightingKey, semanticHighlighting); }
+
+        bool isValid() const override { return contains(semanticHighlightingKey); }
+    };
+
+    Utils::optional<SemanticHighlightingCapabilities> semanticHighlightingCapabilities() const
+    { return optionalValue<SemanticHighlightingCapabilities>(semanticHighlightingCapabilitiesKey); }
+    void setSemanticHighlightingCapabilities(
+        const SemanticHighlightingCapabilities &semanticHighlightingCapabilities)
+    { insert(semanticHighlightingCapabilitiesKey, semanticHighlightingCapabilities); }
+    void clearSemanticHighlightingCapabilities() { remove(semanticHighlightingCapabilitiesKey); }
+
+    class LANGUAGESERVERPROTOCOL_EXPORT CompletionCapabilities : public DynamicRegistrationCapabilities
     {
     public:
-        CompletionCapabilities();
         using DynamicRegistrationCapabilities::DynamicRegistrationCapabilities;
 
-        class CompletionItemCapbilities : public JsonObject
+        class LANGUAGESERVERPROTOCOL_EXPORT CompletionItemCapbilities : public JsonObject
         {
         public:
-            CompletionItemCapbilities();
             using JsonObject::JsonObject;
 
             /*
@@ -164,13 +252,6 @@ public:
             Utils::optional<QList<MarkupKind>> documentationFormat() const;
             void setDocumentationFormat(const QList<MarkupKind> &documentationFormat);
             void clearDocumentationFormat() { remove(documentationFormatKey); }
-
-            bool isValid(QStringList *error) const override
-            {
-                return checkOptional<bool>(error, snippetSupportKey)
-                        && checkOptional<bool>(error, commitCharacterSupportKey)
-                        && checkOptionalArray<int>(error, documentationFormatKey);
-            }
         };
 
         // The client supports the following `CompletionItem` specific capabilities.
@@ -180,7 +261,7 @@ public:
         { insert(completionItemKey, completionItem); }
         void clearCompletionItem() { remove(completionItemKey); }
 
-        class CompletionItemKindCapabilities : public JsonObject
+        class LANGUAGESERVERPROTOCOL_EXPORT CompletionItemKindCapabilities : public JsonObject
         {
         public:
             CompletionItemKindCapabilities();
@@ -198,9 +279,6 @@ public:
             Utils::optional<QList<CompletionItemKind::Kind>> valueSet() const;
             void setValueSet(const QList<CompletionItemKind::Kind> &valueSet);
             void clearValueSet() { remove(valueSetKey); }
-
-            bool isValid(QStringList *error) const override
-            { return checkOptionalArray<int>(error, valueSetKey); }
         };
 
         Utils::optional<CompletionItemKindCapabilities> completionItemKind() const
@@ -216,8 +294,6 @@ public:
         Utils::optional<bool> contextSupport() const { return optionalValue<bool>(contextSupportKey); }
         void setContextSupport(bool contextSupport) { insert(contextSupportKey, contextSupport); }
         void clearContextSupport() { remove(contextSupportKey); }
-
-        bool isValid(QStringList *error) const override;
     };
 
     // Capabilities specific to the `textDocument/completion`
@@ -227,7 +303,7 @@ public:
     { insert(completionKey, completion); }
     void clearCompletion() { remove(completionKey); }
 
-    class HoverCapabilities : public DynamicRegistrationCapabilities
+    class LANGUAGESERVERPROTOCOL_EXPORT HoverCapabilities : public DynamicRegistrationCapabilities
     {
     public:
         using DynamicRegistrationCapabilities::DynamicRegistrationCapabilities;
@@ -238,20 +314,18 @@ public:
         Utils::optional<QList<MarkupKind>> contentFormat() const;
         void setContentFormat(const QList<MarkupKind> &contentFormat);
         void clearContentFormat() { remove(contentFormatKey); }
-
-        bool isValid(QStringList *error) const override;
     };
 
     Utils::optional<HoverCapabilities> hover() const { return optionalValue<HoverCapabilities>(hoverKey); }
     void setHover(const HoverCapabilities &hover) { insert(hoverKey, hover); }
     void clearHover() { remove(hoverKey); }
 
-    class SignatureHelpCapabilities : public DynamicRegistrationCapabilities
+    class LANGUAGESERVERPROTOCOL_EXPORT SignatureHelpCapabilities : public DynamicRegistrationCapabilities
     {
     public:
         using DynamicRegistrationCapabilities::DynamicRegistrationCapabilities;
 
-        class SignatureInformationCapabilities : public JsonObject
+        class LANGUAGESERVERPROTOCOL_EXPORT SignatureInformationCapabilities : public JsonObject
         {
         public:
             using JsonObject::JsonObject;
@@ -262,19 +336,14 @@ public:
             Utils::optional<QList<MarkupKind>> documentationFormat() const;
             void setDocumentationFormat(const QList<MarkupKind> &documentationFormat);
             void clearDocumentationFormat() { remove(documentationFormatKey); }
-
-            bool isValid(QStringList *error) const override
-            { return checkOptionalArray<int>(error, documentationFormatKey); }
         };
 
         // The client supports the following `SignatureInformation` specific properties.
-        Utils::optional<SignatureHelpCapabilities> signatureInformation() const
-        { return optionalValue<SignatureHelpCapabilities>(signatureInformationKey); }
-        void setSignatureInformation(const SignatureHelpCapabilities &signatureInformation)
+        Utils::optional<SignatureInformationCapabilities> signatureInformation() const
+        { return optionalValue<SignatureInformationCapabilities>(signatureInformationKey); }
+        void setSignatureInformation(const SignatureInformationCapabilities &signatureInformation)
         { insert(signatureInformationKey, signatureInformation); }
         void clearSignatureInformation() { remove(signatureInformationKey); }
-
-        bool isValid(QStringList *error) const override;
     };
 
     // Capabilities specific to the `textDocument/signatureHelp`
@@ -355,17 +424,17 @@ public:
     { insert(implementationKey, implementation); }
     void clearImplementation() { remove(implementationKey); }
 
-    class CodeActionCapabilities : public DynamicRegistrationCapabilities
+    class LANGUAGESERVERPROTOCOL_EXPORT CodeActionCapabilities : public DynamicRegistrationCapabilities
     {
     public:
         using DynamicRegistrationCapabilities::DynamicRegistrationCapabilities;
 
-        class CodeActionLiteralSupport : public JsonObject
+        class LANGUAGESERVERPROTOCOL_EXPORT CodeActionLiteralSupport : public JsonObject
         {
         public:
             using JsonObject::JsonObject;
 
-            class CodeActionKind : public JsonObject
+            class LANGUAGESERVERPROTOCOL_EXPORT CodeActionKind : public JsonObject
             {
             public:
                 using JsonObject::JsonObject;
@@ -376,8 +445,7 @@ public:
                 void setValueSet(const QList<QString> &valueSet)
                 { insertArray(valueSetKey, valueSet); }
 
-                bool isValid(QStringList *errorHierarchy) const override
-                { return checkArray<QString>(errorHierarchy, valueSetKey); }
+                bool isValid() const override { return contains(valueSetKey); }
             };
 
             CodeActionKind codeActionKind() const
@@ -385,8 +453,7 @@ public:
             void setCodeActionKind(const CodeActionKind &codeActionKind)
             { insert(codeActionKindKey, codeActionKind); }
 
-            bool isValid(QStringList *errorHierarchy) const override
-            { return check<CodeActionKind>(errorHierarchy, codeActionKindKey); }
+            bool isValid() const override { return contains(codeActionKindKey); }
         };
 
         Utils::optional<CodeActionLiteralSupport> codeActionLiteralSupport() const
@@ -394,8 +461,6 @@ public:
         void setCodeActionLiteralSupport(const CodeActionLiteralSupport &codeActionLiteralSupport)
         { insert(codeActionLiteralSupportKey, codeActionLiteralSupport); }
         void clearCodeActionLiteralSupport() { remove(codeActionLiteralSupportKey); }
-
-        bool isValid(QStringList *errorHierarchy) const override;
     };
 
     // Whether code action supports dynamic registration.
@@ -430,14 +495,32 @@ public:
     { insert(colorProviderKey, colorProvider); }
     void clearColorProvider() { remove(colorProviderKey); }
 
+    class LANGUAGESERVERPROTOCOL_EXPORT RenameClientCapabilities : public DynamicRegistrationCapabilities
+    {
+    public:
+        using DynamicRegistrationCapabilities::DynamicRegistrationCapabilities;
+        /**
+         * Client supports testing for validity of rename operations
+         * before execution.
+         *
+         * @since version 3.12.0
+         */
+
+        Utils::optional<bool> prepareSupport() const { return optionalValue<bool>(prepareSupportKey); }
+        void setPrepareSupport(bool prepareSupport) { insert(prepareSupportKey, prepareSupport); }
+        void clearPrepareSupport() { remove(prepareSupportKey); }
+    };
+
     // Whether rename supports dynamic registration.
-    Utils::optional<DynamicRegistrationCapabilities> rename() const
-    { return optionalValue<DynamicRegistrationCapabilities>(renameKey); }
-    void setRename(const DynamicRegistrationCapabilities &rename)
+    Utils::optional<RenameClientCapabilities> rename() const
+    { return optionalValue<RenameClientCapabilities>(renameKey); }
+    void setRename(const RenameClientCapabilities &rename)
     { insert(renameKey, rename); }
     void clearRename() { remove(renameKey); }
 
-    bool isValid(QStringList *error) const override;
+    Utils::optional<SemanticTokensClientCapabilities> semanticTokens() const;
+    void setSemanticTokens(const SemanticTokensClientCapabilities &semanticTokens);
+    void clearSemanticTokens() { remove(semanticTokensKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT WorkspaceClientCapabilities : public JsonObject
@@ -454,7 +537,7 @@ public:
     void setApplyEdit(bool applyEdit) { insert(applyEditKey, applyEdit); }
     void clearApplyEdit() { remove(applyEditKey); }
 
-    class WorkspaceEditCapabilities : public JsonObject
+    class LANGUAGESERVERPROTOCOL_EXPORT WorkspaceEditCapabilities : public JsonObject
     {
     public:
         using JsonObject::JsonObject;
@@ -465,9 +548,6 @@ public:
         void setDocumentChanges(bool documentChanges)
         { insert(documentChangesKey, documentChanges); }
         void clearDocumentChanges() { remove(documentChangesKey); }
-
-        bool isValid(QStringList *error) const override
-        { return checkOptional<bool>(error, documentChangesKey); }
     };
 
     // Capabilities specific to `WorkspaceEdit`s
@@ -515,14 +595,32 @@ public:
     Utils::optional<bool> configuration() const { return optionalValue<bool>(configurationKey); }
     void setConfiguration(bool configuration) { insert(configurationKey, configuration); }
     void clearConfiguration() { remove(configurationKey); }
+};
 
-    bool isValid(QStringList *error) const override;
+class WindowClientClientCapabilities : public JsonObject
+{
+public:
+    using JsonObject::JsonObject;
+
+    /**
+      * Whether client supports handling progress notifications.
+      * If set, servers are allowed to report in `workDoneProgress` property
+      * in the request specific server capabilities.
+      *
+      */
+    Utils::optional<bool> workDoneProgress() const
+    { return optionalValue<bool>(workDoneProgressKey); }
+    void setWorkDoneProgress(bool workDoneProgress)
+    { insert(workDoneProgressKey, workDoneProgress); }
+    void clearWorkDoneProgress() { remove(workDoneProgressKey); }
+
+private:
+    constexpr static const char workDoneProgressKey[] = "workDoneProgress";
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT ClientCapabilities : public JsonObject
 {
 public:
-    ClientCapabilities();
     using JsonObject::JsonObject;
 
     // Workspace specific client capabilities.
@@ -539,12 +637,17 @@ public:
     { insert(textDocumentKey, textDocument); }
     void clearTextDocument() { remove(textDocumentKey); }
 
+    // Window specific client capabilities.
+    Utils::optional<WindowClientClientCapabilities> window() const
+    { return optionalValue<WindowClientClientCapabilities>(windowKey); }
+    void setWindow(const WindowClientClientCapabilities &window)
+    { insert(windowKey, window); }
+    void clearWindow() { remove(windowKey); }
+
     // Experimental client capabilities.
     QJsonValue experimental() const { return value(experimentalKey); }
     void setExperimental(const QJsonValue &experimental) { insert(experimentalKey, experimental); }
     void clearExperimental() { remove(experimentalKey); }
-
-    bool isValid(QStringList *error) const override;
 };
 
-}
+} // namespace LanguageServerProtocol
