@@ -40,6 +40,10 @@
 #include <utils/runextensions.h>
 #include <utils/stringutils.h>
 
+#ifdef WITH_TESTS
+#include <extensionsystem/pluginmanager.h>
+#endif
+
 #include <QDir>
 #include <QDirIterator>
 #include <QFile>
@@ -77,6 +81,7 @@ QMLJS_EXPORT Q_LOGGING_CATEGORY(qmljsLog, "qtc.qmljs.common", QtWarningMsg)
 */
 
 static ModelManagerInterface *g_instance = nullptr;
+static QMutex g_instanceMutex;
 static const char *qtQuickUISuffix = "ui.qml";
 
 static void maybeAddPath(ViewerContext &context, const QString &path)
@@ -106,6 +111,7 @@ ModelManagerInterface::ModelManagerInterface(QObject *parent)
       m_defaultImportPaths(environmentImportPaths()),
       m_pluginDumper(new PluginDumper(this))
 {
+    m_futureSynchronizer.setCancelOnWait(false);
     m_indexerDisabled = qEnvironmentVariableIsSet("QTC_NO_CODE_INDEXER");
 
     m_updateCppQmlTypesTimer = new QTimer(this);
@@ -133,16 +139,27 @@ ModelManagerInterface::ModelManagerInterface(QObject *parent)
 
     updateImportPaths();
 
+    QMutexLocker locker(&g_instanceMutex);
     Q_ASSERT(! g_instance);
     g_instance = this;
 }
 
 ModelManagerInterface::~ModelManagerInterface()
 {
+    Q_ASSERT(g_instance == this);
     m_cppQmlTypesUpdater.cancel();
     m_cppQmlTypesUpdater.waitForFinished();
-    Q_ASSERT(g_instance == this);
-    g_instance = nullptr;
+
+    while (true) {
+        joinAllThreads(true);
+        // Keep these 2 mutexes in the same order as inside instanceForFuture()
+        QMutexLocker instanceLocker(&g_instanceMutex);
+        QMutexLocker futureLocker(&m_futuresMutex);
+        if (m_futureSynchronizer.isEmpty()) {
+            g_instance = nullptr;
+            return;
+        }
+    }
 }
 
 static QHash<QString, Dialect> defaultLanguageMapping()
@@ -197,6 +214,15 @@ ModelManagerInterface *ModelManagerInterface::instance()
     return g_instance;
 }
 
+// If the returned instance is not null, it's guaranteed that it will be valid at least as long
+// as the passed QFuture object isn't finished.
+ModelManagerInterface *ModelManagerInterface::instanceForFuture(const QFuture<void> &future)
+{
+    QMutexLocker locker(&g_instanceMutex);
+    if (g_instance)
+        g_instance->addFuture(future);
+    return g_instance;
+}
 void ModelManagerInterface::writeWarning(const QString &msg)
 {
     if (ModelManagerInterface *i = instance())
@@ -307,20 +333,6 @@ void ModelManagerInterface::updateSourceFiles(const QStringList &files,
     refreshSourceFiles(files, emitDocumentOnDiskChanged);
 }
 
-void ModelManagerInterface::cleanupFutures()
-{
-    QMutexLocker lock(&m_futuresMutex);
-    const int maxFutures = 10;
-    if (m_futures.size() > maxFutures) {
-        const QList<QFuture<void>> futures = m_futures;
-        m_futures.clear();
-        for (const QFuture<void> &future : futures) {
-            if (!(future.isFinished() || future.isCanceled()))
-                m_futures.append(future);
-        }
-    }
-}
-
 QFuture<void> ModelManagerInterface::refreshSourceFiles(const QStringList &sourceFiles,
                                                         bool emitDocumentOnDiskChanged)
 {
@@ -355,9 +367,9 @@ QFuture<void> ModelManagerInterface::refreshSourceFiles(const QStringList &sourc
 
 void ModelManagerInterface::fileChangedOnDisk(const QString &path)
 {
-    Utils::runAsync(&ModelManagerInterface::parse,
+    addFuture(Utils::runAsync(&ModelManagerInterface::parse,
                     workingCopyInternal(), QStringList(path),
-                    this, Dialect(Dialect::AnyLanguage), true);
+                    this, Dialect(Dialect::AnyLanguage), true));
 }
 
 void ModelManagerInterface::removeFiles(const QStringList &files)
@@ -615,6 +627,9 @@ ModelManagerInterface::ProjectInfo ModelManagerInterface::projectInfoForPath(
         res.applicationDirectories.append(pInfo.applicationDirectories);
         for (const auto &importPath : pInfo.importPaths)
             res.importPaths.maybeInsert(importPath);
+        auto end = pInfo.moduleMappings.cend();
+        for (auto it = pInfo.moduleMappings.cbegin(); it != end; ++it)
+            res.moduleMappings.insert(it.key(), it.value());
     }
     res.applicationDirectories = Utils::filteredUnique(res.applicationDirectories);
     return res;
@@ -641,15 +656,14 @@ QList<ModelManagerInterface::ProjectInfo> ModelManagerInterface::allProjectInfos
         if (!info.project.isNull())
             infos.append(info);
     }
+    if (infos.isEmpty()) {
+        QMutexLocker locker(&m_mutex);
+        return { m_defaultProjectInfo };
+    }
     std::sort(infos.begin(), infos.end(), &pInfoLessThanImports);
     return infos;
 }
 
-bool ModelManagerInterface::isIdle() const
-{
-    return m_futures.isEmpty();
-}
-
 void ModelManagerInterface::emitDocumentChangedOnDisk(Document::Ptr doc)
 {
     emit documentChangedOnDisk(std::move(doc));
@@ -797,7 +811,7 @@ static bool findNewQmlApplicationInPath(const QString &path,
 
     qmltypesFile = it.next();
 
-    LibraryInfo libraryInfo = LibraryInfo(QmlDirParser::TypeInfo(qmltypesFile));
+    LibraryInfo libraryInfo = LibraryInfo(qmltypesFile);
     const QString libraryPath = dir.absolutePath();
     newLibraries->insert(libraryPath);
     modelManager->updateLibraryInfo(path, libraryInfo);
@@ -953,16 +967,31 @@ void ModelManagerInterface::parseLoop(QSet<QString> &scannedPaths,
         doc->setSource(contents);
         doc->parse();
 
-        // update snapshot. requires synchronization, but significantly reduces amount of file
-        // system queries for library imports because queries are cached in libraryInfo
-        const Snapshot snapshot = modelManager->snapshot();
-
+#ifdef WITH_TESTS
+        if (ExtensionSystem::PluginManager::isScenarioRunning("TestModelManagerInterface")) {
+            ExtensionSystem::PluginManager::waitForScenarioFullyInitialized();
+            if (ExtensionSystem::PluginManager::finishScenario()) {
+                qDebug() << "Point 1: Shutdown triggered";
+                QThread::currentThread()->sleep(2);
+                qDebug() << "Point 3: If Point 2 was already reached, expect a crash now";
+            }
+        }
+#endif
         // get list of referenced files not yet in snapshot or in directories already scanned
         QStringList importedFiles;
-        findNewImplicitImports(doc, snapshot, &importedFiles, &scannedPaths);
-        findNewFileImports(doc, snapshot, &importedFiles, &scannedPaths);
-        findNewLibraryImports(doc, snapshot, modelManager, &importedFiles, &scannedPaths,
-                              &newLibraries);
+
+        // update snapshot. requires synchronization, but significantly reduces amount of file
+        // system queries for library imports because queries are cached in libraryInfo
+        {
+            // Make sure the snapshot is destroyed before updateDocument, so that we don't trigger
+            // the copy-on-write mechanism on its internals.
+            const Snapshot snapshot = modelManager->snapshot();
+
+            findNewImplicitImports(doc, snapshot, &importedFiles, &scannedPaths);
+            findNewFileImports(doc, snapshot, &importedFiles, &scannedPaths);
+            findNewLibraryImports(doc, snapshot, modelManager, &importedFiles, &scannedPaths,
+                                  &newLibraries);
+        }
 
         // add new files to parse list
         for (const QString &file : qAsConst(importedFiles)) {
@@ -1200,6 +1229,10 @@ void ModelManagerInterface::updateImportPaths()
             allImportPaths.maybeInsert(Utils::FilePath::fromString(pathAtt), Dialect::QmlQtQuick2);
     }
 
+    for (const auto &importPath : defaultProjectInfo().importPaths) {
+        allImportPaths.maybeInsert(importPath);
+    }
+
     for (const QString &path : qAsConst(m_defaultImportPaths))
         allImportPaths.maybeInsert(Utils::FilePath::fromString(path), Dialect::Qml);
     allImportPaths.compact();
@@ -1251,8 +1284,9 @@ void ModelManagerInterface::maybeQueueCppQmlTypeUpdate(const CPlusPlus::Document
     if (!scan)
         doc->releaseSourceAndAST();
 
-    // delegate actual queuing to the gui thread
-    QMetaObject::invokeMethod(this, [=] { queueCppQmlTypeUpdate(doc, scan); });
+    QMutexLocker locker(&g_instanceMutex);
+    if (g_instance) // delegate actual queuing to the gui thread
+        QMetaObject::invokeMethod(g_instance, [=] { queueCppQmlTypeUpdate(doc, scan); });
 }
 
 void ModelManagerInterface::queueCppQmlTypeUpdate(const CPlusPlus::Document::Ptr &doc, bool scan)
@@ -1432,6 +1466,8 @@ ViewerContext ModelManagerInterface::getVContext(const ViewerContext &vCtx,
         info.qtQmlPath = defaultInfo.qtQmlPath;
         info.qtVersionString = defaultInfo.qtVersionString;
     }
+    if (info.qtQmlPath.isEmpty() && info.importPaths.size() == 0)
+        info.importPaths = defaultInfo.importPaths;
     info.applicationDirectories = Utils::filteredUnique(info.applicationDirectories
                                                         + defaultInfo.applicationDirectories);
     switch (res.flags) {
@@ -1558,41 +1594,38 @@ void ModelManagerInterface::setDefaultVContext(const ViewerContext &vContext)
     m_defaultVContexts[vContext.language] = vContext;
 }
 
+void ModelManagerInterface::joinAllThreads(bool cancelOnWait)
+{
+    while (true) {
+        FutureSynchronizer futureSynchronizer;
+        {
+            QMutexLocker locker(&m_futuresMutex);
+            futureSynchronizer = m_futureSynchronizer;
+            m_futureSynchronizer.clearFutures();
+        }
+        futureSynchronizer.setCancelOnWait(cancelOnWait);
+        if (futureSynchronizer.isEmpty())
+            return;
+    }
+}
+
 void ModelManagerInterface::test_joinAllThreads()
 {
-    // Loop since futures can spawn more futures as they finish.
     while (true) {
-        QFuture<void> f;
-        // get one future
-        {
-            QMutexLocker lock(&m_futuresMutex);
-            for (QFuture<void> &future : m_futures) {
-                if (!future.isFinished() && !future.isCanceled()) {
-                    f = future;
-                    break;
-                }
-            }
-        }
-        if (!f.isFinished() && !f.isCanceled()) {
-            f.waitForFinished();
-
-            // Some futures trigger more futures from connected signals
-            // and in tests, we care about finishing all of these too.
-            QEventLoop().processEvents();
-        } else {
-            break;
-        }
+        joinAllThreads();
+        // In order to process all onFinished handlers of finished futures
+        QCoreApplication::processEvents();
+        QMutexLocker lock(&m_futuresMutex);
+        // If handlers created new futures, repeat the loop
+        if (m_futureSynchronizer.isEmpty())
+            return;
     }
-    m_futures.clear();
 }
 
 void ModelManagerInterface::addFuture(const QFuture<void> &future)
 {
-    {
-        QMutexLocker lock(&m_futuresMutex);
-        m_futures.append(future);
-    }
-    cleanupFutures();
+    QMutexLocker lock(&m_futuresMutex);
+    m_futureSynchronizer.addFuture(future);
 }
 
 Document::Ptr ModelManagerInterface::ensuredGetDocumentForPath(const QString &filePath)
