@@ -39,6 +39,15 @@
 
 #pragma once
 
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is mostly copied from Qt code and should not be touched
+// unless really needed.
+//
+
+
 //
 //  W A R N I N G
 //  -------------
@@ -53,7 +62,9 @@
 #include <QtCore/qhash.h>
 #include <QtCore/qmutex.h>
 QT_BEGIN_NAMESPACE
+class QFileInfo;
 class QIODevice;
+class QUrl;
 QT_END_NAMESPACE
 
 #include "mimetype.h"
@@ -61,9 +72,6 @@ QT_END_NAMESPACE
 #include "mimeglobpattern_p.h"
 
 namespace Utils {
-
-class MimeDatabase;
-
 namespace Internal {
 
 class MimeProviderBase;
@@ -91,11 +99,58 @@ public:
     MimeType mimeTypeForName(const QString &nameOrAlias);
     MimeType mimeTypeForFileNameAndData(const QString &fileName, QIODevice *device, int *priorityPtr);
     MimeType findByData(const QByteArray &data, int *priorityPtr);
-    QStringList mimeTypeForFileName(const QString &fileName, QString *foundSuffix = 0);
+    QStringList mimeTypeForFileName(const QString &fileName, QString *foundSuffix = nullptr);
 
     mutable MimeProviderBase *m_provider;
     const QString m_defaultMimeType;
     QMutex mutex;
+
+    int m_startupPhase = 0;
+};
+
+class MimeDatabase
+{
+    Q_DISABLE_COPY(MimeDatabase)
+
+public:
+    MimeDatabase();
+    ~MimeDatabase();
+
+    MimeType mimeTypeForName(const QString &nameOrAlias) const;
+
+    enum MatchMode {
+        MatchDefault = 0x0,
+        MatchExtension = 0x1,
+        MatchContent = 0x2
+    };
+
+    MimeType mimeTypeForFile(const QString &fileName, MatchMode mode = MatchDefault) const;
+    MimeType mimeTypeForFile(const QFileInfo &fileInfo, MatchMode mode = MatchDefault) const;
+    QList<MimeType> mimeTypesForFileName(const QString &fileName) const;
+
+    MimeType mimeTypeForData(const QByteArray &data) const;
+    MimeType mimeTypeForData(QIODevice *device) const;
+
+    MimeType mimeTypeForUrl(const QUrl &url) const;
+    MimeType mimeTypeForFileNameAndData(const QString &fileName, QIODevice *device) const;
+    MimeType mimeTypeForFileNameAndData(const QString &fileName, const QByteArray &data) const;
+
+    QString suffixForFileName(const QString &fileName) const;
+
+    QList<MimeType> allMimeTypes() const;
+
+    // For debugging purposes.
+    enum StartupPhase {
+        BeforeInitialize,
+        PluginsLoading,
+        PluginsInitializing, // Register up to here.
+        PluginsDelayedInitializing, // Use from here on.
+        UpAndRunning
+    };
+    static void setStartupPhase(StartupPhase);
+
+private:
+    Internal::MimeDatabasePrivate *d;
 };
 
 } // Internal
