@@ -31,6 +31,7 @@
 #include <QObject>
 #include <QVersionNumber>
 
+#include <qtsupport/baseqtversion.h>
 #include <projectexplorer/abi.h>
 
 QT_BEGIN_NAMESPACE
@@ -72,12 +73,6 @@ class ANDROID_EXPORT AndroidManager : public QObject
 public:
     static QString packageName(const ProjectExplorer::Target *target);
     static QString packageName(const Utils::FilePath &manifestFile);
-    static bool packageInstalled(const QString &deviceSerial, const QString &packageName);
-    static int packageVersionCode(const QString &deviceSerial, const QString &packageName);
-    static void apkInfo(const Utils::FilePath &apkPath,
-                        QString *packageName = nullptr,
-                        int *version = nullptr,
-                        QString *activityPath = nullptr);
     static QString activityName(const ProjectExplorer::Target *target);
 
     static QString deviceSerialNumber(const ProjectExplorer::Target *target);
@@ -93,21 +88,25 @@ public:
 
     static int minimumSDK(const ProjectExplorer::Target *target);
     static int minimumSDK(const ProjectExplorer::Kit *kit);
+    static int defaultMinimumSDK(const QtSupport::BaseQtVersion *qtVersion);
 
     static QStringList applicationAbis(const ProjectExplorer::Target *target);
     static QString archTriplet(const QString &abi);
 
+    static bool isQt5CmakeProject(const ProjectExplorer::Target *target);
+
+    // TODO: remove this on 6.0 branch, kept here for binary compatibility for 5.0 release.
     static Utils::FilePath dirPath(const ProjectExplorer::Target *target);
+    static Utils::FilePath androidBuildDirectory(const ProjectExplorer::Target *target);
+    static Utils::FilePath buildDirectory(const ProjectExplorer::Target *target);
     static Utils::FilePath manifestPath(const ProjectExplorer::Target *target);
     static void setManifestPath(ProjectExplorer::Target *target, const Utils::FilePath &path);
     static Utils::FilePath manifestSourcePath(const ProjectExplorer::Target *target);
-    static Utils::FilePath defaultPropertiesPath(const ProjectExplorer::Target *target);
     static Utils::FilePath apkPath(const ProjectExplorer::Target *target);
     static bool matchedAbis(const QStringList &deviceAbis, const QStringList &appAbis);
     static QString devicePreferredAbi(const QStringList &deviceAbis, const QStringList &appAbis);
     static ProjectExplorer::Abi androidAbi2Abi(const QString &androidAbi);
 
-    static QPair<int, int> apiLevelRange();
     static QString androidNameForApiLevel(int x);
 
     static void installQASIPackage(ProjectExplorer::Target *target, const QString &packagePath);
@@ -118,13 +117,11 @@ public:
     static bool checkCertificateExists(const QString &keystorePath, const QString &keystorePasswd,
                                        const QString &alias);
     static bool updateGradleProperties(ProjectExplorer::Target *target, const QString &buildKey);
-    static int findApiLevel(const Utils::FilePath &platformPath);
 
     static QProcess *runAdbCommandDetached(const QStringList &args, QString *err = nullptr,
                                            bool deleteOnFinish = false);
     static SdkToolResult runAdbCommand(const QStringList &args, const QByteArray &writeData = {},
                                        int timeoutS = 30);
-    static SdkToolResult runAaptCommand(const QStringList &args, int timeoutS = 30);
 
     static QJsonObject deploymentSettings(const ProjectExplorer::Target *target);
     static bool isQtCreatorGenerated(const Utils::FilePath &deploymentFile);
