@@ -87,11 +87,19 @@ static bool includesQtQuickTest(const CPlusPlus::Document::Ptr &doc,
                 return true;
         }
     }
+
+    for (const QString &prefix : expectedHeaderPrefixes) {
+        if (CppParser::precompiledHeaderContains(snapshot,
+                                                 Utils::FilePath::fromString(doc->fileName()),
+                                                 QString("%1/quicktest.h").arg(prefix))) {
+            return true;
+        }
+    }
     return false;
 }
 
 static QString quickTestSrcDir(const CppTools::CppModelManager *cppMM,
-                               const QString &fileName)
+                               const Utils::FilePath &fileName)
 {
     const QList<CppTools::ProjectPart::Ptr> parts = cppMM->projectPart(fileName);
     if (parts.size() > 0) {
@@ -115,6 +123,7 @@ static QString quickTestSrcDir(const CppTools::CppModelManager *cppMM,
 QString QuickTestParser::quickTestName(const CPlusPlus::Document::Ptr &doc) const
 {
     const QList<CPlusPlus::Document::MacroUse> macros = doc->macroUses();
+    const Utils::FilePath filePath = Utils::FilePath::fromString(doc->fileName());
 
     for (const CPlusPlus::Document::MacroUse &macro : macros) {
         if (!macro.isFunctionLike())
@@ -122,22 +131,45 @@ QString QuickTestParser::quickTestName(const CPlusPlus::Document::Ptr &doc) cons
         const QByteArray name = macro.macro().name();
         if (QuickTestUtils::isQuickTestMacro(name)) {
             CPlusPlus::Document::Block arg = macro.arguments().at(0);
-            return QLatin1String(getFileContent(doc->fileName())
+            return QLatin1String(getFileContent(filePath)
                                  .mid(int(arg.bytesBegin()), int(arg.bytesEnd() - arg.bytesBegin())));
         }
     }
 
+
+    const QByteArray &fileContent = getFileContent(filePath);
     // check for using quick_test_main() directly
-    const QString fileName = doc->fileName();
-    const QByteArray &fileContent = getFileContent(fileName);
-    CPlusPlus::Document::Ptr document = m_cppSnapshot.preprocessedDocument(fileContent, fileName);
+    CPlusPlus::Document::Ptr document = m_cppSnapshot.preprocessedDocument(fileContent, filePath);
     if (document.isNull())
         return QString();
     document->check();
     CPlusPlus::AST *ast = document->translationUnit()->ast();
     QuickTestAstVisitor astVisitor(document, m_cppSnapshot);
     astVisitor.accept(ast);
-    return astVisitor.testBaseName();
+    if (!astVisitor.testBaseName().isEmpty())
+        return astVisitor.testBaseName();
+
+    // check for precompiled headers
+    static QStringList expectedHeaderPrefixes
+            = Utils::HostOsInfo::isMacHost()
+            ? QStringList({"QtQuickTest.framework/Headers", "QtQuickTest"})
+            : QStringList({"QtQuickTest"});
+    bool pchIncludes = false;
+    for (const QString &prefix : expectedHeaderPrefixes) {
+        if (CppParser::precompiledHeaderContains(m_cppSnapshot, filePath,
+                                                 QString("%1/quicktest.h").arg(prefix))) {
+            pchIncludes = true;
+            break;
+        }
+    }
+
+    if (pchIncludes) {
+        const QRegularExpression regex("\\bQUICK_TEST_(MAIN|OPENGL_MAIN|MAIN_WITH_SETUP)");
+        const QRegularExpressionMatch match = regex.match(QString::fromUtf8(fileContent));
+        if (match.hasMatch())
+            return match.captured(); // we do not care for the name, just return something non-empty
+    }
+    return {};
 }
 
 QList<Document::Ptr> QuickTestParser::scanDirectoryForQuickTestQmlFiles(const QString &srcDir)
@@ -182,7 +214,7 @@ QList<Document::Ptr> QuickTestParser::scanDirectoryForQuickTestQmlFiles(const QS
 static bool checkQmlDocumentForQuickTestCode(QFutureInterface<TestParseResultPtr> futureInterface,
                                              const Document::Ptr &qmlJSDoc,
                                              ITestFramework *framework,
-                                             const QString &proFile = QString())
+                                             const Utils::FilePath &proFile = Utils::FilePath())
 {
     if (qmlJSDoc.isNull())
         return false;
@@ -203,7 +235,7 @@ static bool checkQmlDocumentForQuickTestCode(QFutureInterface<TestParseResultPtr
         parseResult->proFile = proFile;
         parseResult->itemType = TestTreeItem::TestCase;
         if (!testCaseName.isEmpty()) {
-            parseResult->fileName = testCase.m_locationAndType.m_name;
+            parseResult->fileName = testCase.m_locationAndType.m_filePath;
             parseResult->name = testCaseName;
             parseResult->line = testCase.m_locationAndType.m_line;
             parseResult->column = testCase.m_locationAndType.m_column;
@@ -211,12 +243,12 @@ static bool checkQmlDocumentForQuickTestCode(QFutureInterface<TestParseResultPtr
 
         for (const auto &function : testCase.m_functions) {
             QuickTestParseResult *funcResult = new QuickTestParseResult(framework);
-            funcResult->name = function.m_functionName;
-            funcResult->displayName = function.m_functionName;
-            funcResult->itemType = function.m_locationAndType.m_type;
-            funcResult->fileName = function.m_locationAndType.m_name;
-            funcResult->line = function.m_locationAndType.m_line;
-            funcResult->column = function.m_locationAndType.m_column;
+            funcResult->name = function.m_name;
+            funcResult->displayName = function.m_name;
+            funcResult->itemType = function.m_type;
+            funcResult->fileName = function.m_filePath;
+            funcResult->line = function.m_line;
+            funcResult->column = function.m_column;
             funcResult->proFile = proFile;
 
             parseResult->children.append(funcResult);
@@ -235,11 +267,11 @@ bool QuickTestParser::handleQtQuickTest(QFutureInterface<TestParseResultPtr> fut
     if (quickTestName(document).isEmpty())
         return false;
 
-    const QString cppFileName = document->fileName();
-    QList<CppTools::ProjectPart::Ptr> ppList = modelManager->projectPart(cppFileName);
+    QList<CppTools::ProjectPart::Ptr> ppList = modelManager->projectPart(document->fileName());
     if (ppList.isEmpty()) // happens if shutting down while parsing
         return false;
-    const QString &proFile = ppList.at(0)->projectFile;
+    const Utils::FilePath cppFileName = Utils::FilePath::fromString(document->fileName());
+    const Utils::FilePath proFile = Utils::FilePath::fromString(ppList.at(0)->projectFile);
     m_mainCppFiles.insert(cppFileName, proFile);
     const QString srcDir = quickTestSrcDir(modelManager, cppFileName);
     if (srcDir.isEmpty())
@@ -280,6 +312,7 @@ void QuickTestParser::handleDirectoryChanged(const QString &directory)
             return filesAndDates.value(file) != watched.value(file);
         });
         if (timestampChanged) {
+            m_watchedFiles[directory] = filesAndDates;
             PathsAndLanguages paths;
             paths.maybeInsert(Utils::FilePath::fromString(directory), Dialect::Qml);
             QFutureInterface<void> future;
@@ -315,14 +348,14 @@ QuickTestParser::QuickTestParser(ITestFramework *framework)
             this, &QuickTestParser::handleDirectoryChanged);
 }
 
-void QuickTestParser::init(const QStringList &filesToParse, bool fullParse)
+void QuickTestParser::init(const Utils::FilePaths &filesToParse, bool fullParse)
 {
     m_qmlSnapshot = QmlJSTools::Internal::ModelManager::instance()->snapshot();
     if (!fullParse) {
         // in a full parse we get the correct entry points by the respective main
         m_proFilesForQmlFiles = QuickTestUtils::proFilesForQmlFiles(framework(), filesToParse);
         // get rid of cached main cpp files that are going to get processed anyhow
-        for (const QString &file : filesToParse) {
+        for (const Utils::FilePath &file : filesToParse) {
             if (m_mainCppFiles.contains(file)) {
                 m_mainCppFiles.remove(file);
                 if (m_mainCppFiles.isEmpty())
@@ -344,13 +377,13 @@ void QuickTestParser::release()
 }
 
 bool QuickTestParser::processDocument(QFutureInterface<TestParseResultPtr> futureInterface,
-                                      const QString &fileName)
+                                      const Utils::FilePath &fileName)
 {
     if (fileName.endsWith(".qml")) {
-        const QString &proFile = m_proFilesForQmlFiles.value(fileName);
+        const Utils::FilePath &proFile = m_proFilesForQmlFiles.value(fileName);
         if (proFile.isEmpty())
             return false;
-        Document::Ptr qmlJSDoc = m_qmlSnapshot.document(fileName);
+        Document::Ptr qmlJSDoc = m_qmlSnapshot.document(fileName.toString());
         return checkQmlDocumentForQuickTestCode(futureInterface, qmlJSDoc, framework(), proFile);
     }
     if (!m_cppSnapshot.contains(fileName) || !selectedForBuilding(fileName))
@@ -361,9 +394,9 @@ bool QuickTestParser::processDocument(QFutureInterface<TestParseResultPtr> futur
     return handleQtQuickTest(futureInterface, document, framework());
 }
 
-QString QuickTestParser::projectFileForMainCppFile(const QString &fileName) const
+Utils::FilePath QuickTestParser::projectFileForMainCppFile(const Utils::FilePath &fileName) const
 {
-    return m_mainCppFiles.contains(fileName) ? m_mainCppFiles.value(fileName) : QString();
+    return m_mainCppFiles.contains(fileName) ? m_mainCppFiles.value(fileName) : Utils::FilePath();
 }
 
 } // namespace Internal
