@@ -23,17 +23,18 @@
 **
 ****************************************************************************/
 
-#include "autotestplugin.h"
-#include "autotesticons.h"
 #include "testresultspane.h"
-#include "testresultmodel.h"
+
+#include "autotesticons.h"
+#include "autotestplugin.h"
+#include "itestframework.h"
+#include "testeditormark.h"
 #include "testresultdelegate.h"
+#include "testresultmodel.h"
+#include "testresultmodel.h"
 #include "testrunner.h"
 #include "testsettings.h"
 #include "testtreemodel.h"
-#include "testcodeparser.h"
-#include "testeditormark.h"
-#include "testoutputreader.h"
 
 #include <aggregation/aggregate.h>
 #include <coreplugin/actionmanager/actionmanager.h>
@@ -45,7 +46,12 @@
 #include <coreplugin/icore.h>
 #include <projectexplorer/buildmanager.h>
 #include <projectexplorer/projectexplorer.h>
+#include <texteditor/fontsettings.h>
 #include <texteditor/texteditor.h>
+#include <texteditor/texteditorsettings.h>
+#include <utils/proxyaction.h>
+#include <utils/qtcassert.h>
+#include <utils/stylehelper.h>
 #include <utils/theme/theme.h>
 #include <utils/utilsicons.h>
 
@@ -61,6 +67,8 @@
 #include <QToolButton>
 #include <QVBoxLayout>
 
+using namespace Core;
+
 namespace Autotest {
 namespace Internal {
 
@@ -68,6 +76,7 @@ ResultsTreeView::ResultsTreeView(QWidget *parent)
     : Utils::TreeView(parent)
 {
     setAttribute(Qt::WA_MacShowFocusRect, false);
+    setFrameStyle(NoFrame);
 }
 
 void ResultsTreeView::keyPressEvent(QKeyEvent *event)
@@ -80,14 +89,14 @@ void ResultsTreeView::keyPressEvent(QKeyEvent *event)
 }
 
 TestResultsPane::TestResultsPane(QObject *parent) :
-    Core::IOutputPane(parent),
-    m_context(new Core::IContext(this))
+    IOutputPane(parent),
+    m_context(new IContext(this))
 {
     m_outputWidget = new QStackedWidget;
     QWidget *visualOutputWidget = new QWidget;
     m_outputWidget->addWidget(visualOutputWidget);
     QVBoxLayout *outputLayout = new QVBoxLayout;
-    outputLayout->setMargin(0);
+    outputLayout->setContentsMargins(0, 0, 0, 0);
     outputLayout->setSpacing(0);
     visualOutputWidget->setLayout(outputLayout);
 
@@ -100,7 +109,7 @@ TestResultsPane::TestResultsPane(QObject *parent) :
     m_summaryWidget->setPalette(pal);
     m_summaryWidget->setAutoFillBackground(true);
     QHBoxLayout *layout = new QHBoxLayout;
-    layout->setMargin(6);
+    layout->setContentsMargins(6, 6, 6, 6);
     m_summaryWidget->setLayout(layout);
     m_summaryLabel = new QLabel;
     m_summaryLabel->setPalette(pal);
@@ -123,20 +132,18 @@ TestResultsPane::TestResultsPane(QObject *parent) :
     TestResultDelegate *trd = new TestResultDelegate(this);
     m_treeView->setItemDelegate(trd);
 
-    outputLayout->addWidget(Core::ItemViewFind::createSearchableWrapper(m_treeView));
+    outputLayout->addWidget(ItemViewFind::createSearchableWrapper(m_treeView));
 
     m_textOutput = new QPlainTextEdit;
     m_textOutput->setPalette(pal);
-    QFont font("monospace");
-    font.setStyleHint(QFont::TypeWriter);
-    m_textOutput->setFont(font);
+    m_textOutput->setFont(TextEditor::TextEditorSettings::fontSettings().font());
     m_textOutput->setWordWrapMode(QTextOption::WordWrap);
     m_textOutput->setReadOnly(true);
     m_outputWidget->addWidget(m_textOutput);
 
     auto agg = new Aggregation::Aggregate;
     agg->add(m_textOutput);
-    agg->add(new Core::BaseTextFind(m_textOutput));
+    agg->add(new BaseTextFind(m_textOutput));
 
     createToolButtons();
 
@@ -148,7 +155,7 @@ TestResultsPane::TestResultsPane(QObject *parent) :
     connect(m_treeView, &ResultsTreeView::copyShortcutTriggered, [this] () {
         onCopyItemTriggered(getTestResult(m_treeView->currentIndex()));
     });
-    connect(m_model, &TestResultModel::requestExpansion, [this] (QModelIndex idx) {
+    connect(m_model, &TestResultModel::requestExpansion, [this] (const QModelIndex &idx) {
         m_treeView->expand(m_filterModel->mapFromSource(idx));
     });
     connect(TestRunner::instance(), &TestRunner::testRunStarted,
@@ -157,6 +164,9 @@ TestResultsPane::TestResultsPane(QObject *parent) :
             this, &TestResultsPane::onTestRunFinished);
     connect(TestRunner::instance(), &TestRunner::testResultReady,
             this, &TestResultsPane::addTestResult);
+    connect(TestRunner::instance(), &TestRunner::hadDisabledTests,
+            m_model, &TestResultModel::raiseDisabledTests);
+    visualOutputWidget->installEventFilter(this);
 }
 
 void TestResultsPane::createToolButtons()
@@ -174,13 +184,27 @@ void TestResultsPane::createToolButtons()
     });
 
     m_runAll = new QToolButton(m_treeView);
-    m_runAll->setDefaultAction(Core::ActionManager::command(Constants::ACTION_RUN_ALL_ID)->action());
+    m_runAll->setDefaultAction(
+                Utils::ProxyAction::proxyActionWithIcon(
+                    ActionManager::command(Constants::ACTION_RUN_ALL_ID)->action(),
+                    Utils::Icons::RUN_SMALL_TOOLBAR.icon()));
 
     m_runSelected = new QToolButton(m_treeView);
-    m_runSelected->setDefaultAction(Core::ActionManager::command(Constants::ACTION_RUN_SELECTED_ID)->action());
+    m_runSelected->setDefaultAction(
+                Utils::ProxyAction::proxyActionWithIcon(
+                    ActionManager::command(Constants::ACTION_RUN_SELECTED_ID)->action(),
+                    Utils::Icons::RUN_SELECTED_TOOLBAR.icon()));
 
+    m_runFailed = new QToolButton(m_treeView);
+    m_runFailed->setDefaultAction(
+                Utils::ProxyAction::proxyActionWithIcon(
+                    ActionManager::command(Constants::ACTION_RUN_FAILED_ID)->action(),
+                    Icons::RUN_FAILED_TOOLBAR.icon()));
     m_runFile = new QToolButton(m_treeView);
-    m_runFile->setDefaultAction(Core::ActionManager::command(Constants::ACTION_RUN_FILE_ID)->action());
+    m_runFile->setDefaultAction(
+                Utils::ProxyAction::proxyActionWithIcon(
+                    ActionManager::command(Constants::ACTION_RUN_FILE_ID)->action(),
+                    Utils::Icons::RUN_FILE_TOOLBAR.icon()));
 
     m_stopTestRun = new QToolButton(m_treeView);
     m_stopTestRun->setIcon(Utils::Icons::STOP_SMALL_TOOLBAR.icon());
@@ -192,7 +216,6 @@ void TestResultsPane::createToolButtons()
     m_filterButton->setIcon(Utils::Icons::FILTER.icon());
     m_filterButton->setToolTip(tr("Filter Test Results"));
     m_filterButton->setProperty("noArrow", true);
-    m_filterButton->setAutoRaise(true);
     m_filterButton->setPopupMode(QToolButton::InstantPopup);
     m_filterMenu = new QMenu(m_filterButton);
     initializeFilterMenu();
@@ -228,16 +251,64 @@ void TestResultsPane::addTestResult(const TestResultPtr &result)
     m_atEnd = scrollBar ? scrollBar->value() == scrollBar->maximum() : true;
 
     m_model->addTestResult(result, m_expandCollapse->isChecked());
-    setIconBadgeNumber(m_model->resultTypeCount(Result::Fail)
-                       + m_model->resultTypeCount(Result::MessageFatal)
-                       + m_model->resultTypeCount(Result::UnexpectedPass));
+    m_filterModel->invalidate();
+    setIconBadgeNumber(m_model->resultTypeCount(ResultType::Fail)
+                       + m_model->resultTypeCount(ResultType::MessageFatal)
+                       + m_model->resultTypeCount(ResultType::UnexpectedPass));
     flash();
     navigateStateChanged();
 }
 
-void TestResultsPane::addOutput(const QByteArray &output)
+static void checkAndFineTuneColors(QTextCharFormat *format)
 {
-    m_textOutput->appendPlainText(QString::fromLatin1(TestOutputReader::chopLineBreak(output)));
+    QTC_ASSERT(format, return);
+    const QColor bgColor = format->background().color();
+    QColor fgColor = format->foreground().color();
+
+    if (Utils::StyleHelper::isReadableOn(bgColor, fgColor))
+        return;
+
+    int h, s, v;
+    fgColor.getHsv(&h, &s, &v);
+    // adjust the color value to ensure better readability
+    if (Utils::StyleHelper::luminance(bgColor) < .5)
+        v = v + 64;
+    else
+        v = v - 64;
+
+    fgColor.setHsv(h, s, v);
+    if (!Utils::StyleHelper::isReadableOn(bgColor, fgColor)) {
+        s = (s + 128) % 255;    // adjust the saturation to ensure better readability
+        fgColor.setHsv(h, s, v);
+        if (!Utils::StyleHelper::isReadableOn(bgColor, fgColor))
+            return;
+    }
+
+    format->setForeground(fgColor);
+}
+
+void TestResultsPane::addOutputLine(const QByteArray &outputLine, OutputChannel channel)
+{
+    if (!QTC_GUARD(!outputLine.contains('\n'))) {
+        for (const auto &line : outputLine.split('\n'))
+            addOutputLine(line, channel);
+        return;
+    }
+
+    const Utils::FormattedText formattedText
+            = Utils::FormattedText{QString::fromUtf8(outputLine), m_defaultFormat};
+    const QList<Utils::FormattedText> formatted = channel == OutputChannel::StdOut
+            ? m_stdOutHandler.parseText(formattedText)
+            : m_stdErrHandler.parseText(formattedText);
+
+    QTextCursor cursor = m_textOutput->textCursor();
+    cursor.beginEditBlock();
+    for (auto formattedText : formatted) {
+        checkAndFineTuneColors(&formattedText.format);
+        cursor.insertText(formattedText.text, formattedText.format);
+    }
+    cursor.insertText("\n");
+    cursor.endEditBlock();
 }
 
 QWidget *TestResultsPane::outputWidget(QWidget *parent)
@@ -252,7 +323,7 @@ QWidget *TestResultsPane::outputWidget(QWidget *parent)
 
 QList<QWidget *> TestResultsPane::toolBarWidgets() const
 {
-    return {m_expandCollapse, m_runAll, m_runSelected, m_runFile, m_stopTestRun,
+    return {m_expandCollapse, m_runAll, m_runSelected, m_runFailed, m_runFile, m_stopTestRun,
             m_outputToggleButton, m_filterButton};
 }
 
@@ -278,11 +349,15 @@ void TestResultsPane::clearContents()
     connect(m_treeView->verticalScrollBar(), &QScrollBar::rangeChanged,
             this, &TestResultsPane::onScrollBarRangeChanged, Qt::UniqueConnection);
     m_textOutput->clear();
-    clearMarks();
-}
+    m_defaultFormat.setBackground(Utils::creatorTheme()->palette().color(
+                                      m_textOutput->backgroundRole()));
+    m_defaultFormat.setForeground(Utils::creatorTheme()->palette().color(
+                                      m_textOutput->foregroundRole()));
 
-void TestResultsPane::visibilityChanged(bool /*visible*/)
-{
+    // in case they had been forgotten to reset
+    m_stdErrHandler.endFormatScope();
+    m_stdOutHandler.endFormatScope();
+    clearMarks();
 }
 
 void TestResultsPane::setFocus()
@@ -401,7 +476,7 @@ void TestResultsPane::onItemActivated(const QModelIndex &index)
 
     const TestResult *testResult = m_filterModel->testResult(index);
     if (testResult && !testResult->fileName().isEmpty())
-        Core::EditorManager::openEditorAt(testResult->fileName(), testResult->line(), 0);
+        EditorManager::openEditorAt(Utils::Link{testResult->fileName(), testResult->line(), 0});
 }
 
 void TestResultsPane::onRunAllTriggered()
@@ -423,68 +498,77 @@ void TestResultsPane::initializeFilterMenu()
     const bool omitIntern = AutotestPlugin::settings()->omitInternalMssg;
     // FilterModel has all messages enabled by default
     if (omitIntern)
-        m_filterModel->toggleTestResultType(Result::MessageInternal);
+        m_filterModel->toggleTestResultType(ResultType::MessageInternal);
 
-    QMap<Result::Type, QString> textAndType;
-    textAndType.insert(Result::Pass, tr("Pass"));
-    textAndType.insert(Result::Fail, tr("Fail"));
-    textAndType.insert(Result::ExpectedFail, tr("Expected Fail"));
-    textAndType.insert(Result::UnexpectedPass, tr("Unexpected Pass"));
-    textAndType.insert(Result::Skip, tr("Skip"));
-    textAndType.insert(Result::Benchmark, tr("Benchmarks"));
-    textAndType.insert(Result::MessageDebug, tr("Debug Messages"));
-    textAndType.insert(Result::MessageWarn, tr("Warning Messages"));
-    textAndType.insert(Result::MessageInternal, tr("Internal Messages"));
-    for (Result::Type result : textAndType.keys()) {
+    QMap<ResultType, QString> textAndType;
+    textAndType.insert(ResultType::Pass, tr("Pass"));
+    textAndType.insert(ResultType::Fail, tr("Fail"));
+    textAndType.insert(ResultType::ExpectedFail, tr("Expected Fail"));
+    textAndType.insert(ResultType::UnexpectedPass, tr("Unexpected Pass"));
+    textAndType.insert(ResultType::Skip, tr("Skip"));
+    textAndType.insert(ResultType::Benchmark, tr("Benchmarks"));
+    textAndType.insert(ResultType::MessageDebug, tr("Debug Messages"));
+    textAndType.insert(ResultType::MessageWarn, tr("Warning Messages"));
+    textAndType.insert(ResultType::MessageInternal, tr("Internal Messages"));
+    for (auto it = textAndType.cbegin(); it != textAndType.cend(); ++it) {
+        const ResultType &result = it.key();
         QAction *action = new QAction(m_filterMenu);
-        action->setText(textAndType.value(result));
+        action->setText(it.value());
         action->setCheckable(true);
-        action->setChecked(result != Result::MessageInternal || !omitIntern);
-        action->setData(result);
+        action->setChecked(result != ResultType::MessageInternal || !omitIntern);
+        action->setData(int(result));
         m_filterMenu->addAction(action);
     }
     m_filterMenu->addSeparator();
-    QAction *action = new QAction(m_filterMenu);
-    action->setText(tr("Check All Filters"));
-    action->setCheckable(false);
+    QAction *action = new QAction(tr("Check All Filters"), m_filterMenu);
     m_filterMenu->addAction(action);
-    connect(action, &QAction::triggered, this, &TestResultsPane::enableAllFilter);
+    connect(action, &QAction::triggered, this, [this]() { TestResultsPane::checkAllFilter(true); });
+    action = new QAction(tr("Uncheck All Filters"), m_filterMenu);
+    m_filterMenu->addAction(action);
+    connect(action, &QAction::triggered, this, [this]() { TestResultsPane::checkAllFilter(false); });
 }
 
 void TestResultsPane::updateSummaryLabel()
 {
-    QString labelText = QString("<p>Test summary:&nbsp;&nbsp; %1 %2, %3 %4")
-            .arg(QString::number(m_model->resultTypeCount(Result::Pass)), tr("passes"),
-                 QString::number(m_model->resultTypeCount(Result::Fail)), tr("fails"));
-    int count = m_model->resultTypeCount(Result::UnexpectedPass);
+    QString labelText = QString("<p>");
+    labelText.append(tr("Test summary"));
+    labelText.append(":&nbsp;&nbsp; ");
+    int count = m_model->resultTypeCount(ResultType::Pass);
+    labelText += QString::number(count) + ' ' + tr("passes");
+    count = m_model->resultTypeCount(ResultType::Fail);
+    labelText += ", " + QString::number(count) + ' ' + tr("fails");
+    count = m_model->resultTypeCount(ResultType::UnexpectedPass);
     if (count)
-        labelText.append(QString(", %1 %2").arg(QString::number(count), tr("unexpected passes")));
-    count = m_model->resultTypeCount(Result::ExpectedFail);
+        labelText += ", " + QString::number(count) + ' ' + tr("unexpected passes");
+    count = m_model->resultTypeCount(ResultType::ExpectedFail);
     if (count)
-        labelText.append(QString(", %1 %2").arg(QString::number(count), tr("expected fails")));
-    count = m_model->resultTypeCount(Result::MessageFatal);
+        labelText += ", " + QString::number(count) + ' ' + tr("expected fails");
+    count = m_model->resultTypeCount(ResultType::MessageFatal);
     if (count)
-        labelText.append(QString(", %1 %2").arg(QString::number(count), tr("fatals")));
-    count = m_model->resultTypeCount(Result::BlacklistedFail)
-            + m_model->resultTypeCount(Result::BlacklistedPass);
+        labelText += ", " + QString::number(count) + ' ' + tr("fatals");
+    count = m_model->resultTypeCount(ResultType::BlacklistedFail)
+            + m_model->resultTypeCount(ResultType::BlacklistedXFail)
+            + m_model->resultTypeCount(ResultType::BlacklistedPass)
+            + m_model->resultTypeCount(ResultType::BlacklistedXPass);
     if (count)
-        labelText.append(QString(", %1 %2").arg(QString::number(count), tr("blacklisted")));
-
+        labelText += ", " + QString::number(count) + ' ' + tr("blacklisted");
+    count = m_model->resultTypeCount(ResultType::Skip);
+    if (count)
+        labelText += ", " + QString::number(count) + ' ' + tr("skipped");
     count = m_model->disabledTests();
     if (count)
-        labelText.append(tr(", %1 disabled").arg(count));
-
+        labelText += ", " + QString::number(count) + ' ' + tr("disabled");
     labelText.append(".</p>");
     m_summaryLabel->setText(labelText);
 }
 
-void TestResultsPane::enableAllFilter()
+void TestResultsPane::checkAllFilter(bool checked)
 {
     for (QAction *action : m_filterMenu->actions()) {
         if (action->isCheckable())
-            action->setChecked(true);
+            action->setChecked(checked);
     }
-    m_filterModel->enableAllResultTypes();
+    m_filterModel->enableAllResultTypes(checked);
 }
 
 void TestResultsPane::filterMenuTriggered(QAction *action)
@@ -493,6 +577,14 @@ void TestResultsPane::filterMenuTriggered(QAction *action)
     navigateStateChanged();
 }
 
+bool TestResultsPane::eventFilter(QObject *object, QEvent *event)
+{
+    QTC_ASSERT(m_outputWidget, return false);
+    if (event->type() == QEvent::Resize && object->parent() == m_outputWidget)
+        static_cast<TestResultDelegate *>(m_treeView->itemDelegate())->clearCache();
+    return false;
+}
+
 void TestResultsPane::onTestRunStarted()
 {
     m_testRunning = true;
@@ -501,6 +593,13 @@ void TestResultsPane::onTestRunStarted()
     m_summaryWidget->setVisible(false);
 }
 
+static bool hasFailedTests(const TestResultModel *model)
+{
+    return (model->resultTypeCount(ResultType::Fail) > 0
+            || model->resultTypeCount(ResultType::MessageFatal) > 0
+            || model->resultTypeCount(ResultType::UnexpectedPass) > 0);
+}
+
 void TestResultsPane::onTestRunFinished()
 {
     m_testRunning = false;
@@ -512,8 +611,10 @@ void TestResultsPane::onTestRunFinished()
     m_model->removeCurrentTestMessage();
     disconnect(m_treeView->verticalScrollBar(), &QScrollBar::rangeChanged,
                this, &TestResultsPane::onScrollBarRangeChanged);
-    if (!m_treeView->isVisible())
-        popup(Core::IOutputPane::NoModeSwitch);
+    if (AutotestPlugin::settings()->popupOnFinish
+            && (!AutotestPlugin::settings()->popupOnFail || hasFailedTests(m_model))) {
+        popup(IOutputPane::NoModeSwitch);
+    }
     createMarks();
 }
 
@@ -557,7 +658,14 @@ void TestResultsPane::onCustomContextMenuRequested(const QPoint &pos)
     menu.addAction(action);
 
     action = new QAction(tr("Debug This Test"), &menu);
-    action->setEnabled(correlatingItem && correlatingItem->canProvideDebugConfiguration());
+    bool debugEnabled = false;
+    if (correlatingItem) {
+        if (correlatingItem->testBase()->type() == ITestBase::Framework) {
+            auto testTreeItem = static_cast<const TestTreeItem *>(correlatingItem);
+            debugEnabled = testTreeItem && testTreeItem->canProvideDebugConfiguration();
+        }
+    }
+    action->setEnabled(debugEnabled);
     connect(action, &QAction::triggered, this, [this, clicked] {
         onRunThisTestTriggered(TestRunMode::Debug, clicked);
     });
@@ -590,14 +698,14 @@ void TestResultsPane::onCopyWholeTriggered()
 
 void TestResultsPane::onSaveWholeTriggered()
 {
-    const QString fileName = QFileDialog::getSaveFileName(Core::ICore::dialogParent(),
+    const QString fileName = QFileDialog::getSaveFileName(ICore::dialogParent(),
                                                           tr("Save Output To"));
     if (fileName.isEmpty())
         return;
 
-    Utils::FileSaver saver(fileName, QIODevice::Text);
+    Utils::FileSaver saver(Utils::FilePath::fromString(fileName), QIODevice::Text);
     if (!saver.write(getWholeOutput().toUtf8()) || !saver.finalize()) {
-        QMessageBox::critical(Core::ICore::dialogParent(), tr("Error"),
+        QMessageBox::critical(ICore::dialogParent(), tr("Error"),
                               tr("Failed to write \"%1\".\n\n%2").arg(fileName)
                               .arg(saver.errorString()));
     }
@@ -607,7 +715,7 @@ void TestResultsPane::onRunThisTestTriggered(TestRunMode runMode, const TestResu
 {
     QTC_ASSERT(result, return);
 
-    const TestTreeItem *item = result->findTestTreeItem();
+    const ITestTreeItem *item = result->findTestTreeItem();
 
     if (item)
         TestRunner::instance()->runTest(runMode, item);
@@ -629,7 +737,8 @@ QString TestResultsPane::getWholeOutput(const QModelIndex &parent)
         QModelIndex current = m_model->index(row, 0, parent);
         const TestResult *result = m_model->testResult(current);
         QTC_ASSERT(result, continue);
-        output.append(TestResult::resultToString(result->result())).append('\t');
+        if (auto item = m_model->itemForIndex(current))
+            output.append(item->resultString()).append('\t');
         output.append(result->outputString(true)).append('\n');
         output.append(getWholeOutput(current));
     }
@@ -639,8 +748,8 @@ QString TestResultsPane::getWholeOutput(const QModelIndex &parent)
 void TestResultsPane::createMarks(const QModelIndex &parent)
 {
     const TestResult *parentResult = m_model->testResult(parent);
-    Result::Type parentType = parentResult ? parentResult->result() : Result::Invalid;
-    const QVector<Result::Type> interested{Result::Fail, Result::UnexpectedPass};
+    ResultType parentType = parentResult ? parentResult->result() : ResultType::Invalid;
+    const QVector<ResultType> interested{ResultType::Fail, ResultType::UnexpectedPass};
     for (int row = 0, count = m_model->rowCount(parent); row < count; ++row) {
         const QModelIndex index = m_model->index(row, 0, parent);
         const TestResult *result = m_model->testResult(index);
@@ -649,11 +758,10 @@ void TestResultsPane::createMarks(const QModelIndex &parent)
         if (m_model->hasChildren(index))
             createMarks(index);
 
-        bool isLocationItem = result->result() == Result::MessageLocation;
+        bool isLocationItem = result->result() == ResultType::MessageLocation;
         if (interested.contains(result->result())
                 || (isLocationItem && interested.contains(parentType))) {
-            const Utils::FileName fileName = Utils::FileName::fromString(result->fileName());
-            TestEditorMark *mark = new TestEditorMark(index, fileName, result->line());
+            TestEditorMark *mark = new TestEditorMark(index, result->fileName(), result->line());
             mark->setIcon(index.data(Qt::DecorationRole).value<QIcon>());
             mark->setColor(Utils::Theme::OutputPanes_TestFailTextColor);
             mark->setPriority(TextEditor::TextMark::NormalPriority);
@@ -673,7 +781,7 @@ void TestResultsPane::showTestResult(const QModelIndex &index)
 {
     QModelIndex mapped = m_filterModel->mapFromSource(index);
     if (mapped.isValid()) {
-        popup(Core::IOutputPane::NoModeSwitch);
+        popup(IOutputPane::NoModeSwitch);
         m_treeView->setCurrentIndex(mapped);
     }
 }
