@@ -26,15 +26,15 @@
 #include "bazaarclient.h"
 #include "constants.h"
 
-#include <coreplugin/id.h>
-
 #include <vcsbase/vcsbaseplugin.h>
 #include <vcsbase/vcsoutputwindow.h>
-#include <vcsbase/vcsbaseeditorparameterwidget.h>
-#include <utils/synchronousprocess.h>
+#include <vcsbase/vcsbaseeditorconfig.h>
+
+#include <utils/hostosinfo.h>
 
 #include <QDir>
 #include <QFileInfo>
+#include <QRegularExpression>
 #include <QTextStream>
 #include <QDebug>
 
@@ -44,90 +44,69 @@ using namespace VcsBase;
 namespace Bazaar {
 namespace Internal {
 
-class BazaarDiffExitCodeInterpreter : public ExitCodeInterpreter
-{
-    Q_OBJECT
-public:
-    BazaarDiffExitCodeInterpreter(QObject *parent) : ExitCodeInterpreter(parent) {}
-    SynchronousProcessResponse::Result interpretExitCode(int code) const;
-};
-
-SynchronousProcessResponse::Result BazaarDiffExitCodeInterpreter::interpretExitCode(int code) const
-{
-    if (code < 0 || code > 2)
-        return SynchronousProcessResponse::FinishedError;
-    return SynchronousProcessResponse::Finished;
-}
-
 // Parameter widget controlling whitespace diff mode, associated with a parameter
-class BazaarDiffParameterWidget : public VcsBaseEditorParameterWidget
+class BazaarDiffConfig : public VcsBaseEditorConfig
 {
     Q_OBJECT
 public:
-    BazaarDiffParameterWidget(VcsBaseClientSettings &settings, QWidget *parent = 0) :
-        VcsBaseEditorParameterWidget(parent)
+    BazaarDiffConfig(BazaarSettings &settings, QToolBar *toolBar) :
+        VcsBaseEditorConfig(toolBar)
     {
-        mapSetting(addToggleButton(QLatin1String("-w"), tr("Ignore Whitespace")),
-                   settings.boolPointer(BazaarSettings::diffIgnoreWhiteSpaceKey));
-        mapSetting(addToggleButton(QLatin1String("-B"), tr("Ignore Blank Lines")),
-                   settings.boolPointer(BazaarSettings::diffIgnoreBlankLinesKey));
+        mapSetting(addToggleButton("-w", tr("Ignore Whitespace")),
+                   &settings.diffIgnoreWhiteSpace);
+        mapSetting(addToggleButton("-B", tr("Ignore Blank Lines")),
+                   &settings.diffIgnoreBlankLines);
     }
 
-    QStringList arguments() const
+    QStringList arguments() const override
     {
         QStringList args;
         // Bazaar wants "--diff-options=-w -B.."
-        const QStringList formatArguments = VcsBaseEditorParameterWidget::arguments();
+        const QStringList formatArguments = VcsBaseEditorConfig::arguments();
         if (!formatArguments.isEmpty()) {
-            const QString a = QLatin1String("--diff-options=")
-                    + formatArguments.join(QString(QLatin1Char(' ')));
+            const QString a = "--diff-options=" + formatArguments.join(' ');
             args.append(a);
         }
         return args;
     }
 };
 
-class BazaarLogParameterWidget : public VcsBaseEditorParameterWidget
+class BazaarLogConfig : public VcsBaseEditorConfig
 {
     Q_OBJECT
 public:
-    BazaarLogParameterWidget(VcsBaseClientSettings &settings, QWidget *parent = 0) :
-        VcsBaseEditorParameterWidget(parent)
+    BazaarLogConfig(BazaarSettings &settings, QToolBar *toolBar) :
+        VcsBaseEditorConfig(toolBar)
     {
-        mapSetting(addToggleButton(QLatin1String("--verbose"), tr("Verbose"),
+        mapSetting(addToggleButton("--verbose", tr("Verbose"),
                                    tr("Show files changed in each revision.")),
-                   settings.boolPointer(BazaarSettings::logVerboseKey));
-        mapSetting(addToggleButton(QLatin1String("--forward"), tr("Forward"),
+                   &settings.logVerbose);
+        mapSetting(addToggleButton("--forward", tr("Forward"),
                                    tr("Show from oldest to newest.")),
-                   settings.boolPointer(BazaarSettings::logForwardKey));
-        mapSetting(addToggleButton(QLatin1String("--include-merges"), tr("Include Merges"),
+                   &settings.logForward);
+        mapSetting(addToggleButton("--include-merges", tr("Include Merges"),
                                    tr("Show merged revisions.")),
-                   settings.boolPointer(BazaarSettings::logIncludeMergesKey));
+                   &settings.logIncludeMerges);
 
-        QList<ComboBoxItem> logChoices;
-        logChoices << ComboBoxItem(tr("Detailed"), QLatin1String("long"))
-                   << ComboBoxItem(tr("Moderately Short"), QLatin1String("short"))
-                   << ComboBoxItem(tr("One Line"), QLatin1String("line"))
-                   << ComboBoxItem(tr("GNU Change Log"), QLatin1String("gnu-changelog"));
-        mapSetting(addComboBox(QStringList(QLatin1String("--log-format=%1")), logChoices),
-                   settings.stringPointer(BazaarSettings::logFormatKey));
+        const QList<ChoiceItem> logChoices = {
+            {tr("Detailed"), "long"},
+            {tr("Moderately Short"), "short"},
+            {tr("One Line"), "line"},
+            {tr("GNU Change Log"), "gnu-changelog"}
+        };
+        mapSetting(addChoices(tr("Format"), { "--log-format=%1" }, logChoices),
+                   &settings.logFormat);
     }
 };
 
-BazaarClient::BazaarClient() : VcsBaseClient(new BazaarSettings)
+BazaarClient::BazaarClient(BazaarSettings *settings) : VcsBaseClient(settings)
 {
-    setDiffParameterWidgetCreator([this] { return new BazaarDiffParameterWidget(settings()); });
-    setLogParameterWidgetCreator([this] { return new BazaarLogParameterWidget(settings()); });
-}
-
-bool BazaarClient::synchronousSetUserId()
-{
-    QStringList args;
-    args << QLatin1String("whoami")
-         << (settings().stringValue(BazaarSettings::userNameKey) + QLatin1String(" <")
-             + settings().stringValue(BazaarSettings::userEmailKey) + QLatin1Char('>'));
-    QByteArray stdOut;
-    return vcsFullySynchronousExec(QDir::currentPath(), args, &stdOut);
+    setDiffConfigCreator([settings](QToolBar *toolBar) {
+        return new BazaarDiffConfig(*settings, toolBar);
+    });
+    setLogConfigCreator([settings](QToolBar *toolBar) {
+        return new BazaarLogConfig(*settings, toolBar);
+    });
 }
 
 BranchInfo BazaarClient::synchronousBranchQuery(const QString &repositoryRoot) const
@@ -141,14 +120,18 @@ BranchInfo BazaarClient::synchronousBranchQuery(const QString &repositoryRoot) c
     QTextStream ts(&branchConfFile);
     QString branchLocation;
     QString isBranchBound;
-    QRegExp branchLocationRx(QLatin1String("bound_location\\s*=\\s*(.+)$"));
-    QRegExp isBranchBoundRx(QLatin1String("bound\\s*=\\s*(.+)$"));
+    QRegularExpression branchLocationRx("bound_location\\s*=\\s*(.+)$");
+    QRegularExpression isBranchBoundRx("bound\\s*=\\s*(.+)$");
     while (!ts.atEnd() && (branchLocation.isEmpty() || isBranchBound.isEmpty())) {
         const QString line = ts.readLine();
-        if (branchLocationRx.indexIn(line) != -1)
-            branchLocation = branchLocationRx.cap(1);
-        else if (isBranchBoundRx.indexIn(line) != -1)
-            isBranchBound = isBranchBoundRx.cap(1);
+        QRegularExpressionMatch match = branchLocationRx.match(line);
+        if (match.hasMatch()) {
+            branchLocation = match.captured(1);
+        } else {
+            QRegularExpressionMatch match = isBranchBoundRx.match(line);
+            if (match.hasMatch())
+                isBranchBound = match.captured(1);
+        }
     }
     if (isBranchBound.simplified().toLower() == QLatin1String("true"))
         return BranchInfo(branchLocation, true);
@@ -166,11 +149,11 @@ bool BazaarClient::synchronousUncommit(const QString &workingDir,
          << QLatin1String("--verbose") // Will print out what is being removed
          << revisionSpec(revision)
          << extraOptions;
-    QByteArray stdOut;
-    const bool success = vcsFullySynchronousExec(workingDir, args, &stdOut);
-    if (!stdOut.isEmpty())
-        VcsOutputWindow::append(QString::fromUtf8(stdOut));
-    return success;
+
+    QtcProcess proc;
+    vcsFullySynchronousExec(proc, workingDir, args);
+    VcsOutputWindow::append(proc.stdOut());
+    return proc.result() == QtcProcess::FinishedWithSuccess;
 }
 
 void BazaarClient::commit(const QString &repositoryRoot, const QStringList &files,
@@ -180,12 +163,18 @@ void BazaarClient::commit(const QString &repositoryRoot, const QStringList &file
                           QStringList(extraOptions) << QLatin1String("-F") << commitMessageFile);
 }
 
-void BazaarClient::annotate(const QString &workingDir, const QString &file,
-                            const QString &revision, int lineNumber,
-                            const QStringList &extraOptions)
+VcsBaseEditorWidget *BazaarClient::annotate(
+        const QString &workingDir, const QString &file, const QString &revision,
+        int lineNumber, const QStringList &extraOptions)
 {
-    VcsBaseClient::annotate(workingDir, file, revision, lineNumber,
-                            QStringList(extraOptions) << QLatin1String("--long"));
+    return VcsBaseClient::annotate(workingDir, file, revision, lineNumber,
+                                   QStringList(extraOptions) << QLatin1String("--long"));
+}
+
+bool BazaarClient::isVcsDirectory(const FilePath &fileName) const
+{
+    return fileName.isDir()
+            && !fileName.fileName().compare(Constants::BAZAARREPO, HostOsInfo::fileNameCaseSensitivity());
 }
 
 QString BazaarClient::findTopLevelForFile(const QFileInfo &file) const
@@ -193,20 +182,20 @@ QString BazaarClient::findTopLevelForFile(const QFileInfo &file) const
     const QString repositoryCheckFile =
             QLatin1String(Constants::BAZAARREPO) + QLatin1String("/branch-format");
     return file.isDir() ?
-                VcsBasePlugin::findRepositoryForDirectory(file.absoluteFilePath(),
-                                                          repositoryCheckFile) :
-                VcsBasePlugin::findRepositoryForDirectory(file.absolutePath(),
-                                                          repositoryCheckFile);
+                VcsBase::findRepositoryForDirectory(file.absoluteFilePath(), repositoryCheckFile) :
+                VcsBase::findRepositoryForDirectory(file.absolutePath(), repositoryCheckFile);
 }
 
 bool BazaarClient::managesFile(const QString &workingDirectory, const QString &fileName) const
 {
     QStringList args(QLatin1String("status"));
     args << fileName;
-    QByteArray stdOut;
-    if (!vcsFullySynchronousExec(workingDirectory, args, &stdOut))
+
+    QtcProcess proc;
+    vcsFullySynchronousExec(proc, workingDirectory, args);
+    if (proc.result() != QtcProcess::FinishedWithSuccess)
         return false;
-    return !stdOut.startsWith("unknown");
+    return proc.rawStdOut().startsWith("unknown");
 }
 
 void BazaarClient::view(const QString &source, const QString &id, const QStringList &extraOptions)
@@ -216,7 +205,7 @@ void BazaarClient::view(const QString &source, const QString &id, const QStringL
     VcsBaseClient::view(source, id, args);
 }
 
-Core::Id BazaarClient::vcsEditorKind(VcsCommandTag cmd) const
+Utils::Id BazaarClient::vcsEditorKind(VcsCommandTag cmd) const
 {
     switch (cmd) {
     case AnnotateCommand:
@@ -226,7 +215,7 @@ Core::Id BazaarClient::vcsEditorKind(VcsCommandTag cmd) const
     case LogCommand:
         return Constants::FILELOG_ID;
     default:
-        return Core::Id();
+        return Utils::Id();
     }
 }
 
@@ -240,14 +229,15 @@ QString BazaarClient::vcsCommandString(VcsCommandTag cmd) const
     }
 }
 
-ExitCodeInterpreter *BazaarClient::exitCodeInterpreter(VcsCommandTag cmd, QObject *parent) const
+ExitCodeInterpreter BazaarClient::exitCodeInterpreter(VcsCommandTag cmd) const
 {
-    switch (cmd) {
-    case DiffCommand:
-        return new BazaarDiffExitCodeInterpreter(parent);
-    default:
-        return 0;
+    if (cmd == DiffCommand) {
+        return [](int code) {
+            return (code < 0 || code > 2) ? QtcProcess::FinishedWithError
+                                          : QtcProcess::FinishedWithSuccess;
+        };
     }
+    return {};
 }
 
 QStringList BazaarClient::revisionSpec(const QString &revision) const
