@@ -34,7 +34,6 @@
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
-#include <QRegExp>
 #include <QVariant>
 
 #include <QMenu>
@@ -42,17 +41,17 @@
 #include <QMessageBox>
 #include <QHBoxLayout>
 #include <QLineEdit>
-#include <QRegExpValidator>
+#include <QRegularExpressionValidator>
 #include <QToolBar>
 
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/coreconstants.h>
-#include <coreplugin/id.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/find/ifindsupport.h>
 #include <coreplugin/idocument.h>
 #include <extensionsystem/pluginmanager.h>
+
 #include <utils/reloadpromptutils.h>
 #include <utils/qtcassert.h>
 
@@ -148,7 +147,7 @@ public:
             } else {
                 result = NotFound;
                 m_contPos = -1;
-                m_widget->highlightSearchResults(QByteArray(), 0);
+                m_widget->highlightSearchResults(QByteArray(), {});
             }
         }
         return result;
@@ -231,28 +230,27 @@ public:
         return type == TypeRemoved ? BehaviorSilent : IDocument::reloadBehavior(state, type);
     }
 
-    bool save(QString *errorString, const QString &fn, bool autoSave) override
+    bool save(QString *errorString, const Utils::FilePath &filePath, bool autoSave) override
     {
         QTC_ASSERT(!autoSave, return true); // bineditor does not support autosave - it would be a bit expensive
-        const FileName fileNameToUse = fn.isEmpty() ? filePath() : FileName::fromString(fn);
-        if (m_widget->save(errorString, filePath().toString(), fileNameToUse.toString())) {
+        const FilePath &fileNameToUse = filePath.isEmpty() ? this->filePath() : filePath;
+        if (m_widget->save(errorString, this->filePath().toString(), fileNameToUse.toString())) {
             setFilePath(fileNameToUse);
             return true;
-        } else {
-            return false;
         }
+        return false;
     }
 
-    OpenResult open(QString *errorString, const QString &fileName,
-                    const QString &realFileName) override
+    OpenResult open(QString *errorString, const Utils::FilePath &filePath,
+                    const Utils::FilePath &realFilePath) override
     {
-        QTC_CHECK(fileName == realFileName); // The bineditor can do no autosaving
-        return openImpl(errorString, fileName);
+        QTC_CHECK(filePath == realFilePath); // The bineditor can do no autosaving
+        return openImpl(errorString, filePath);
     }
 
-    OpenResult openImpl(QString *errorString, const QString &fileName, quint64 offset = 0)
+    OpenResult openImpl(QString *errorString, const Utils::FilePath &filePath, quint64 offset = 0)
     {
-        QFile file(fileName);
+        QFile file(filePath.toString());
         if (file.open(QIODevice::ReadOnly)) {
             file.close();
             quint64 size = static_cast<quint64>(file.size());
@@ -261,7 +259,7 @@ public:
                 if (errorString)
                     *errorString = msg;
                 else
-                    QMessageBox::critical(ICore::mainWindow(), tr("File Error"), msg);
+                    QMessageBox::critical(ICore::dialogParent(), tr("File Error"), msg);
                 return OpenResult::CannotHandle;
             }
             if (size / 16 >= qint64(1) << 31) {
@@ -270,27 +268,26 @@ public:
                 if (errorString)
                     *errorString = msg;
                 else
-                    QMessageBox::critical(ICore::mainWindow(), tr("File Error"), msg);
+                    QMessageBox::critical(ICore::dialogParent(), tr("File Error"), msg);
                 return OpenResult::CannotHandle;
             }
             if (offset >= size)
                 return OpenResult::CannotHandle;
-            setFilePath(FileName::fromString(fileName));
+            setFilePath(filePath);
             m_widget->setSizes(offset, file.size());
             return OpenResult::Success;
         }
-        QString errStr = tr("Cannot open %1: %2").arg(
-                QDir::toNativeSeparators(fileName), file.errorString());
+        QString errStr = tr("Cannot open %1: %2").arg(filePath.toUserOutput(), file.errorString());
         if (errorString)
             *errorString = errStr;
         else
-            QMessageBox::critical(ICore::mainWindow(), tr("File Error"), errStr);
+            QMessageBox::critical(ICore::dialogParent(), tr("File Error"), errStr);
         return OpenResult::ReadError;
     }
 
     void provideData(quint64 address)
     {
-        const FileName fn = filePath();
+        const FilePath fn = filePath();
         if (fn.isEmpty())
             return;
         QFile file(fn.toString());
@@ -304,7 +301,7 @@ public:
                 data += QByteArray(blockSize - dataSize, 0);
             m_widget->addData(address, data);
         } else {
-            QMessageBox::critical(ICore::mainWindow(), tr("File Error"),
+            QMessageBox::critical(ICore::dialogParent(), tr("File Error"),
                                   tr("Cannot open %1: %2").arg(
                                         fn.toUserOutput(), file.errorString()));
         }
@@ -313,7 +310,7 @@ public:
     void provideNewRange(quint64 offset)
     {
         if (filePath().exists())
-            openImpl(0, filePath().toString(), offset);
+            openImpl(nullptr, filePath(), offset);
     }
 
 public:
@@ -323,31 +320,20 @@ public:
                                                  : m_widget->isModified();
     }
 
-    bool isFileReadOnly() const override {
-        const FileName fn = filePath();
-        if (fn.isEmpty())
-            return false;
-        return !fn.toFileInfo().isWritable();
-    }
-
     bool isSaveAsAllowed() const override { return true; }
 
     bool reload(QString *errorString, ReloadFlag flag, ChangeType type) override
     {
+        Q_UNUSED(type)
         if (flag == FlagIgnore)
             return true;
-        if (type == TypePermissions) {
-            emit changed();
-        } else {
-            emit aboutToReload();
-            int cPos = m_widget->cursorPosition();
-            m_widget->clear();
-            const bool success = (openImpl(errorString, filePath().toString()) == OpenResult::Success);
-            m_widget->setCursorPosition(cPos);
-            emit reloadFinished(success);
-            return success;
-        }
-        return true;
+        emit aboutToReload();
+        int cPos = m_widget->cursorPosition();
+        m_widget->clear();
+        const bool success = (openImpl(errorString, filePath()) == OpenResult::Success);
+        m_widget->setCursorPosition(cPos);
+        emit reloadFinished(success);
+        return success;
     }
 
 private:
@@ -363,12 +349,11 @@ public:
         setWidget(widget);
         m_file = new BinEditorDocument(widget);
         m_addressEdit = new QLineEdit;
-        auto addressValidator = new QRegExpValidator(QRegExp("[0-9a-fA-F]{1,16}"), m_addressEdit);
+        auto addressValidator = new QRegularExpressionValidator(QRegularExpression("[0-9a-fA-F]{1,16}"), m_addressEdit);
         m_addressEdit->setValidator(addressValidator);
 
         auto l = new QHBoxLayout;
         auto w = new QWidget;
-        l->setMargin(0);
         l->setContentsMargins(0, 0, 5, 0);
         l->addStretch(1);
         l->addWidget(m_addressEdit);
@@ -404,13 +389,13 @@ private:
     }
 
     void jumpToAddress() {
-        editorWidget()->jumpToAddress(m_addressEdit->text().toULongLong(0, 16));
+        editorWidget()->jumpToAddress(m_addressEdit->text().toULongLong(nullptr, 16));
         updateCursorPosition(editorWidget()->cursorPosition());
     }
 
     BinEditorWidget *editorWidget() const
     {
-        QTC_ASSERT(qobject_cast<BinEditorWidget *>(m_widget.data()), return 0);
+        QTC_ASSERT(qobject_cast<BinEditorWidget *>(m_widget.data()), return nullptr);
         return static_cast<BinEditorWidget *>(m_widget.data());
     }
 
@@ -426,7 +411,7 @@ class BinEditorPluginPrivate : public QObject
 {
 public:
     BinEditorPluginPrivate();
-    ~BinEditorPluginPrivate();
+    ~BinEditorPluginPrivate() override;
 
     QAction *m_undoAction = nullptr;
     QAction *m_redoAction = nullptr;
@@ -472,33 +457,32 @@ BinEditorFactory::BinEditorFactory()
     setId(Core::Constants::K_DEFAULT_BINARY_EDITOR_ID);
     setDisplayName(QCoreApplication::translate("OpenWith::Editors", Constants::C_BINEDITOR_DISPLAY_NAME));
     addMimeType(Constants::C_BINEDITOR_MIMETYPE);
-}
 
-IEditor *BinEditorFactory::createEditor()
-{
-    auto widget = new BinEditorWidget();
-    auto editor = new BinEditor(widget);
+    setEditorCreator([this] {
+        auto widget = new BinEditorWidget();
+        auto editor = new BinEditor(widget);
 
-    connect(dd->m_undoAction, &QAction::triggered, widget, &BinEditorWidget::undo);
-    connect(dd->m_redoAction, &QAction::triggered, widget, &BinEditorWidget::redo);
-    connect(dd->m_copyAction, &QAction::triggered, widget, &BinEditorWidget::copy);
-    connect(dd->m_selectAllAction, &QAction::triggered, widget, &BinEditorWidget::selectAll);
+        connect(dd->m_undoAction, &QAction::triggered, widget, &BinEditorWidget::undo);
+        connect(dd->m_redoAction, &QAction::triggered, widget, &BinEditorWidget::redo);
+        connect(dd->m_copyAction, &QAction::triggered, widget, &BinEditorWidget::copy);
+        connect(dd->m_selectAllAction, &QAction::triggered, widget, &BinEditorWidget::selectAll);
 
-    auto updateActions = [widget] {
-        dd->m_selectAllAction->setEnabled(true);
-        dd->m_undoAction->setEnabled(widget->isUndoAvailable());
-        dd->m_redoAction->setEnabled(widget->isRedoAvailable());
-    };
+        auto updateActions = [widget] {
+            dd->m_selectAllAction->setEnabled(true);
+            dd->m_undoAction->setEnabled(widget->isUndoAvailable());
+            dd->m_redoAction->setEnabled(widget->isRedoAvailable());
+        };
 
-    connect(widget, &BinEditorWidget::undoAvailable, widget, updateActions);
-    connect(widget, &BinEditorWidget::redoAvailable, widget, updateActions);
+        connect(widget, &BinEditorWidget::undoAvailable, widget, updateActions);
+        connect(widget, &BinEditorWidget::redoAvailable, widget, updateActions);
 
-    auto aggregate = new Aggregation::Aggregate;
-    auto binEditorFind = new BinEditorFind(widget);
-    aggregate->add(binEditorFind);
-    aggregate->add(widget);
+        auto aggregate = new Aggregation::Aggregate;
+        auto binEditorFind = new BinEditorFind(widget);
+        aggregate->add(binEditorFind);
+        aggregate->add(widget);
 
-    return editor;
+        return editor;
+    });
 }
 
 ///////////////////////////////// BinEditor Services //////////////////////////////////
@@ -511,7 +495,7 @@ EditorService *FactoryServiceImpl::createEditorService(const QString &title0, bo
         IEditor *editor = EditorManager::openEditorWithContents(
                     Core::Constants::K_DEFAULT_BINARY_EDITOR_ID, &title);
         if (!editor)
-            return 0;
+            return nullptr;
         widget = qobject_cast<BinEditorWidget *>(editor->widget());
         widget->setEditor(editor);
     } else {
