@@ -25,15 +25,14 @@
 
 #pragma once
 
-#include <coreplugin/id.h>
+#include <coreplugin/core_global.h>
+
+#include <utils/fileutils.h>
+#include <utils/id.h>
 
 #include <QObject>
 #include <QPair>
 
-QT_BEGIN_NAMESPACE
-class QStringList;
-QT_END_NAMESPACE
-
 namespace Utils { class FilePath; }
 
 namespace Core {
@@ -54,7 +53,7 @@ public:
         KeepLinks
     };
 
-    using RecentFile = QPair<QString, Id>;
+    using RecentFile = QPair<Utils::FilePath, Utils::Id>;
 
     static DocumentManager *instance();
 
@@ -64,24 +63,23 @@ public:
     static bool removeDocument(IDocument *document);
     static QList<IDocument *> modifiedDocuments();
 
-    static void renamedFile(const QString &from, const QString &to);
+    static void renamedFile(const Utils::FilePath &from, const Utils::FilePath &to);
 
-    static void expectFileChange(const QString &fileName);
-    static void unexpectFileChange(const QString &fileName);
+    static void expectFileChange(const Utils::FilePath &filePath);
+    static void unexpectFileChange(const Utils::FilePath &filePath);
 
     // recent files
-    static void addToRecentFiles(const QString &fileName, Id editorId = Id());
+    static void addToRecentFiles(const Utils::FilePath &filePath, Utils::Id editorId = {});
     Q_SLOT void clearRecentFiles();
     static QList<RecentFile> recentFiles();
 
     static void saveSettings();
 
     // helper functions
-    static QString cleanAbsoluteFilePath(const QString &filePath, ResolveMode resolveMode);
-    static QString filePathKey(const QString &filePath, ResolveMode resolveMode);
+    static Utils::FilePath filePathKey(const Utils::FilePath &filePath, ResolveMode resolveMode);
 
     static bool saveDocument(IDocument *document,
-                             const QString &fileName = QString(),
+                             const Utils::FilePath &filePath = Utils::FilePath(),
                              bool *isReadOnly = nullptr);
 
     static QString allDocumentFactoryFiltersString(QString *allFilesFilter);
@@ -141,16 +139,18 @@ public:
 
     /* Used to notify e.g. the code model to update the given files. Does *not*
        lead to any editors to reload or any other editor manager actions. */
-    static void notifyFilesChangedInternally(const QStringList &files);
+    static void notifyFilesChangedInternally(const Utils::FilePaths &filePaths);
 
 signals:
     /* Used to notify e.g. the code model to update the given files. Does *not*
        lead to any editors to reload or any other editor manager actions. */
-    void filesChangedInternally(const QStringList &files);
+    void filesChangedInternally(const Utils::FilePaths &filePaths);
     /// emitted if all documents changed their name e.g. due to the file changing on disk
-    void allDocumentsRenamed(const QString &from, const QString &to);
+    void allDocumentsRenamed(const Utils::FilePath &from, const Utils::FilePath &to);
     /// emitted if one document changed its name e.g. due to save as
-    void documentRenamed(Core::IDocument *document, const QString &from, const QString &to);
+    void documentRenamed(Core::IDocument *document,
+                         const Utils::FilePath &from,
+                         const Utils::FilePath &to);
     void projectsDirectoryChanged(const Utils::FilePath &directory);
 
 private:
@@ -162,6 +162,8 @@ private:
     void checkForReload();
     void changedFile(const QString &file);
     void filePathChanged(const Utils::FilePath &oldName, const Utils::FilePath &newName);
+    void updateSaveAll();
+    static void registerSaveAllAction();
 
     friend class Core::Internal::MainWindow;
     friend class Core::Internal::DocumentManagerPrivate;
@@ -170,10 +172,10 @@ private:
 class CORE_EXPORT FileChangeBlocker
 {
 public:
-    explicit FileChangeBlocker(const QString &fileName);
+    explicit FileChangeBlocker(const Utils::FilePath &filePath);
     ~FileChangeBlocker();
 private:
-    const QString m_fileName;
+    const Utils::FilePath m_filePath;
     Q_DISABLE_COPY(FileChangeBlocker)
 };
 
