@@ -25,7 +25,6 @@
 
 #include "searchresultwindow.h"
 #include "searchresultwidget.h"
-#include "searchresultcolor.h"
 #include "textfindconstants.h"
 
 #include <coreplugin/icore.h>
@@ -111,13 +110,15 @@ namespace Internal {
         void moveWidgetToTop();
         void popupRequested(bool focus);
         void handleExpandCollapseToolButton(bool checked);
+        void updateFilterButton();
 
         SearchResultWindow *q;
         QList<Internal::SearchResultWidget *> m_searchResultWidgets;
         QToolButton *m_expandCollapseButton;
+        QToolButton *m_filterButton;
         QToolButton *m_newSearchButton;
         QAction *m_expandCollapseAction;
-        static const bool m_initiallyExpand = false;
+        static const bool m_initiallyExpand;
         QWidget *m_spacer;
         QLabel *m_historyLabel;
         QWidget *m_spacer2;
@@ -126,11 +127,13 @@ namespace Internal {
         QList<SearchResult *> m_searchResults;
         int m_currentIndex;
         QFont m_font;
-        SearchResultColor m_color;
+        SearchResultColors m_colors;
         int m_tabWidth;
 
     };
 
+    const bool SearchResultWindowPrivate::m_initiallyExpand = false;
+
     SearchResultWindowPrivate::SearchResultWindowPrivate(SearchResultWindow *window, QWidget *nsp) :
         q(window),
         m_expandCollapseButton(nullptr),
@@ -167,6 +170,11 @@ namespace Internal {
         cmd->setAttribute(Command::CA_UpdateText);
         m_expandCollapseButton->setDefaultAction(cmd->action());
 
+        m_filterButton = new QToolButton(m_widget);
+        m_filterButton->setText(tr("Filter Results"));
+        m_filterButton->setIcon(Utils::Icons::FILTER.icon());
+        m_filterButton->setEnabled(false);
+
         QAction *newSearchAction = new QAction(tr("New Search"), this);
         newSearchAction->setIcon(Utils::Icons::NEWSEARCH_TOOLBAR.icon());
         cmd = ActionManager::command(Constants::ADVANCED_FIND);
@@ -177,6 +185,11 @@ namespace Internal {
         connect(m_expandCollapseAction, &QAction::toggled,
                 this, &SearchResultWindowPrivate::handleExpandCollapseToolButton);
 
+        connect(m_filterButton, &QToolButton::clicked, this, [this] {
+            if (!isSearchVisible())
+                return;
+            m_searchResultWidgets.at(visibleSearchIndex())->showFilterWidget(m_filterButton);
+        });
     }
 
     void SearchResultWindowPrivate::setCurrentIndex(int index, bool focus)
@@ -199,16 +212,17 @@ namespace Internal {
             m_newSearchButton->setEnabled(true);
         }
         q->navigateStateChanged();
+        updateFilterButton();
     }
 
     void SearchResultWindowPrivate::moveWidgetToTop()
     {
         auto widget = qobject_cast<SearchResultWidget *>(sender());
         QTC_ASSERT(widget, return);
-        int index = m_searchResultWidgets.indexOf(widget);
+        const int index = m_searchResultWidgets.indexOf(widget);
         if (index == 0)
             return; // nothing to do
-        int internalIndex = index + 1/*account for "new search" entry*/;
+        const int internalIndex = index + 1/*account for "new search" entry*/;
         QString searchEntry = m_recentSearchesBox->itemText(internalIndex);
 
         m_searchResultWidgets.removeAt(index);
@@ -260,7 +274,9 @@ using namespace Core::Internal;
 
 /*!
     \class Core::SearchResult
+    \inheaderfile coreplugin/find/searchresultwindow.h
     \inmodule QtCreator
+
     \brief The SearchResult class reports user interaction, such as the
     activation of a search result item.
 
@@ -340,7 +356,9 @@ using namespace Core::Internal;
 
 /*!
     \class Core::SearchResultWindow
+    \inheaderfile coreplugin/find/searchresultwindow.h
     \inmodule QtCreator
+
     \brief The SearchResultWindow class is the implementation of a commonly
     shared \uicontrol{Search Results} output pane.
 
@@ -440,7 +458,7 @@ QWidget *SearchResultWindow::outputWidget(QWidget *)
 */
 QList<QWidget*> SearchResultWindow::toolBarWidgets() const
 {
-    return {d->m_expandCollapseButton, d->m_newSearchButton, d->m_spacer,
+    return {d->m_expandCollapseButton, d->m_filterButton, d->m_newSearchButton, d->m_spacer,
             d->m_historyLabel, d->m_spacer2, d->m_recentSearchesBox};
 }
 
@@ -480,17 +498,23 @@ SearchResult *SearchResultWindow::startNewSearch(const QString &label,
                                                  const QString &cfgGroup)
 {
     if (d->m_searchResults.size() >= MAX_SEARCH_HISTORY) {
+        if (d->m_currentIndex >= d->m_recentSearchesBox->count() - 1) {
+            // temporarily set the index to the last but one existing
+            d->m_currentIndex = d->m_recentSearchesBox->count() - 2;
+        }
         d->m_searchResultWidgets.last()->notifyVisibilityChanged(false);
         // widget first, because that might send interesting signals to SearchResult
         delete d->m_searchResultWidgets.takeLast();
         delete d->m_searchResults.takeLast();
         d->m_recentSearchesBox->removeItem(d->m_recentSearchesBox->count()-1);
-        if (d->m_currentIndex >= d->m_recentSearchesBox->count()) {
-            // temporarily set the index to the last existing
-            d->m_currentIndex = d->m_recentSearchesBox->count() - 1;
-        }
     }
     auto widget = new SearchResultWidget;
+    connect(widget, &SearchResultWidget::filterInvalidated, this, [this, widget] {
+        if (widget == d->m_searchResultWidgets.at(d->visibleSearchIndex()))
+            d->handleExpandCollapseToolButton(d->m_expandCollapseButton->isChecked());
+    });
+    connect(widget, &SearchResultWidget::filterChanged,
+            d, &SearchResultWindowPrivate::updateFilterButton);
     d->m_searchResultWidgets.prepend(widget);
     d->m_widget->insertWidget(1, widget);
     connect(widget, &SearchResultWidget::navigateStateChanged,
@@ -499,7 +523,7 @@ SearchResult *SearchResultWindow::startNewSearch(const QString &label,
             d, &SearchResultWindowPrivate::moveWidgetToTop);
     connect(widget, &SearchResultWidget::requestPopup,
             d, &SearchResultWindowPrivate::popupRequested);
-    widget->setTextEditorFont(d->m_font, d->m_color);
+    widget->setTextEditorFont(d->m_font, d->m_colors);
     widget->setTabWidth(d->m_tabWidth);
     widget->setSupportPreserveCase(preserveCaseMode == PreserveCaseEnabled);
     bool supportsReplace = searchOrSearchAndReplace != SearchOnly;
@@ -570,25 +594,12 @@ void SearchResultWindow::setFocus()
 /*!
     \internal
 */
-void SearchResultWindow::setTextEditorFont(const QFont &font,
-                                           const QColor &textForegroundColor,
-                                           const QColor &textBackgroundColor,
-                                           const QColor &highlightForegroundColor,
-                                           const QColor &highlightBackgroundColor)
+void SearchResultWindow::setTextEditorFont(const QFont &font, const SearchResultColors &colors)
 {
     d->m_font = font;
-    Internal::SearchResultColor color;
-    color.textBackground = textBackgroundColor;
-    color.textForeground = textForegroundColor;
-    color.highlightBackground = highlightBackgroundColor.isValid()
-            ? highlightBackgroundColor
-            : textBackgroundColor;
-    color.highlightForeground = highlightForegroundColor.isValid()
-            ? highlightForegroundColor
-            : textForegroundColor;
-    d->m_color = color;
+    d->m_colors = colors;
     foreach (Internal::SearchResultWidget *widget, d->m_searchResultWidgets)
-        widget->setTextEditorFont(font, color);
+        widget->setTextEditorFont(font, colors);
 }
 
 /*!
@@ -623,6 +634,12 @@ void SearchResultWindowPrivate::handleExpandCollapseToolButton(bool checked)
     }
 }
 
+void SearchResultWindowPrivate::updateFilterButton()
+{
+    m_filterButton->setEnabled(isSearchVisible()
+                               && m_searchResultWidgets.at(visibleSearchIndex())->hasFilter());
+}
+
 /*!
     \internal
 */
@@ -630,7 +647,8 @@ void SearchResultWindow::readSettings()
 {
     QSettings *s = ICore::settings();
     s->beginGroup(QLatin1String(SETTINGSKEYSECTIONNAME));
-    d->m_expandCollapseAction->setChecked(s->value(QLatin1String(SETTINGSKEYEXPANDRESULTS), d->m_initiallyExpand).toBool());
+    d->m_expandCollapseAction->setChecked(s->value(QLatin1String(SETTINGSKEYEXPANDRESULTS),
+                                                   SearchResultWindowPrivate::m_initiallyExpand).toBool());
     s->endGroup();
 }
 
@@ -639,9 +657,11 @@ void SearchResultWindow::readSettings()
 */
 void SearchResultWindow::writeSettings()
 {
-    QSettings *s = ICore::settings();
-    s->beginGroup(QLatin1String(SETTINGSKEYSECTIONNAME));
-    s->setValue(QLatin1String(SETTINGSKEYEXPANDRESULTS), d->m_expandCollapseAction->isChecked());
+    Utils::QtcSettings *s = ICore::settings();
+    s->beginGroup(SETTINGSKEYSECTIONNAME);
+    s->setValueWithDefault(SETTINGSKEYEXPANDRESULTS,
+                           d->m_expandCollapseAction->isChecked(),
+                           SearchResultWindowPrivate::m_initiallyExpand);
     s->endGroup();
 }
 
@@ -708,6 +728,8 @@ SearchResult::SearchResult(SearchResultWidget *widget)
     connect(widget, &SearchResultWidget::activated, this, &SearchResult::activated);
     connect(widget, &SearchResultWidget::replaceButtonClicked,
             this, &SearchResult::replaceButtonClicked);
+    connect(widget, &SearchResultWidget::replaceTextChanged,
+            this, &SearchResult::replaceTextChanged);
     connect(widget, &SearchResultWidget::cancelled, this, &SearchResult::cancelled);
     connect(widget, &SearchResultWidget::paused, this, &SearchResult::paused);
     connect(widget, &SearchResultWidget::visibilityChanged,
@@ -737,6 +759,11 @@ QVariant SearchResult::userData() const
     return m_userData;
 }
 
+bool SearchResult::supportsReplace() const
+{
+    return m_widget->supportsReplace();
+}
+
 /*!
     Returns the text that should replace the text in search results.
 */
@@ -770,48 +797,28 @@ QWidget *SearchResult::additionalReplaceWidget() const
 }
 
 /*!
-    Adds a single result line to the \uicontrol {Search Results} output pane.
-
-    \a fileName, \a lineNumber, and \a lineText are shown on the result line.
-    \a searchTermStart and \a searchTermLength specify the region that
-    should be visually marked (string position and length in \a lineText).
-    You can attach arbitrary \a userData to the search result, which can
-    be used, for example, when reacting to the signals of the search results
-    for your search.
-
-    \sa addResults()
+    Sets a \a widget as UI for a global search and replace action.
 */
-void SearchResult::addResult(const QString &fileName, int lineNumber, const QString &lineText,
-                             int searchTermStart, int searchTermLength, const QVariant &userData)
+void SearchResult::setAdditionalReplaceWidget(QWidget *widget)
 {
-    Search::TextRange mainRange;
-    mainRange.begin.line = lineNumber;
-    mainRange.begin.column = searchTermStart;
-    mainRange.end.line = mainRange.begin.line;
-    mainRange.end.column = mainRange.begin.column + searchTermLength;
-
-    m_widget->addResult(fileName, lineText, mainRange, userData);
+    m_widget->setAdditionalReplaceWidget(widget);
 }
 
 /*!
     Adds a single result line to the \uicontrol {Search Results} output pane.
 
-    \a mainRange specifies the region from the beginning of the search term
+    \a {item}.mainRange() specifies the region from the beginning of the search term
     through its length that should be visually marked.
-    \a fileName and \a lineText are shown on the result line.
-    You can attach arbitrary \a userData to the search result, which can
+    \a {item}.path(), \a {item}.text() are shown on the result line.
+    You can attach arbitrary \a {item}.userData() to the search result, which can
     be used, for example, when reacting to the signals of the search results
     for your search.
 
     \sa addResults()
 */
-void SearchResult::addResult(const QString &fileName,
-                             const QString &lineText,
-                             Search::TextRange mainRange,
-                             const QVariant &userData)
+void SearchResult::addResult(const SearchResultItem &item)
 {
-    m_widget->addResult(fileName, lineText, mainRange, userData);
-    emit countChanged(m_widget->count());
+    m_widget->addResults({item}, AddOrdered);
 }
 
 /*!
@@ -826,6 +833,11 @@ void SearchResult::addResults(const QList<SearchResultItem> &items, AddMode mode
     emit countChanged(m_widget->count());
 }
 
+void SearchResult::setFilter(SearchResultFilter *filter)
+{
+    m_widget->setFilter(filter);
+}
+
 /*!
     Notifies the \uicontrol {Search Results} output pane that the current search
     has been \a canceled, and the UI should reflect that.
@@ -844,6 +856,14 @@ void SearchResult::setTextToReplace(const QString &textToReplace)
     m_widget->setTextToReplace(textToReplace);
 }
 
+/*!
+    Sets whether replace is enabled and can be triggered by the user
+*/
+void SearchResult::setReplaceEnabled(bool enabled)
+{
+    m_widget->setReplaceEnabled(enabled);
+}
+
 /*!
  * Removes all search results.
  */
