@@ -25,6 +25,7 @@
 
 #include "filesystemfilter.h"
 
+#include "basefilefilter.h"
 #include "locatorwidget.h"
 
 #include <coreplugin/coreconstants.h>
@@ -33,19 +34,22 @@
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
+#include <coreplugin/vcsmanager.h>
 #include <utils/checkablemessagebox.h>
 #include <utils/fileutils.h>
+#include <utils/link.h>
 
 #include <QDir>
+#include <QJsonObject>
 #include <QPushButton>
 #include <QRegularExpression>
-#include <QTimer>
 
 using namespace Core;
 using namespace Core::Internal;
+using namespace Utils;
 
 ILocatorFilter::MatchLevel FileSystemFilter::matchLevelFor(const QRegularExpressionMatch &match,
-                                                           const QString &matchText) const
+                                                           const QString &matchText)
 {
     const int consecutivePos = match.capturedStart(1);
     if (consecutivePos == 0)
@@ -64,14 +68,18 @@ FileSystemFilter::FileSystemFilter()
 {
     setId("Files in file system");
     setDisplayName(tr("Files in File System"));
-    setShortcutString("f");
-    setIncludedByDefault(false);
+    setDescription(tr("Opens a file given by a relative path to the current document, or absolute "
+                      "path. \"~\" refers to your home directory. You have the option to create a "
+                      "file if it does not exist yet."));
+    setDefaultShortcutString("f");
+    setDefaultIncludedByDefault(false);
 }
 
 void FileSystemFilter::prepareSearch(const QString &entry)
 {
     Q_UNUSED(entry)
     m_currentDocumentDirectory = DocumentManager::fileDialogInitialDirectory();
+    m_currentIncludeHidden = m_includeHidden;
 }
 
 QList<LocatorFilterEntry> FileSystemFilter::matchesFor(QFutureInterface<LocatorFilterEntry> &future,
@@ -90,7 +98,7 @@ QList<LocatorFilterEntry> FileSystemFilter::matchesFor(QFutureInterface<LocatorF
     const QDir dirInfo(directory);
     QDir::Filters dirFilter = QDir::Dirs|QDir::Drives|QDir::NoDot|QDir::NoDotDot;
     QDir::Filters fileFilter = QDir::Files;
-    if (m_includeHidden) {
+    if (m_currentIncludeHidden) {
         dirFilter |= QDir::Hidden;
         fileFilter |= QDir::Hidden;
     }
@@ -115,18 +123,18 @@ QList<LocatorFilterEntry> FileSystemFilter::matchesFor(QFutureInterface<LocatorF
             const MatchLevel level = matchLevelFor(match, dir);
             const QString fullPath = dirInfo.filePath(dir);
             LocatorFilterEntry filterEntry(this, dir, QVariant());
-            filterEntry.fileName = fullPath;
+            filterEntry.filePath = FilePath::fromString(fullPath);
             filterEntry.highlightInfo = highlightInfo(match);
 
             entries[int(level)].append(filterEntry);
         }
     }
     // file names can match with +linenumber or :linenumber
-    const EditorManager::FilePathInfo fp = EditorManager::splitLineAndColumnNumber(entryFileName);
-    regExp = createRegExp(fp.filePath, caseSensitivity_);
+    QString postfix;
+    Link link = Link::fromString(entryFileName, true, &postfix);
+    regExp = createRegExp(link.targetFilePath.toString(), caseSensitivity_);
     if (!regExp.isValid())
         return {};
-    const QString fileName = QFileInfo(fp.filePath).fileName();
     for (const QString &file : files) {
         if (future.isCanceled())
             break;
@@ -135,8 +143,8 @@ QList<LocatorFilterEntry> FileSystemFilter::matchesFor(QFutureInterface<LocatorF
         if (match.hasMatch()) {
             const MatchLevel level = matchLevelFor(match, file);
             const QString fullPath = dirInfo.filePath(file);
-            LocatorFilterEntry filterEntry(this, file, QString(fullPath + fp.postfix));
-            filterEntry.fileName = fullPath;
+            LocatorFilterEntry filterEntry(this, file, QString(fullPath + postfix));
+            filterEntry.filePath = FilePath::fromString(fullPath);
             filterEntry.highlightInfo = highlightInfo(match);
 
             entries[int(level)].append(filterEntry);
@@ -144,11 +152,15 @@ QList<LocatorFilterEntry> FileSystemFilter::matchesFor(QFutureInterface<LocatorF
     }
 
     // "create and open" functionality
-    const QString fullFilePath = dirInfo.filePath(fileName);
+    const QString fullFilePath = dirInfo.filePath(entryFileName);
     const bool containsWildcard = entry.contains('?') || entry.contains('*');
     if (!containsWildcard && !QFileInfo::exists(fullFilePath) && dirInfo.exists()) {
-        LocatorFilterEntry createAndOpen(this, tr("Create and Open \"%1\"").arg(entry), fullFilePath);
-        createAndOpen.extraInfo = Utils::FilePath::fromString(dirInfo.absolutePath()).shortNativePath();
+        LocatorFilterEntry createAndOpen(this,
+                                         tr("Create and Open \"%1\"").arg(entry),
+                                         fullFilePath);
+        createAndOpen.filePath = FilePath::fromString(fullFilePath);
+        createAndOpen.extraInfo = Utils::FilePath::fromString(dirInfo.absolutePath())
+                                      .shortNativePath();
         entries[int(MatchLevel::Normal)].append(createAndOpen);
     }
 
@@ -163,8 +175,7 @@ void FileSystemFilter::accept(LocatorFilterEntry selection,
                               int *selectionLength) const
 {
     Q_UNUSED(selectionLength)
-    QString fileName = selection.fileName;
-    QFileInfo info(fileName);
+    QFileInfo info = selection.filePath.toFileInfo();
     if (info.isDir()) {
         const QString value = shortcutString() + ' '
                 + QDir::toNativeSeparators(info.absoluteFilePath() + '/');
@@ -172,7 +183,7 @@ void FileSystemFilter::accept(LocatorFilterEntry selection,
         *selectionStart = value.length();
     } else {
         // Don't block locator filter execution with dialog
-        QTimer::singleShot(0, EditorManager::instance(), [info, selection] {
+        QMetaObject::invokeMethod(EditorManager::instance(), [info, selection] {
             const QString targetFile = selection.internalData.toString();
             if (!info.exists()) {
                 if (Utils::CheckableMessageBox::shouldAskAgain(ICore::settings(), kAlwaysCreate)) {
@@ -198,13 +209,10 @@ void FileSystemFilter::accept(LocatorFilterEntry selection,
                 QFile file(targetFile);
                 file.open(QFile::WriteOnly);
                 file.close();
+                VcsManager::promptToAdd(QFileInfo(targetFile).absolutePath(), { targetFile });
             }
-            const QFileInfo fileInfo(targetFile);
-            const QString cleanedFilePath = QDir::cleanPath(fileInfo.absoluteFilePath());
-            EditorManager::openEditor(cleanedFilePath,
-                                      Id(),
-                                      EditorManager::CanContainLineAndColumnNumber);
-        });
+            BaseFileFilter::openEditorAt(selection);
+        }, Qt::QueuedConnection);
     }
 }
 
@@ -232,28 +240,36 @@ bool FileSystemFilter::openConfigDialog(QWidget *parent, bool &needsRefresh)
     return false;
 }
 
-QByteArray FileSystemFilter::saveState() const
+const char kIncludeHiddenKey[] = "includeHidden";
+
+void FileSystemFilter::saveState(QJsonObject &object) const
 {
-    QByteArray value;
-    QDataStream out(&value, QIODevice::WriteOnly);
-    out << m_includeHidden;
-    out << shortcutString();
-    out << isIncludedByDefault();
-    return value;
+    if (m_includeHidden != kIncludeHiddenDefault)
+        object.insert(kIncludeHiddenKey, m_includeHidden);
+}
+
+void FileSystemFilter::restoreState(const QJsonObject &object)
+{
+    m_currentIncludeHidden = object.value(kIncludeHiddenKey).toBool(kIncludeHiddenDefault);
 }
 
 void FileSystemFilter::restoreState(const QByteArray &state)
 {
-    QDataStream in(state);
-    in >> m_includeHidden;
+    if (isOldSetting(state)) {
+        // TODO read old settings, remove some time after Qt Creator 4.15
+        QDataStream in(state);
+        in >> m_includeHidden;
 
-    // An attempt to prevent setting this on old configuration
-    if (!in.atEnd()) {
-        QString shortcut;
-        bool defaultFilter;
-        in >> shortcut;
-        in >> defaultFilter;
-        setShortcutString(shortcut);
-        setIncludedByDefault(defaultFilter);
+        // An attempt to prevent setting this on old configuration
+        if (!in.atEnd()) {
+            QString shortcut;
+            bool defaultFilter;
+            in >> shortcut;
+            in >> defaultFilter;
+            setShortcutString(shortcut);
+            setIncludedByDefault(defaultFilter);
+        }
+    } else {
+        ILocatorFilter::restoreState(state);
     }
 }
