@@ -28,13 +28,18 @@
 #include "cppeditorplugin.h"
 #include "cppeditortestcase.h"
 
+#include <cpptools/cppcodemodelsettings.h>
 #include <cpptools/cppelementevaluator.h>
 #include <cpptools/cppfollowsymbolundercursor.h>
 #include <cpptools/cppvirtualfunctionassistprovider.h>
 #include <cpptools/cppvirtualfunctionproposalitem.h>
+#include <cpptools/cpptoolsreuse.h>
 #include <cpptools/cpptoolstestcase.h>
 #include <cpptools/cppmodelmanager.h>
 
+#include <projectexplorer/kitmanager.h>
+#include <projectexplorer/projectexplorer.h>
+
 #include <texteditor/codeassist/genericproposalmodel.h>
 #include <texteditor/codeassist/iassistprocessor.h>
 #include <texteditor/codeassist/iassistproposal.h>
@@ -76,6 +81,7 @@ using namespace CPlusPlus;
 using namespace CppTools;
 using namespace TextEditor;
 using namespace Core;
+using namespace ProjectExplorer;
 
 class OverrideItem {
 public:
@@ -250,7 +256,7 @@ public:
 
     F2TestCase(CppEditorAction action,
                const QList<TestDocumentPtr> &testFiles,
-               const OverrideItemList &expectedVirtualFunctionProposal = OverrideItemList());
+               OverrideItemList expectedVirtualFunctionProposal = OverrideItemList());
 
 private:
     static TestDocumentPtr testFileWithInitialCursorMarker(const QList<TestDocumentPtr> &testFiles);
@@ -263,10 +269,12 @@ private:
 /// It can be the same document.
 F2TestCase::F2TestCase(CppEditorAction action,
                        const QList<TestDocumentPtr> &testFiles,
-                       const OverrideItemList &expectedVirtualFunctionProposal)
+                       OverrideItemList expectedVirtualFunctionProposal)
 {
     QVERIFY(succeededSoFar());
 
+    setUseClangd();
+
     // Check if there are initial and target position markers
     TestDocumentPtr initialTestFile = testFileWithInitialCursorMarker(testFiles);
     QVERIFY2(initialTestFile,
@@ -275,13 +283,62 @@ F2TestCase::F2TestCase(CppEditorAction action,
     QVERIFY2(targetTestFile,
         "No test file with target cursor marker is provided.");
 
+    const QString curTestName = QLatin1String(QTest::currentTestFunction());
+    const QString tag = QLatin1String(QTest::currentDataTag());
+    const bool useClangd = CppTools::ClangdSettings::instance().useClangd();
+    if (useClangd) {
+        if (curTestName == "test_FollowSymbolUnderCursor_QObject_connect"
+                || curTestName == "test_FollowSymbolUnderCursor_QObject_oldStyleConnect") {
+            QSKIP("TODO: Implement fall-back");
+        }
+        if (curTestName == "test_FollowSymbolUnderCursor_classOperator" && tag == "backward")
+            QSKIP("clangd goes to operator name first");
+        if (tag.toLower().contains("fuzzy"))
+            QSKIP("fuzzy matching is not supposed to work with clangd"); // TODO: Implement fallback as we do with libclang
+        if (tag == "baseClassFunctionIntroducedByUsingDeclaration")
+            QSKIP("clangd points to the using declaration");
+        if (tag == "classDestructor" || tag == "fromDestructorDefinitionSymbol"
+                || tag == "fromDestructorBody") {
+            QSKIP("clangd wants the cursor before the ~ character");
+        }
+        if (curTestName == "test_FollowSymbolUnderCursor_classOperator_inOp")
+            QSKIP("clangd goes to operator name first");
+    }
+
     // Write files to disk
     CppTools::Tests::TemporaryDir temporaryDir;
     QVERIFY(temporaryDir.isValid());
+    QString projectFileContent = "CppApplication { files: [";
     foreach (TestDocumentPtr testFile, testFiles) {
         QVERIFY(testFile->baseDirectory().isEmpty());
         testFile->setBaseDirectory(temporaryDir.path());
         QVERIFY(testFile->writeToDisk());
+        projectFileContent += QString::fromLatin1("\"%1\",").arg(testFile->filePath());
+    }
+    projectFileContent += "]}\n";
+
+    class ProjectCloser {
+    public:
+        void setProject(Project *p) { m_p = p; }
+        ~ProjectCloser() { if (m_p) ProjectExplorerPlugin::unloadProject(m_p); }
+    private:
+        Project * m_p = nullptr;
+    } projectCloser;
+
+    if (useClangd) {
+        TestDocument projectFile(projectFileContent.toUtf8(), "project.qbs");
+        projectFile.setBaseDirectory(temporaryDir.path());
+        QVERIFY(projectFile.writeToDisk());
+        const auto openProjectResult = ProjectExplorerPlugin::openProject(projectFile.filePath());
+        QVERIFY2(openProjectResult && openProjectResult.project(),
+                 qPrintable(openProjectResult.errorMessage()));
+        projectCloser.setProject(openProjectResult.project());
+        openProjectResult.project()->configureAsExampleProject(
+                    CppEditorPlugin::instance()->m_testKit);
+
+        // Wait until project is fully indexed.
+        QVERIFY(CppTools::Tests::waitForSignalOrTimeout(openProjectResult.project(),
+                &Project::indexingFinished, CppTools::Tests::clangdIndexingTimeout()));
     }
 
     // Update Code Model
@@ -294,7 +351,8 @@ F2TestCase::F2TestCase(CppEditorAction action,
     foreach (TestDocumentPtr testFile, testFiles) {
         QVERIFY(openCppEditor(testFile->filePath(), &testFile->m_editor,
                               &testFile->m_editorWidget));
-        closeEditorAtEndOfTestCase(testFile->m_editor);
+        if (!useClangd) // Editors get closed when unloading project.
+            closeEditorAtEndOfTestCase(testFile->m_editor);
 
         // Wait until the indexer processed the just opened file.
         // The file is "Full Checked" since it is in the working copy now,
@@ -303,13 +361,16 @@ F2TestCase::F2TestCase(CppEditorAction action,
             const Document::Ptr document = waitForFileInGlobalSnapshot(testFile->filePath());
             QVERIFY(document);
             if (document->checkMode() == Document::FullCheck) {
+                if (!document->diagnosticMessages().isEmpty())
+                    qDebug() << document->diagnosticMessages().first().text();
                 QVERIFY(document->diagnosticMessages().isEmpty());
                 break;
             }
         }
 
         // Rehighlight
-        waitForRehighlightedSemanticDocument(testFile->m_editorWidget);
+        if (!useClangd)
+            waitForRehighlightedSemanticDocument(testFile->m_editorWidget);
     }
 
     // Activate editor of initial test file
@@ -329,15 +390,9 @@ F2TestCase::F2TestCase(CppEditorAction action,
         FollowSymbolInterface &delegate = CppModelManager::instance()->followSymbolInterface();
         auto* builtinFollowSymbol = dynamic_cast<FollowSymbolUnderCursor *>(&delegate);
         if (!builtinFollowSymbol) {
-            if (filePaths.size() > 1)
-                QSKIP("Clang FollowSymbol does not currently support multiple files (except cpp+header)");
-            const QString curTestName = QLatin1String(QTest::currentTestFunction());
-            if (curTestName == "test_FollowSymbolUnderCursor_QObject_connect"
-                    || curTestName == "test_FollowSymbolUnderCursor_virtualFunctionCall"
-                    || curTestName == "test_FollowSymbolUnderCursor_QTCREATORBUG7903") {
+            if (curTestName == "test_FollowSymbolUnderCursor_QTCREATORBUG7903")
                 QSKIP((curTestName + " is not supported by Clang FollowSymbol").toLatin1());
-            }
-
+            widget->inTestMode = true;
             widget->openLinkUnderCursor();
             break;
         }
@@ -358,14 +413,49 @@ F2TestCase::F2TestCase(CppEditorAction action,
         break;
     }
     case SwitchBetweenMethodDeclarationDefinitionAction:
-        CppEditorPlugin::instance()->switchDeclarationDefinition();
+        // Some test cases were erroneously added as decl/def, but they are really
+        // follow symbol functionality (in commit a0764603d0).
+        if (useClangd && tag.endsWith("Var"))
+            initialTestFile->m_editorWidget->openLinkUnderCursor();
+        else
+            CppEditorPlugin::instance()->switchDeclarationDefinition();
         break;
     default:
         QFAIL("Unknown test action");
         break;
     }
 
-    QCoreApplication::processEvents();
+    if (useClangd) {
+        QEXPECT_FAIL("infiniteLoopLocalTypedef_QTCREATORBUG-11999",
+                     "clangd bug: Go to definition does not return", Abort);
+        if (expectedVirtualFunctionProposal.size() <= 1) {
+            QVERIFY(CppTools::Tests::waitForSignalOrTimeout(EditorManager::instance(),
+                                                            &EditorManager::linkOpened, 10000));
+        } else {
+            QTimer t;
+            QEventLoop l;
+            t.setSingleShot(true);
+            QObject::connect(&t, &QTimer::timeout, &l, &QEventLoop::quit);
+            const IAssistProposal *immediateProposal = nullptr;
+            const IAssistProposal *finalProposal = nullptr;
+            QObject::connect(initialTestFile->m_editorWidget, &CppEditorWidget::proposalsReady,
+                             [&](const IAssistProposal *i, const IAssistProposal *f) {
+                immediateProposal = i;
+                finalProposal = f;
+                l.quit();
+            });
+            t.start(10000);
+            l.exec();
+            QVERIFY(immediateProposal);
+            QVERIFY(finalProposal);
+            immediateVirtualSymbolResults = VirtualFunctionTestAssistProvider::itemList(
+                        immediateProposal->model());
+            finalVirtualSymbolResults = VirtualFunctionTestAssistProvider::itemList(
+                        finalProposal->model());
+        }
+    } else {
+        QCoreApplication::processEvents();
+    }
 
     // Compare
     IEditor *currentEditor = EditorManager::currentEditor();
@@ -374,24 +464,52 @@ F2TestCase::F2TestCase(CppEditorAction action,
 
     QCOMPARE(currentTextEditor->document()->filePath().toString(), targetTestFile->filePath());
     int expectedLine, expectedColumn;
-    currentTextEditor->convertPosition(targetTestFile->m_targetCursorPosition,
-                                       &expectedLine, &expectedColumn);
+    if (useClangd && expectedVirtualFunctionProposal.size() == 1) {
+        expectedLine = expectedVirtualFunctionProposal.first().line;
+        expectedColumn = -1;
+        expectedVirtualFunctionProposal.clear();
+    } else {
+        currentTextEditor->convertPosition(targetTestFile->m_targetCursorPosition,
+                                           &expectedLine, &expectedColumn);
+    }
 //    qDebug() << "Expected line:" << expectedLine;
 //    qDebug() << "Expected column:" << expectedColumn;
 
-    QEXPECT_FAIL("globalVarFromEnum", "Contributor works on a fix.", Abort);
-    QEXPECT_FAIL("matchFunctionSignature_Follow_5", "foo(int) resolved as CallAST", Abort);
+    if (!CppTools::ClangdSettings::instance().useClangd()) {
+        QEXPECT_FAIL("globalVarFromEnum", "Contributor works on a fix.", Abort);
+        QEXPECT_FAIL("matchFunctionSignature_Follow_5", "foo(int) resolved as CallAST", Abort);
+    }
+
     QCOMPARE(currentTextEditor->currentLine(), expectedLine);
-    QCOMPARE(currentTextEditor->currentColumn(), expectedColumn);
+    if (expectedColumn != -1)
+        QCOMPARE(currentTextEditor->currentColumn(), expectedColumn);
 
 //    qDebug() << immediateVirtualSymbolResults;
 //    qDebug() << finalVirtualSymbolResults;
     OverrideItemList expectedImmediate;
     if (!expectedVirtualFunctionProposal.isEmpty()) {
-        expectedImmediate << expectedVirtualFunctionProposal.first();
-        expectedImmediate << OverrideItem(QLatin1String("...searching overrides"));
+        bool offerBaseDecl = true;
+        if (useClangd) {
+            if (tag == "allOverrides from base declaration") {
+                expectedVirtualFunctionProposal.removeFirst();
+                offerBaseDecl = false;
+            }
+        }
+        if (offerBaseDecl) {
+            OverrideItem first = expectedVirtualFunctionProposal.first();
+            if (useClangd)
+                first.text = "<base declaration>";
+            expectedImmediate << first;
+        }
+        expectedImmediate << OverrideItem(QLatin1String("collecting overrides ..."));
     }
     QCOMPARE(immediateVirtualSymbolResults, expectedImmediate);
+    if (useClangd) {
+        QEXPECT_FAIL("noSiblings_references", "FIXME: too many results", Abort);
+        QEXPECT_FAIL("noSiblings_pointers", "FIXME: too many results", Abort);
+        QEXPECT_FAIL("noSiblings_noBaseExpression", "FIXME: too many results", Abort);
+    }
+    QCOMPARE(finalVirtualSymbolResults.size(), expectedVirtualFunctionProposal.size());
     QCOMPARE(finalVirtualSymbolResults, expectedVirtualFunctionProposal);
 }
 
@@ -421,6 +539,24 @@ Q_DECLARE_METATYPE(QList<CppEditor::Internal::TestDocumentPtr>)
 namespace CppEditor {
 namespace Internal {
 
+void CppEditorPlugin::initTestCase()
+{
+    const QString clangdFromEnv = qEnvironmentVariable("QTC_CLANGD");
+    if (clangdFromEnv.isEmpty())
+        return;
+    ClangdSettings::setClangdFilePath(Utils::FilePath::fromString(clangdFromEnv));
+    const auto clangd = ClangdSettings::instance().clangdFilePath();
+    if (clangd.isEmpty() || !clangd.exists())
+        return;
+
+    // Find suitable kit.
+    m_testKit = Utils::findOr(KitManager::kits(), nullptr, [](const Kit *k) {
+        return k->isValid();
+    });
+    if (!m_testKit)
+        QSKIP("This test requires at least one kit to be present");
+}
+
 void CppEditorPlugin::test_SwitchMethodDeclarationDefinition_data()
 {
     QTest::addColumn<QByteArray>("header");
@@ -486,6 +622,126 @@ void CppEditorPlugin::test_SwitchMethodDeclarationDefinition_data()
         "}\n"                   // Line 10
     );
 
+    QTest::newRow("fromConstructorDeclarationSymbol") << _(
+        "class C\n"
+        "{\n"
+        "public:\n"
+        "    C();\n"
+        "    int @function();\n"  // Line 5
+        "};\n"
+        ) << _(
+        "#include \"file.h\"\n"
+        "\n"
+        "C::C()\n"
+        "{\n"
+        "}\n"                   // Line 5
+        "\n"
+        "int C::$function()\n"
+        "{\n"
+        "    return 1 + 1;\n"
+        "}\n"                   // Line 10
+        );
+
+    QTest::newRow("fromConstructorDefinitionSymbol") << _(
+        "class C\n"
+        "{\n"
+        "public:\n"
+        "    $C();\n"
+        "    int function();\n"
+        "};\n"
+        ) << _(
+        "#include \"file.h\"\n"
+        "\n"
+        "C::@C()\n"
+        "{\n"
+        "}\n"
+        "\n"
+        "int C::function()\n"
+        "{\n"
+        "    return 1 + 1;\n"
+        "}\n"
+        );
+
+    QTest::newRow("fromConstructorBody") << _(
+        "class C\n"
+        "{\n"
+        "public:\n"
+        "    $C();\n"
+        "    int function();\n"
+        "};\n"
+        ) << _(
+        "#include \"file.h\"\n"
+        "\n"
+        "C::C()\n"
+        "{@\n"
+        "}\n"                   // Line 5
+        "\n"
+        "int C::function()\n"
+        "{\n"
+        "    return 1 + 1;\n"
+        "}\n"                   // Line 10
+        );
+
+    QTest::newRow("fromDestructorDeclarationSymbol") << _(
+        "class C\n"
+        "{\n"
+        "public:\n"
+        "    @C();\n"
+        "    int function();\n"  // Line 5
+        "};\n"
+        ) << _(
+        "#include \"file.h\"\n"
+        "\n"
+        "C::$C()\n"
+        "{\n"
+        "}\n"                   // Line 5
+        "\n"
+        "int C::function()\n"
+        "{\n"
+        "    return 1 + 1;\n"
+        "}\n"                   // Line 10
+        );
+
+    QTest::newRow("fromDestructorDefinitionSymbol") << _(
+        "class C\n"
+        "{\n"
+        "public:\n"
+        "    ~$C();\n"
+        "    int function();\n"
+        "};\n"
+        ) << _(
+        "#include \"file.h\"\n"
+        "\n"
+        "C::@~C()\n"
+        "{\n"
+        "}\n"
+        "\n"
+        "int C::function()\n"
+        "{\n"
+        "    return 1 + 1;\n"
+        "}\n"
+        );
+
+    QTest::newRow("fromDestructorBody") << _(
+        "class C\n"
+        "{\n"
+        "public:\n"
+        "    ~$C();\n"
+        "    int function();\n"
+        "};\n"
+        ) << _(
+        "#include \"file.h\"\n"
+        "\n"
+        "C::~C()\n"
+        "{@\n"
+        "}\n"                   // Line 5
+        "\n"
+        "int C::function()\n"
+        "{\n"
+        "    return 1 + 1;\n"
+        "}\n"                   // Line 10
+        );
+
     QTest::newRow("fromReturnType") << _(
         "class C\n"
         "{\n"
@@ -576,9 +832,21 @@ void CppEditorPlugin::test_SwitchMethodDeclarationDefinition_data()
                  "int OtherClass::var;\n"
                  "namespace NS {\n"
                  "int OtherClass::var;\n"
-                 "float Test::var;\n"
                  "int Test::$var;\n"
                  "}\n");
+
+    QTest::newRow("conversionOperatorDecl2Def")
+            << _("struct Foo {\n"
+                 "    operator @int() const;\n"
+                 "};\n")
+            << _("#include \"file.h\"\n"
+                 "Foo::$operator int() const { return {}; }\n");
+    QTest::newRow("conversionOperatorDef2Decl")
+            << _("struct Foo {\n"
+                 "    $operator int() const;\n"
+                 "};\n")
+            << _("#include \"file.h\"\n"
+                 "Foo::@operator int() const { return {}; }\n");
 }
 
 void CppEditorPlugin::test_SwitchMethodDeclarationDefinition()
@@ -935,6 +1203,16 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_data()
         "void Foo::foo(int) {}\n"
     );
 
+    QTest::newRow("matchFunctionSignature_Follow_3.5") << _(
+        "void foo(int);\n"
+        "void @$foo() {}\n"
+    );
+
+    QTest::newRow("matchFunctionSignature_Follow_3.6") << _(
+        "void foo(int);\n"
+        "void @$foo(double) {}\n"
+    );
+
     QTest::newRow("matchFunctionSignature_Follow_4") << _(
         "class Foo {\n"
         "    void foo(int);\n"
@@ -963,17 +1241,33 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_data()
         "void Foo::@foo(int) {}\n"
     );
 
-    QTest::newRow("matchFunctionSignature_Follow_8") << _(
+    QTest::newRow("matchFunctionSignature_Follow_8_fuzzy") << _(
         "class Foo {\n"
-        "    void @$foo(int *);\n"
+        "    void @foo(int *);\n"
         "};\n"
-        "void Foo::foo(const int *) {}\n"
+        "void Foo::$foo(const int *) {}\n"
     );
 
-    QTest::newRow("matchFunctionSignature_Follow_9") << _(
+    QTest::newRow("matchFunctionSignature_Follow_8_exact") << _(
         "class Foo {\n"
-        "    void @$foo(int&);\n"
+        "    void @foo(int *);\n"
         "};\n"
+        "void Foo::foo(const int *) {}\n"
+        "void Foo::$foo(int *) {}\n"
+    );
+
+    QTest::newRow("matchFunctionSignature_Follow_9_fuzzy") << _(
+        "class Foo {\n"
+        "    void @foo(int&);\n"
+        "};\n"
+        "void Foo::$foo(const int&) {}\n"
+    );
+
+    QTest::newRow("matchFunctionSignature_Follow_9_exact") << _(
+        "class Foo {\n"
+        "    void @foo(int&);\n"
+        "};\n"
+        "void Foo::$foo(int&) {}\n"
         "void Foo::foo(const int&) {}\n"
     );
 
@@ -1171,6 +1465,48 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_multipleDocuments_data()
                                 "foo.cpp")
     );
 
+    QTest::newRow("matchFunctionSignatureFuzzy1Forward") << (QList<TestDocumentPtr>()
+        << TestDocument::create("class Foo {\n"
+                                "    void @foo(int);\n"
+                                "    void foo();\n"
+                                "};\n",
+                                "foo.h")
+        << TestDocument::create("#include \"foo.h\"\n"
+                                "void Foo::$foo() {}\n",
+                                "foo.cpp")
+    );
+
+    QTest::newRow("matchFunctionSignatureFuzzy1Backward") << (QList<TestDocumentPtr>()
+        << TestDocument::create("class Foo {\n"
+                                "    void $foo(int);\n"
+                                "};\n",
+                                "foo.h")
+        << TestDocument::create("#include \"foo.h\"\n"
+                                "void Foo::@foo() {}\n",
+                                "foo.cpp")
+    );
+
+    QTest::newRow("matchFunctionSignatureFuzzy2Forward") << (QList<TestDocumentPtr>()
+        << TestDocument::create("class Foo {\n"
+                                "    void foo(int);\n"
+                                "    void @foo();\n"
+                                "};\n",
+                                "foo.h")
+        << TestDocument::create("#include \"foo.h\"\n"
+                                "void Foo::$foo(int) {}\n",
+                                "foo.cpp")
+    );
+
+    QTest::newRow("matchFunctionSignatureFuzzy2Backward") << (QList<TestDocumentPtr>()
+        << TestDocument::create("class Foo {\n"
+                                "    void $foo();\n"
+                                "};\n",
+                                "foo.h")
+        << TestDocument::create("#include \"foo.h\"\n"
+                                "void Foo::@foo(int) {}\n",
+                                "foo.cpp")
+    );
+
     QTest::newRow("globalVar") << QList<TestDocumentPtr>{
             TestDocument::create("namespace NS { extern int @globalVar; }\n", "file.h"),
             TestDocument::create(
@@ -1195,7 +1531,6 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_multipleDocuments_data()
             "int OtherClass::var;\n"
             "namespace NS {\n"
             "int OtherClass::var;\n"
-            "float Test::var;\n"
             "int Test::$var;\n"
             "}\n", "file.cpp")};
 }
@@ -1309,6 +1644,26 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_QObject_connect()
     F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
 }
 
+void CppEditorPlugin::test_FollowSymbolUnderCursor_QObject_oldStyleConnect()
+{
+    const QByteArray source =
+            "class O : public QObject {\n"
+            "    Q_OBJECT\n"
+            "signals:\n"
+            "    void $theSignal();\n"
+            "};\n"
+            "struct S {\n"
+            "    O* o();\n"
+            "};\n"
+            "int main()\n"
+            "{\n"
+            "    S s;\n"
+            "    QObject::connect(s.o(), SIGNAL(@theSignal()), s.o(), SIGNAL(theSignal()));\n"
+            "}\n";
+
+    F2TestCase(F2TestCase::FollowSymbolUnderCursorAction, singleDocument(source));
+}
+
 void CppEditorPlugin::test_FollowSymbolUnderCursor_classOperator_onOperatorToken_data()
 {
     QTest::addColumn<bool>("toDeclaration");
@@ -1413,6 +1768,31 @@ void CppEditorPlugin::test_FollowSymbolUnderCursor_virtualFunctionCall_data()
             << OverrideItem(QLatin1String("CD1::virt"), 11)
             << OverrideItem(QLatin1String("CD2::virt"), 14));
 
+    /// Check: Cursor on unimplemented base declaration.
+    QTest::newRow("allOverrides from base declaration") << _(
+            "struct A { virtual void $@virt() = 0; };\n"
+            "\n"
+            "struct B : A { void virt(); };\n"
+            "void B::virt() {}\n"
+            "\n"
+            "struct C : B { void virt(); };\n"
+            "void C::virt() {}\n"
+            "\n"
+            "struct CD1 : C { void virt(); };\n"
+            "void CD1::virt() {}\n"
+            "\n"
+            "struct CD2 : C { void virt(); };\n"
+            "void CD2::virt() {}\n"
+            "\n"
+            "int f(A *o) { o->virt(); }\n"
+            "\n")
+        << (OverrideItemList()
+            << OverrideItem(QLatin1String("A::virt = 0"), 1)
+            << OverrideItem(QLatin1String("B::virt"), 4)
+            << OverrideItem(QLatin1String("C::virt"), 7)
+            << OverrideItem(QLatin1String("CD1::virt"), 10)
+            << OverrideItem(QLatin1String("CD2::virt"), 13));
+
     /// Check: Static type is derived class pointer, only overrides of sub classes are presented.
     QTest::newRow("possibleOverrides1") << _(
             "struct A { virtual void virt() = 0; };\n"
