@@ -29,7 +29,6 @@
 #include <cplusplus/Lexer.h>
 
 #include <utils/qtcassert.h>
-#include <utils/qtcfallthrough.h>
 
 #include <QDebug>
 #include <QMetaEnum>
@@ -41,9 +40,7 @@ using namespace CppTools;
 using namespace TextEditor;
 using namespace CppTools::Internal;
 
-CodeFormatter::~CodeFormatter()
-{
-}
+CodeFormatter::~CodeFormatter() = default;
 
 void CodeFormatter::setTabSize(int tabSize)
 {
@@ -182,8 +179,8 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             switch (kind) {
             case T_LBRACE:      enter(brace_list_open); break;
             case T_RBRACE:      leave(true); continue;
-            case T_SEMICOLON:   leave(); continue;
-            case T_RPAREN:      leave(); continue;
+            case T_SEMICOLON:
+            case T_RPAREN:
             case T_COMMA:       leave(); continue;
             default:            enter(assign_open); continue;
             } break;
@@ -202,14 +199,20 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
                     turnInto(substatement_open);
                 }
                 break;
+            case T_ARROW: // Trailing return type?
+                if (m_currentState.at(m_currentState.size() - 2).type == declaration_start)
+                    leave();
+                else
+                    tryExpression();
+                break;
             default:            tryExpression(); break;
             } break;
 
         case assign_open:
             switch (kind) {
             case T_RBRACE:      leave(true); continue;
-            case T_SEMICOLON:   leave(); continue;
-            case T_RPAREN:      leave(); continue;
+            case T_SEMICOLON:
+            case T_RPAREN:
             case T_COMMA:       leave(); continue;
             default:            tryExpression(); break;
             } break;
@@ -270,7 +273,13 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             case T_RBRACKET:    // '[', ']' and '->' can be part of lambda declarator
             case T_LBRACKET:
             case T_ARROW:       break;
-            default:            leave(); continue;
+            default:
+                if (m_tokenIndex > 0 && tokenAt(m_tokenIndex - 1).kind() == T_ARROW) {
+                    break;
+                } else {
+                    leave();
+                    continue;
+                }
             } break;
 
         case lambda_declarator:
@@ -519,7 +528,7 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
                 }
             }
 
-            QStringRef tokenText = currentTokenText();
+            QStringView tokenText = currentTokenText();
             if (tokenText == QLatin1String("ifdef")
                     || tokenText == QLatin1String("if")
                     || tokenText == QLatin1String("ifndef")) {
@@ -550,6 +559,7 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             break;
 
         case string_open:
+        case raw_string_open:
             if (!m_currentToken.isStringLiteral()) {
                 leave();
                 continue;
@@ -626,7 +636,6 @@ void CodeFormatter::updateStateUntil(const QTextBlock &endBlock)
             break;
         if (loadLexerState(it) == -1)
             break;
-
         previousState = blockData.m_endState;
     }
 
@@ -664,12 +673,22 @@ void CodeFormatter::updateLineStateChange(const QTextBlock &block)
     saveBlockData(&next, BlockData());
 }
 
+bool CodeFormatter::isInRawStringLiteral(const QTextBlock &block) const
+{
+    if (!block.previous().isValid())
+        return false;
+    BlockData blockData;
+    if (!loadBlockData(block.previous(), &blockData))
+        return false;
+    return !blockData.m_endState.isEmpty() && blockData.m_endState.top().type == raw_string_open;
+}
+
 CodeFormatter::State CodeFormatter::state(int belowTop) const
 {
     if (belowTop < m_currentState.size())
         return m_currentState.at(m_currentState.size() - 1 - belowTop);
     else
-        return State();
+        return {};
 }
 
 int CodeFormatter::tokenIndex() const
@@ -786,7 +805,7 @@ bool CodeFormatter::tryExpression(bool alsoExpression)
         newState = stream_op;
         for (int i = m_currentState.size() - 1; i >= 0; --i) {
             const int type = m_currentState.at(i).type;
-            if (type == arglist_open) { // likely a left-shift instead
+            if (type == arglist_open || type == braceinit_open) { // likely a left-shift instead
                 newState = -1;
                 break;
             }
@@ -807,7 +826,7 @@ bool CodeFormatter::tryExpression(bool alsoExpression)
     }
 
     if (m_currentToken.isStringLiteral())
-        newState = string_open;
+        newState = m_currentToken.kind() == T_RAW_STRING_LITERAL ? raw_string_open : string_open;
 
     if (newState != -1) {
         if (alsoExpression)
@@ -836,7 +855,7 @@ bool CodeFormatter::tryDeclaration()
         return true;
     case T_IDENTIFIER:
         if (m_tokenIndex == 0) {
-            const QStringRef tokenText = currentTokenText();
+            const QStringView tokenText = currentTokenText();
             if (tokenText.startsWith(QLatin1String("Q_"))
                     || tokenText.startsWith(QLatin1String("QT_"))
                     || tokenText.startsWith(QLatin1String("QML_"))
@@ -866,6 +885,7 @@ bool CodeFormatter::tryDeclaration()
     case T_AUTO:
     case T___TYPEOF__:
     case T___ATTRIBUTE__:
+    case T___DECLSPEC:
     case T_STATIC:
     case T_FRIEND:
     case T_CONST:
@@ -990,9 +1010,11 @@ int CodeFormatter::column(int index) const
     return col;
 }
 
-QStringRef CodeFormatter::currentTokenText() const
+QStringView CodeFormatter::currentTokenText() const
 {
-    return m_currentLine.midRef(m_currentToken.utf16charsBegin(), m_currentToken.utf16chars());
+    if (m_currentToken.utf16charsEnd() > m_currentLine.size())
+        return QStringView(m_currentLine).mid(m_currentToken.utf16charsBegin());
+    return QStringView(m_currentLine).mid(m_currentToken.utf16charsBegin(), m_currentToken.utf16chars());
 }
 
 void CodeFormatter::turnInto(int newState)
@@ -1057,6 +1079,8 @@ int CodeFormatter::tokenizeBlock(const QTextBlock &block, bool *endedJoined)
     features.qtKeywordsEnabled = true;
     features.cxxEnabled = true;
     features.objCEnabled = true;
+    features.cxx11Enabled = true;
+    features.cxx14Enabled = true;
 
     SimpleLexer tokenize;
     tokenize.setLanguageFeatures(features);
@@ -1099,9 +1123,7 @@ namespace Internal {
 }
 }
 
-QtStyleCodeFormatter::QtStyleCodeFormatter()
-{
-}
+QtStyleCodeFormatter::QtStyleCodeFormatter() = default;
 
 QtStyleCodeFormatter::QtStyleCodeFormatter(const TabSettings &tabSettings,
                                            const CppCodeStyleSettings &settings)
@@ -1125,7 +1147,7 @@ void QtStyleCodeFormatter::setCodeStyleSettings(const CppCodeStyleSettings &sett
 void QtStyleCodeFormatter::saveBlockData(QTextBlock *block, const BlockData &data) const
 {
     TextBlockUserData *userData = TextDocumentLayout::userData(*block);
-    CppCodeFormatterData *cppData = static_cast<CppCodeFormatterData *>(userData->codeFormatterData());
+    auto cppData = static_cast<CppCodeFormatterData *>(userData->codeFormatterData());
     if (!cppData) {
         cppData = new CppCodeFormatterData;
         userData->setCodeFormatterData(cppData);
@@ -1135,10 +1157,10 @@ void QtStyleCodeFormatter::saveBlockData(QTextBlock *block, const BlockData &dat
 
 bool QtStyleCodeFormatter::loadBlockData(const QTextBlock &block, BlockData *data) const
 {
-    TextBlockUserData *userData = TextDocumentLayout::testUserData(block);
+    TextBlockUserData *userData = TextDocumentLayout::textUserData(block);
     if (!userData)
         return false;
-    CppCodeFormatterData *cppData = static_cast<CppCodeFormatterData *>(userData->codeFormatterData());
+    auto cppData = static_cast<const CppCodeFormatterData *>(userData->codeFormatterData());
     if (!cppData)
         return false;
 
@@ -1418,6 +1440,7 @@ void QtStyleCodeFormatter::onEnter(int newState, int *indentDepth, int *savedInd
         break;
 
     case string_open:
+    case raw_string_open:
         *paddingDepth = tokenPosition - *indentDepth;
         break;
     }
@@ -1460,6 +1483,7 @@ void QtStyleCodeFormatter::adjustIndent(const Tokens &tokens, int lexerState, in
         }
         break;
     case string_open:
+    case raw_string_open:
         if (!tokenAt(0).isStringLiteral()) {
             *paddingDepth = topState.savedPaddingDepth;
             topState = previousState;
@@ -1524,6 +1548,7 @@ void QtStyleCodeFormatter::adjustIndent(const Tokens &tokens, int lexerState, in
                 && topState.type != block_open
                 && topState.type != substatement_open
                 && topState.type != brace_list_open
+                && topState.type != arglist_open
                 && !topWasMaybeElse) {
             *indentDepth = topState.savedIndentDepth;
             *paddingDepth = 0;
@@ -1592,7 +1617,7 @@ void QtStyleCodeFormatter::adjustIndent(const Tokens &tokens, int lexerState, in
         if (m_styleSettings.indentDeclarationsRelativeToAccessSpecifiers
                 && topState.type == class_open) {
             if (tokenAt(1).is(T_COLON) || tokenAt(2).is(T_COLON)
-                    || (tokenAt(tokenCount() - 1).is(T_COLON) && tokenAt(1).is(T___ATTRIBUTE__))) {
+                || (tokenAt(tokenCount() - 1).is(T_COLON) && (tokenAt(1).is(T___ATTRIBUTE__) || tokenAt(1).is(T___DECLSPEC)))) {
                 *indentDepth = topState.savedIndentDepth;
                 if (m_styleSettings.indentAccessSpecifiers)
                     *indentDepth += m_tabSettings.m_indentSize;
