@@ -35,14 +35,15 @@
 
 #include <QTimer>
 
+#include <set>
+
 using namespace ProjectExplorer;
 
 namespace CppTools {
 namespace Internal {
 
-ProjectInfoGenerator::ProjectInfoGenerator(
-    const QFutureInterface<void> &futureInterface,
-    const ProjectUpdateInfo &projectUpdateInfo)
+ProjectInfoGenerator::ProjectInfoGenerator(const QFutureInterface<ProjectInfo> &futureInterface,
+                                           const ProjectUpdateInfo &projectUpdateInfo)
     : m_futureInterface(futureInterface)
     , m_projectUpdateInfo(projectUpdateInfo)
 {
@@ -94,8 +95,17 @@ static ProjectPart::Ptr projectPartFromRawProjectPart(
     part->projectMacros = rawProjectPart.projectMacros;
     if (!part->projectConfigFile.isEmpty())
         part->projectMacros += Macro::toMacros(ProjectPart::readProjectConfigFile(part));
-    part->headerPaths = rawProjectPart.headerPaths;
+
+    // Prevent duplicate include paths.
+    std::set<QString> seenPaths;
+    for (const HeaderPath &p : qAsConst(rawProjectPart.headerPaths)) {
+        const QString cleanPath = QDir::cleanPath(p.path);
+        if (seenPaths.insert(cleanPath).second)
+            part->headerPaths << HeaderPath(cleanPath, p.type);
+    }
+
     part->precompiledHeaders = rawProjectPart.precompiledHeaders;
+    part->includedFiles = rawProjectPart.includedFiles;
     part->selectedForBuilding = rawProjectPart.selectedForBuilding;
 
     return part;
@@ -186,46 +196,10 @@ ProjectPart::Ptr ProjectInfoGenerator::createProjectPart(
     ProjectPart::Ptr part(templateProjectPart->copy());
     part->displayName = partName;
     part->files = projectFiles;
-    part->toolchainType = tcInfo.type;
-    part->isMsvc2015Toolchain = tcInfo.isMsvc2015ToolChain;
-    part->toolChainWordWidth = tcInfo.wordWidth == 64 ? ProjectPart::WordWidth64Bit
-                                                      : ProjectPart::WordWidth32Bit;
-    part->toolChainInstallDir = tcInfo.installDir;
-    part->toolChainTargetTriple = tcInfo.targetTriple;
-    part->extraCodeModelFlags = tcInfo.extraCodeModelFlags;
-    part->compilerFlags = flags.commandLineFlags;
     part->warningFlags = flags.warningFlags;
     part->language = language;
-    part->languageExtensions = flags.languageExtensions;
-
-    // Toolchain macros and language version
-    if (tcInfo.macroInspectionRunner) {
-        auto macroInspectionReport = tcInfo.macroInspectionRunner(flags.commandLineFlags);
-        part->toolChainMacros = macroInspectionReport.macros;
-        part->languageVersion = macroInspectionReport.languageVersion;
-    // No compiler set in kit.
-    } else if (language == Language::C) {
-        part->languageVersion = Utils::LanguageVersion::LatestC;
-    } else {
-        part->languageVersion = Utils::LanguageVersion::LatestCxx;
-    }
-
-    // Header paths
-    if (tcInfo.headerPathsRunner) {
-        const HeaderPaths builtInHeaderPaths
-            = tcInfo.headerPathsRunner(flags.commandLineFlags,
-                                       tcInfo.sysRootPath,
-                                       tcInfo.targetTriple);
-
-        HeaderPaths &headerPaths = part->headerPaths;
-        for (const HeaderPath &header : builtInHeaderPaths) {
-            const HeaderPath headerPath{header.path, header.type};
-            if (!headerPaths.contains(headerPath))
-                headerPaths.push_back(headerPath);
-        }
-    }
-
-    part->languageExtensions |= languageExtensions;
+    part->languageExtensions = flags.languageExtensions | languageExtensions;
+    part->setupToolchainProperties(tcInfo, flags.commandLineFlags);
     part->updateLanguageFeatures();
 
     return part;
