@@ -29,6 +29,7 @@
 #include <extensionsystem/iplugin.h>
 
 namespace ProjectExplorer { class RunControl; }
+namespace Utils { class FilePath; }
 
 namespace Debugger {
 namespace Internal {
@@ -40,9 +41,7 @@ class DebuggerPlugin : public ExtensionSystem::IPlugin
 
 public:
     DebuggerPlugin();
-    ~DebuggerPlugin();
-
-    static DebuggerPlugin *instance();
+    ~DebuggerPlugin() override;
 
 private:
     // IPlugin implementation.
@@ -56,8 +55,20 @@ private:
     // Called from AppOutputPane::attachToRunControl().
     Q_SLOT void attachExternalApplication(ProjectExplorer::RunControl *rc);
 
-    QList<QObject *> createTestObjects() const override;
+    // Called from GammaRayIntegration
+    Q_SLOT void getEnginesState(QByteArray *json) const;
+
+    // Called from DockerDevice
+    Q_SLOT void autoDetectDebuggersForDevice(const Utils::FilePath &deviceRoot,
+                                             const QString &detectionId,
+                                             QString *logMessage);
+    Q_SLOT void removeDetectedDebuggers(const QString &detectionId, QString *logMessage);
+    Q_SLOT void listDetectedDebuggers(const QString &detectionId, QString *logMessage);
+
+    QVector<QObject *> createTestObjects() const override;
 };
 
 } // namespace Internal
 } // namespace Debugger
+
+Q_DECLARE_METATYPE(QString *)
