@@ -39,6 +39,7 @@
 #include <extensionsystem/pluginmanager.h>
 
 #include <projectexplorer/abi.h>
+#include <projectexplorer/buildinfo.h>
 #include <projectexplorer/buildsteplist.h>
 #include <projectexplorer/buildsystem.h>
 #include <projectexplorer/customexecutablerunconfiguration.h>
@@ -134,9 +135,9 @@ public:
                 || action == Rename;
     }
 
-    RemovedFilesFromProject removeFiles(Node *, const QStringList &filePaths, QStringList *) final;
-    bool renameFile(Node *, const QString &filePath, const QString &newFilePath) final;
-    bool addFiles(Node *, const QStringList &filePaths, QStringList *) final;
+    RemovedFilesFromProject removeFiles(Node *, const FilePaths &filePaths, FilePaths *) final;
+    bool renameFile(Node *, const FilePath &oldFilePath, const FilePath &newFilePath) final;
+    bool addFiles(Node *, const FilePaths &filePaths, FilePaths *) final;
 
     FilePath filesFilePath() const { return ::FilePath::fromString(m_filesFileName); }
 
@@ -154,7 +155,7 @@ public:
     void updateDeploymentData();
 
     bool setFiles(const QStringList &filePaths);
-    void removeFiles(const QStringList &filesToRemove);
+    void removeFiles(const FilePaths &filesToRemove);
 
 private:
     QString m_filesFileName;
@@ -173,8 +174,6 @@ private:
     CppTools::CppProjectUpdaterInterface *m_cppCodeModelUpdater = nullptr;
 
     Utils::FileSystemWatcher m_deployFileWatcher;
-
-    ParseGuard m_guard;
 };
 
 
@@ -186,7 +185,8 @@ private:
 
 static bool writeFile(const QString &filePath, const QString &contents)
 {
-    Utils::FileSaver saver(filePath, QIODevice::Text | QIODevice::WriteOnly);
+    Utils::FileSaver saver(Utils::FilePath::fromString(filePath),
+                           QIODevice::Text | QIODevice::WriteOnly);
     return saver.write(contents.toUtf8()) && saver.finalize();
 }
 
@@ -195,7 +195,7 @@ GenericProject::GenericProject(const Utils::FilePath &fileName)
 {
     setId(Constants::GENERICPROJECT_ID);
     setProjectLanguages(Context(ProjectExplorer::Constants::CXX_LANGUAGE_ID));
-    setDisplayName(fileName.toFileInfo().completeBaseName());
+    setDisplayName(fileName.completeBaseName());
     setBuildSystemCreator([](Target *t) { return new GenericBuildSystem(t); });
 }
 
@@ -253,7 +253,14 @@ GenericBuildSystem::GenericBuildSystem(Target *target)
     connect(&m_deployFileWatcher, &FileSystemWatcher::fileChanged,
             this, &GenericBuildSystem::updateDeploymentData);
 
-    connect(target, &Target::activeBuildConfigurationChanged, this, [this] { refresh(Everything); });
+    connect(target, &Target::activeBuildConfigurationChanged, this, [this, target] {
+        if (target == project()->activeTarget())
+            refresh(Everything);
+    });
+    connect(project(), &Project::activeTargetChanged, this, [this, target] {
+        if (target == project()->activeTarget())
+            refresh(Everything);
+    });
 }
 
 GenericBuildSystem::~GenericBuildSystem()
@@ -263,7 +270,6 @@ GenericBuildSystem::~GenericBuildSystem()
 
 void GenericBuildSystem::triggerParsing()
 {
-    m_guard = guardParsingRun();
     refresh(Everything);
 }
 
@@ -296,16 +302,17 @@ bool GenericBuildSystem::saveRawFileList(const QStringList &rawFileList)
 
 bool GenericBuildSystem::saveRawList(const QStringList &rawList, const QString &fileName)
 {
-    FileChangeBlocker changeGuard(fileName);
+    const FilePath filePath = FilePath::fromString(fileName);
+    FileChangeBlocker changeGuard(filePath);
     // Make sure we can open the file for writing
-    Utils::FileSaver saver(fileName, QIODevice::Text);
+    Utils::FileSaver saver(filePath, QIODevice::Text);
     if (!saver.hasError()) {
         QTextStream stream(saver.file());
         for (const QString &filePath : rawList)
             stream << filePath << '\n';
         saver.setResult(&stream);
     }
-    bool result = saver.finalize(ICore::mainWindow());
+    bool result = saver.finalize(ICore::dialogParent());
     return result;
 }
 
@@ -318,8 +325,9 @@ static void insertSorted(QStringList *list, const QString &value)
         list->insert(it, value);
 }
 
-bool GenericBuildSystem::addFiles(Node *, const QStringList &filePaths, QStringList *)
+bool GenericBuildSystem::addFiles(Node *, const FilePaths &filePaths_, FilePaths *)
 {
+    const QStringList filePaths = Utils::transform(filePaths_, &FilePath::toString);
     const QDir baseDir(projectDirectory().toString());
     QStringList newList = m_rawFileList;
     if (filePaths.size() > m_rawFileList.size()) {
@@ -359,12 +367,12 @@ bool GenericBuildSystem::addFiles(Node *, const QStringList &filePaths, QStringL
     return result;
 }
 
-RemovedFilesFromProject GenericBuildSystem::removeFiles(Node *, const QStringList &filePaths, QStringList *)
+RemovedFilesFromProject GenericBuildSystem::removeFiles(Node *, const FilePaths &filePaths, FilePaths *)
 {
     QStringList newList = m_rawFileList;
 
-    for (const QString &filePath : filePaths) {
-        QHash<QString, QString>::iterator i = m_rawListEntries.find(filePath);
+    for (const FilePath &filePath : filePaths) {
+        QHash<QString, QString>::iterator i = m_rawListEntries.find(filePath.toString());
         if (i != m_rawListEntries.end())
             newList.removeOne(i.value());
     }
@@ -384,17 +392,17 @@ bool GenericBuildSystem::setFiles(const QStringList &filePaths)
     return saveRawFileList(newList);
 }
 
-bool GenericBuildSystem::renameFile(Node *, const QString &filePath, const QString &newFilePath)
+bool GenericBuildSystem::renameFile(Node *, const FilePath &oldFilePath, const FilePath &newFilePath)
 {
     QStringList newList = m_rawFileList;
 
-    QHash<QString, QString>::iterator i = m_rawListEntries.find(filePath);
+    QHash<QString, QString>::iterator i = m_rawListEntries.find(oldFilePath.toString());
     if (i != m_rawListEntries.end()) {
         int index = newList.indexOf(i.value());
         if (index != -1) {
             QDir baseDir(projectDirectory().toString());
             newList.removeAt(index);
-            insertSorted(&newList, baseDir.relativeFilePath(newFilePath));
+            insertSorted(&newList, baseDir.relativeFilePath(newFilePath.toString()));
         }
     }
 
@@ -408,7 +416,7 @@ static QStringList readFlags(const QString &filePath)
         return QStringList();
     QStringList flags;
     for (const auto &line : lines)
-        flags.append(QtcProcess::splitArgs(line));
+        flags.append(ProcessArgs::splitArgs(line));
     return flags;
 }
 
@@ -445,7 +453,7 @@ void GenericBuildSystem::parse(RefreshOptions options)
 FilePath GenericBuildSystem::findCommonSourceRoot()
 {
     if (m_files.isEmpty())
-        return FilePath::fromFileInfo(QFileInfo(m_filesFileName).absolutePath());
+        return FilePath::fromFileInfo(QFileInfo(m_filesFileName));
 
     QString root = m_files.front();
     for (const QString &item : qAsConst(m_files)) {
@@ -469,7 +477,7 @@ void GenericBuildSystem::refresh(RefreshOptions options)
 
     if (options & Files) {
         auto newRoot = std::make_unique<ProjectNode>(projectDirectory());
-        newRoot->setDisplayName(projectFilePath().toFileInfo().completeBaseName());
+        newRoot->setDisplayName(projectFilePath().completeBaseName());
 
         // find the common base directory of all source files
         FilePath baseDir = findCommonSourceRoot();
@@ -556,6 +564,8 @@ void GenericBuildSystem::refreshCppCodeModel()
 {
     if (!m_cppCodeModelUpdater)
         return;
+    if (target() != project()->activeTarget())
+        return;
     QtSupport::CppKitInfo kitInfo(kit());
     QTC_ASSERT(kitInfo.isValid(), return);
 
@@ -565,8 +575,8 @@ void GenericBuildSystem::refreshCppCodeModel()
     rpp.setQtVersion(kitInfo.projectPartQtVersion);
     rpp.setHeaderPaths(m_projectIncludePaths);
     rpp.setConfigFileName(m_configFileName);
-    rpp.setFlagsForCxx({nullptr, m_cxxflags});
-    rpp.setFlagsForC({nullptr, m_cflags});
+    rpp.setFlagsForCxx({nullptr, m_cxxflags, projectDirectory().toString()});
+    rpp.setFlagsForC({nullptr, m_cflags, projectDirectory().toString()});
     rpp.setFiles(m_files);
 
     m_cppCodeModelUpdater->update({project(), kitInfo, activeParseEnvironment(), {rpp}});
@@ -598,7 +608,7 @@ void GenericBuildSystem::updateDeploymentData()
     }
 }
 
-void GenericBuildSystem::removeFiles(const QStringList &filesToRemove)
+void GenericBuildSystem::removeFiles(const FilePaths &filesToRemove)
 {
     if (removeFiles(nullptr, filesToRemove, nullptr) == RemovedFilesFromProject::Error) {
         TaskHub::addTask(BuildSystemTask(Task::Error,
@@ -641,13 +651,30 @@ ProjectExplorer::DeploymentKnowledge GenericProject::deploymentKnowledge() const
     return DeploymentKnowledge::Approximative;
 }
 
+void GenericProject::configureAsExampleProject(ProjectExplorer::Kit *kit)
+{
+    QList<BuildInfo> infoList;
+    const QList<Kit *> kits(kit != nullptr ? QList<Kit *>({kit}) : KitManager::kits());
+    for (Kit *k : kits) {
+        if (auto factory = BuildConfigurationFactory::find(k, projectFilePath())) {
+            for (int i = 0; i < 5; ++i) {
+                BuildInfo buildInfo;
+                buildInfo.displayName = tr("Build %1").arg(i + 1);
+                buildInfo.factory = factory;
+                buildInfo.kitId = kit->id();
+                buildInfo.buildDirectory = projectFilePath();
+                infoList << buildInfo;
+            }
+        }
+    }
+    setup(infoList);
+}
+
 bool GenericProjectFile::reload(QString *errorString, IDocument::ReloadFlag flag, IDocument::ChangeType type)
 {
     Q_UNUSED(errorString)
     Q_UNUSED(flag)
-    if (type == TypePermissions)
-        return true;
-
+    Q_UNUSED(type)
     if (Target *t = m_project->activeTarget())
         static_cast<GenericBuildSystem *>(t->buildSystem())->refresh(m_options);
 
@@ -658,7 +685,7 @@ void GenericProject::editFilesTriggered()
 {
     SelectableFilesDialogEditFiles sfd(projectDirectory(),
                                        files(Project::AllFiles),
-                                       ICore::mainWindow());
+                                       ICore::dialogParent());
     if (sfd.exec() == QDialog::Accepted) {
         if (Target *t = activeTarget()) {
             auto bs = static_cast<GenericBuildSystem *>(t->buildSystem());
@@ -667,7 +694,7 @@ void GenericProject::editFilesTriggered()
     }
 }
 
-void GenericProject::removeFilesTriggered(const QStringList &filesToRemove)
+void GenericProject::removeFilesTriggered(const FilePaths &filesToRemove)
 {
     if (Target *t = activeTarget())
         static_cast<GenericBuildSystem *>(t->buildSystem())->removeFiles(filesToRemove);
