@@ -98,7 +98,7 @@ Macro& Macro::operator=(const Macro &other)
 
 bool Macro::load(QString fileName)
 {
-    if (d->events.count())
+    if (!d->events.isEmpty())
         return true; // the macro is not empty
 
     // Take the current filename if the parameter is null
@@ -138,7 +138,7 @@ bool Macro::loadHeader(const QString &fileName)
 
 bool Macro::save(const QString &fileName, QWidget *parent)
 {
-    Utils::FileSaver saver(fileName);
+    Utils::FileSaver saver(Utils::FilePath::fromString(fileName));
     if (!saver.hasError()) {
         QDataStream stream(saver.file());
         stream << d->version;
