@@ -31,7 +31,6 @@
 
 #include <QDebug>
 #include <QtEndian>
-#include <QFile>
 #include <QRegularExpression>
 #include <QString>
 #include <QStringList>
@@ -240,6 +239,10 @@ static Abis parseCoffHeader(const QByteArray &data)
         arch = Abi::ArmArchitecture;
         width = 32;
         break;
+    case 0xaa64: // ARM64
+        arch = Abi::ArmArchitecture;
+        width = 64;
+        break;
     case 0x8664: // x86_64
         arch = Abi::X86Architecture;
         width = 64;
@@ -413,9 +416,13 @@ static Abis abiOf(const QByteArray &data)
             result.append(macAbiForCpu(type));
             pos += 20;
         }
-    } else if (getUint8(data, 0) == 'B' && getUint8(data, 1) == 'C'
-                && getUint8(data, 2) == 0xc0 && getUint8(data, 3) == 0xde) {
-        // https://llvm.org/docs/BitCodeFormat.html#llvm-ir-magic-number
+    } else if (// Qt 5.15+ https://webassembly.github.io/spec/core/binary/modules.html#binary-module
+               (getUint8(data, 0) == 0 && getUint8(data, 1) == 'a'
+                && getUint8(data, 2) == 's' && getUint8(data, 3) == 'm')
+
+               // Qt < 5.15 https://llvm.org/docs/BitCodeFormat.html#llvm-ir-magic-number
+               || (getUint8(data, 0) == 'B' && getUint8(data, 1) == 'C'
+                   && getUint8(data, 2) == 0xc0 && getUint8(data, 3) == 0xde)) {
         result.append(Abi(Abi::AsmJsArchitecture, Abi::UnknownOS, Abi::UnknownFlavor,
                           Abi::EmscriptenFormat, 32));
     } else if (data.size() >= 64){
@@ -455,7 +462,7 @@ Abi Abi::abiFromTargetTriplet(const QString &triple)
     if (machine.isEmpty())
         return Abi();
 
-    const QVector<QStringRef> parts = machine.splitRef(QRegularExpression("[ /-]"));
+    const QStringList parts = machine.split(QRegularExpression("[ /-]"));
 
     Architecture arch = UnknownArchitecture;
     OS os = UnknownOS;
@@ -464,7 +471,7 @@ Abi Abi::abiFromTargetTriplet(const QString &triple)
     unsigned char width = 0;
     int unknownCount = 0;
 
-    for (const QStringRef &p : parts) {
+    for (const QString &p : parts) {
         if (p == "unknown" || p == "pc"
                 || p == "gnu" || p == "uclibc"
                 || p == "86_64" || p == "redhat"
@@ -722,14 +729,6 @@ bool Abi::isCompatibleWith(const Abi &other) const
     return isCompat;
 }
 
-bool Abi::isFullyCompatibleWith(const Abi &other) const
-{
-    return *this == other
-            || (wordWidth() == other.wordWidth()
-                && architecture() == other.architecture()
-                && compatibleMSVCFlavors(osFlavor(), other.osFlavor()));
-}
-
 bool Abi::isValid() const
 {
     return m_architecture != UnknownArchitecture
@@ -881,7 +880,7 @@ QString Abi::toString(int w)
 Abi Abi::fromString(const QString &abiString)
 {
     Abi::Architecture architecture = UnknownArchitecture;
-    const QVector<QStringRef> abiParts = abiString.splitRef('-');
+    const QStringList abiParts = abiString.split('-');
     if (!abiParts.isEmpty()) {
         architecture = architectureFromString(abiParts.at(0));
         if (abiParts.at(0) != toString(architecture))
@@ -919,7 +918,7 @@ Abi Abi::fromString(const QString &abiString)
     return Abi(architecture, os, flavor, format, wordWidth);
 }
 
-Abi::Architecture Abi::architectureFromString(const QStringRef &a)
+Abi::Architecture Abi::architectureFromString(const QString &a)
 {
     if (a == "unknown")
         return UnknownArchitecture;
@@ -983,7 +982,7 @@ Abi::Architecture Abi::architectureFromString(const QStringRef &a)
     return UnknownArchitecture;
 }
 
-Abi::OS Abi::osFromString(const QStringRef &o)
+Abi::OS Abi::osFromString(const QString &o)
 {
     if (o == "unknown")
         return UnknownOS;
@@ -1006,7 +1005,7 @@ Abi::OS Abi::osFromString(const QStringRef &o)
     return UnknownOS;
 }
 
-Abi::OSFlavor Abi::osFlavorFromString(const QStringRef &of, const OS os)
+Abi::OSFlavor Abi::osFlavorFromString(const QString &of, const OS os)
 {
     const int index = indexOfFlavor(of.toUtf8());
     const auto flavor = OSFlavor(index);
@@ -1015,7 +1014,7 @@ Abi::OSFlavor Abi::osFlavorFromString(const QStringRef &of, const OS os)
     return UnknownFlavor;
 }
 
-Abi::BinaryFormat Abi::binaryFormatFromString(const QStringRef &bf)
+Abi::BinaryFormat Abi::binaryFormatFromString(const QString &bf)
 {
     if (bf == "unknown")
         return UnknownFormat;
@@ -1036,13 +1035,13 @@ Abi::BinaryFormat Abi::binaryFormatFromString(const QStringRef &bf)
     return UnknownFormat;
 }
 
-unsigned char Abi::wordWidthFromString(const QStringRef &w)
+unsigned char Abi::wordWidthFromString(const QString &w)
 {
     if (!w.endsWith("bit"))
         return 0;
 
     bool ok = false;
-    const QStringRef number = w.string()->midRef(w.position(), w.count() - 3);
+    const QString number = w.left(w.size() - 3);
     const int bitCount = number.toInt(&ok);
     if (!ok)
         return 0;
@@ -1161,14 +1160,7 @@ Abis Abi::abisOfBinary(const Utils::FilePath &path)
     if (path.isEmpty())
         return tmp;
 
-    QFile f(path.toString());
-    if (!f.exists())
-        return tmp;
-
-    if (!f.open(QFile::ReadOnly))
-        return tmp;
-
-    QByteArray data = f.read(1024);
+    QByteArray data = path.fileContents(1024);
     if (data.size() >= 67
             && getUint8(data, 0) == '!' && getUint8(data, 1) == '<' && getUint8(data, 2) == 'a'
             && getUint8(data, 3) == 'r' && getUint8(data, 4) == 'c' && getUint8(data, 5) == 'h'
@@ -1187,7 +1179,7 @@ Abis Abi::abisOfBinary(const Utils::FilePath &path)
             const QString fileName = QString::fromLocal8Bit(data.mid(0, 16));
             quint64 fileNameOffset = 0;
             if (fileName.startsWith("#1/"))
-                fileNameOffset = fileName.midRef(3).toInt();
+                fileNameOffset = fileName.mid(3).toInt();
             const QString fileLength = QString::fromLatin1(data.mid(48, 10));
 
             int toSkip = 60 + fileNameOffset;
@@ -1201,13 +1193,11 @@ Abis Abi::abisOfBinary(const Utils::FilePath &path)
                 break;
 
             offset += (offset % 2); // ar is 2 byte aligned
-            f.seek(offset);
-            data = f.read(1024);
+            data = path.fileContents(1024, offset);
         }
     } else {
         tmp = abiOf(data);
     }
-    f.close();
 
     // Remove duplicates:
     Abis result;
@@ -1237,19 +1227,19 @@ void ProjectExplorer::ProjectExplorerPlugin::testAbiRoundTrips()
 {
     for (int i = 0; i <= Abi::UnknownArchitecture; ++i) {
         const QString string = Abi::toString(static_cast<Abi::Architecture>(i));
-        const Abi::Architecture arch = Abi::architectureFromString(QStringRef(&string));
+        const Abi::Architecture arch = Abi::architectureFromString(string);
         QCOMPARE(static_cast<Abi::Architecture>(i), arch);
     }
     for (int i = 0; i <= Abi::UnknownOS; ++i) {
         const QString string = Abi::toString(static_cast<Abi::OS>(i));
-        const Abi::OS os = Abi::osFromString(QStringRef(&string));
+        const Abi::OS os = Abi::osFromString(string);
         QCOMPARE(static_cast<Abi::OS>(i), os);
     }
     for (const Abi::OSFlavor flavorIt : Abi::allOsFlavors()) {
         const QString string = Abi::toString(flavorIt);
         for (int os = 0; os <= Abi::UnknownOS; ++os) {
             const auto osEnum = static_cast<Abi::OS>(os);
-            const Abi::OSFlavor flavor = Abi::osFlavorFromString(QStringRef(&string), osEnum);
+            const Abi::OSFlavor flavor = Abi::osFlavorFromString(string, osEnum);
             if (isGenericFlavor(flavorIt) && flavor != Abi::UnknownFlavor)
                 QVERIFY(isGenericFlavor(flavor));
             else if (flavor == Abi::UnknownFlavor && flavorIt != Abi::UnknownFlavor)
@@ -1260,12 +1250,12 @@ void ProjectExplorer::ProjectExplorerPlugin::testAbiRoundTrips()
     }
     for (int i = 0; i <= Abi::UnknownFormat; ++i) {
         QString string = Abi::toString(static_cast<Abi::BinaryFormat>(i));
-        Abi::BinaryFormat format = Abi::binaryFormatFromString(QStringRef(&string));
+        Abi::BinaryFormat format = Abi::binaryFormatFromString(string);
         QCOMPARE(static_cast<Abi::BinaryFormat>(i), format);
     }
     for (unsigned char i : {0, 8, 16, 32, 64}) {
         QString string = Abi::toString(i);
-        unsigned char wordwidth = Abi::wordWidthFromString(QStringRef(&string));
+        unsigned char wordwidth = Abi::wordWidthFromString(string);
         QCOMPARE(i, wordwidth);
     }
 }
@@ -1282,6 +1272,7 @@ void ProjectExplorer::ProjectExplorerPlugin::testAbiOfBinary_data()
             << QString::fromLatin1("/does/not/exist")
             << (QStringList());
 
+    // Clone test data from: https://git.qt.io/chstenge/creator-test-data
     // Set up prefix for test data now that we can be sure to have some tests to run:
     QString prefix = QString::fromLocal8Bit(qgetenv("QTC_TEST_EXTRADATALOCATION"));
     if (prefix.isEmpty())
@@ -1318,9 +1309,12 @@ void ProjectExplorer::ProjectExplorerPlugin::testAbiOfBinary_data()
     QTest::newRow("static QtCore: linux 64bit")
             << QString::fromLatin1("%1/static/linux-64bit-release.a").arg(prefix)
             << (QStringList() << QString::fromLatin1("x86-linux-generic-elf-64bit"));
-    QTest::newRow("static QtCore: asmjs emscripten 32bit")
+    QTest::newRow("static QtCore: asmjs emscripten 32bit (Qt < 5.15)")
             << QString::fromLatin1("%1/static/asmjs-emscripten.a").arg(prefix)
             << (QStringList() << QString::fromLatin1("asmjs-unknown-unknown-emscripten-32bit"));
+    QTest::newRow("static QtCore: asmjs emscripten 32bit (Qt >= 5.15)")
+            << QString::fromLatin1("%1/static/asmjs-emscripten-5.15.a").arg(prefix)
+            << (QStringList() << QString::fromLatin1("asmjs-unknown-unknown-emscripten-32bit"));
 
     QTest::newRow("static stdc++: mac fat")
             << QString::fromLatin1("%1/static/mac-fat.a").arg(prefix)
@@ -1408,7 +1402,12 @@ void ProjectExplorer::ProjectExplorerPlugin::testAbiOfBinary()
     QFETCH(QString, file);
     QFETCH(QStringList, abis);
 
-    const Abis result = Abi::abisOfBinary(Utils::FilePath::fromString(file));
+    const Utils::FilePath fp = Utils::FilePath::fromString(file);
+    const QString dataTag = QString::fromLocal8Bit(QTest::currentDataTag());
+    if (!fp.exists() && dataTag != "no file" && dataTag != "non existing file")
+        QSKIP("binary file not present");
+
+    const Abis result = Abi::abisOfBinary(fp);
     QCOMPARE(result.count(), abis.count());
     for (int i = 0; i < abis.count(); ++i)
         QCOMPARE(result.at(i).toString(), abis.at(i));
