@@ -148,7 +148,7 @@ void CustomWizard::setParameters(const CustomWizardParametersPtr &p)
 
     setId(p->id);
     setSupportedProjectTypes((p->kind == Core::IWizardFactory::FileWizard)
-                             ?  QSet<Core::Id>() : QSet<Core::Id>() << "UNKNOWN_PROJECT");
+                             ?  QSet<Utils::Id>() : QSet<Utils::Id>() << "UNKNOWN_PROJECT");
     setIcon(p->icon);
     setDescription(p->description);
     setDisplayName(p->displayName);
@@ -198,7 +198,7 @@ static inline bool createFile(CustomWizardFile cwFile,
     const QFile::OpenMode openMode
             = cwFile.binary ? QIODevice::ReadOnly : (QIODevice::ReadOnly|QIODevice::Text);
     Utils::FileReader reader;
-    if (!reader.fetch(sourcePath, openMode, errorMessage))
+    if (!reader.fetch(Utils::FilePath::fromString(sourcePath), openMode, errorMessage))
         return false;
 
     Core::GeneratedFile generatedFile;
@@ -213,7 +213,7 @@ static inline bool createFile(CustomWizardFile cwFile,
         generatedFile.setContents(CustomWizardContext::processFile(fm, contentsIn));
     }
 
-    Core::GeneratedFile::Attributes attributes = {};
+    Core::GeneratedFile::Attributes attributes;
     if (cwFile.openEditor)
         attributes |= Core::GeneratedFile::OpenEditorAttribute;
     if (cwFile.openProject)
@@ -383,13 +383,9 @@ QList<Core::IWizardFactory *> CustomWizard::createWizards()
 {
     QString errorMessage;
     QString verboseLog;
-    const QString templateDirName = Core::ICore::resourcePath() +
-                                    QLatin1Char('/') + QLatin1String(templatePathC);
-
-
-    const QString userTemplateDirName = Core::ICore::userResourcePath() +
-                                        QLatin1Char('/') + QLatin1String(templatePathC);
 
+    const QString templateDirName = Core::ICore::resourcePath(templatePathC).toString();
+    const QString userTemplateDirName = Core::ICore::userResourcePath(templatePathC).toString();
 
     const QDir templateDir(templateDirName);
     if (CustomWizardPrivate::verbose)
@@ -406,7 +402,7 @@ QList<Core::IWizardFactory *> CustomWizard::createWizards()
 
     const QDir::Filters filters = QDir::Dirs|QDir::Readable|QDir::NoDotAndDotDot;
     const QDir::SortFlags sortflags = QDir::Name|QDir::IgnoreCase;
-    QList<QFileInfo> dirs;
+    QFileInfoList dirs;
     if (userTemplateDir.exists()) {
         if (CustomWizardPrivate::verbose)
             verboseLog += QString::fromLatin1("### CustomWizard: userTemplateDir \"%1\" found, adding\n").arg(userTemplateDirName);
@@ -446,7 +442,7 @@ QList<Core::IWizardFactory *> CustomWizard::createWizards()
                 break;
             }
         } else {
-            QList<QFileInfo> subDirs = dir.entryInfoList(filters, sortflags);
+            QFileInfoList subDirs = dir.entryInfoList(filters, sortflags);
             if (!subDirs.isEmpty()) {
                 // There is no QList::prepend(QList)...
                 dirs.swap(subDirs);
@@ -470,7 +466,7 @@ QList<Core::IWizardFactory *> CustomWizard::createWizards()
 
     if (CustomWizardPrivate::verbose) { // Print to output pane for Windows.
         qWarning("%s", qPrintable(verboseLog));
-        Core::MessageManager::write(verboseLog, Core::MessageManager::ModeSwitch);
+        Core::MessageManager::writeDisrupting(verboseLog);
     }
     return rc;
 }
