@@ -203,7 +203,7 @@ void WorkingDirectoryAspect::addToLayout(LayoutBuilder &builder)
     builder.addItems({tr("Working directory:"), m_chooser.data(), m_resetButton.data()});
 }
 
-void WorkingDirectoryAspect::acquaintSiblings(const BaseAspects &siblings)
+void WorkingDirectoryAspect::acquaintSiblings(const AspectContainer &siblings)
 {
     m_envAspect = siblings.aspect<EnvironmentAspect>();
 }
@@ -235,8 +235,8 @@ void WorkingDirectoryAspect::toMap(QVariantMap &data) const
 {
     const QString wd = m_workingDirectory == m_defaultWorkingDirectory
         ? QString() : m_workingDirectory.toString();
-    saveToMap(data, wd, QString());
-    saveToMap(data, m_defaultWorkingDirectory.toString(), QString(), ".default");
+    saveToMap(data, wd, QString(), settingsKey());
+    saveToMap(data, m_defaultWorkingDirectory.toString(), QString(), settingsKey() + ".default");
 }
 
 /*!
@@ -246,12 +246,14 @@ void WorkingDirectoryAspect::toMap(QVariantMap &data) const
 */
 FilePath WorkingDirectoryAspect::workingDirectory(const MacroExpander *expander) const
 {
-    const Utils::Environment env = m_envAspect ? m_envAspect->environment()
-                                               : Utils::Environment::systemEnvironment();
-    QString workingDir = m_workingDirectory.toUserOutput();
+    const Environment env = m_envAspect ? m_envAspect->environment()
+                                        : Environment::systemEnvironment();
+    FilePath res = m_workingDirectory;
+    QString workingDir = m_workingDirectory.path();
     if (expander)
         workingDir = expander->expandProcessArgs(workingDir);
-    return FilePath::fromString(PathChooser::expandedDirectory(workingDir, env, QString()));
+    res.setPath(PathChooser::expandedDirectory(workingDir, env, QString()));
+    return res;
 }
 
 FilePath WorkingDirectoryAspect::defaultWorkingDirectory() const
@@ -392,7 +394,7 @@ void ArgumentsAspect::fromMap(const QVariantMap &map)
     QVariant args = map.value(settingsKey());
     // Until 3.7 a QStringList was stored for Remote Linux
     if (args.type() == QVariant::StringList)
-        m_arguments = QtcProcess::joinArgs(args.toStringList(), OsTypeLinux);
+        m_arguments = ProcessArgs::joinArgs(args.toStringList(), OsTypeLinux);
     else
         m_arguments = args.toString();
 
@@ -411,8 +413,8 @@ void ArgumentsAspect::fromMap(const QVariantMap &map)
 */
 void ArgumentsAspect::toMap(QVariantMap &map) const
 {
-    saveToMap(map, m_arguments, QString());
-    saveToMap(map, m_multiLine, false, ".multi");
+    saveToMap(map, m_arguments, QString(), settingsKey());
+    saveToMap(map, m_multiLine, false, settingsKey() + ".multi");
 }
 
 /*!
