@@ -29,25 +29,28 @@
 #include "buildconfiguration.h"
 #include "buildinfo.h"
 #include "buildmanager.h"
+#include "buildsystem.h"
 #include "buildtargetinfo.h"
 #include "deployconfiguration.h"
 #include "deploymentdata.h"
 #include "devicesupport/devicemanager.h"
+#include "environmentaspect.h"
 #include "kit.h"
 #include "kitinformation.h"
 #include "kitmanager.h"
+#include "miniprojecttargetselector.h"
 #include "project.h"
 #include "projectexplorer.h"
 #include "projectexplorericons.h"
 #include "projectexplorersettings.h"
 #include "runconfiguration.h"
+#include "runconfigurationaspects.h"
 #include "session.h"
 
 #include <coreplugin/coreconstants.h>
 
-#include <extensionsystem/pluginmanager.h>
-
 #include <utils/algorithm.h>
+#include <utils/commandline.h>
 #include <utils/macroexpander.h>
 #include <utils/qtcassert.h>
 #include <utils/stringutils.h>
@@ -98,25 +101,30 @@ public:
         m_runConfigurationModel(t)
     { }
 
-    bool m_isEnabled = true;
+    ~TargetPrivate()
+    {
+        delete m_buildSystem;
+    }
+
     QIcon m_overlayIcon;
 
     QList<BuildConfiguration *> m_buildConfigurations;
-    BuildConfiguration *m_activeBuildConfiguration = nullptr;
+    QPointer<BuildConfiguration> m_activeBuildConfiguration;
     QList<DeployConfiguration *> m_deployConfigurations;
     DeployConfiguration *m_activeDeployConfiguration = nullptr;
     QList<RunConfiguration *> m_runConfigurations;
     RunConfiguration* m_activeRunConfiguration = nullptr;
-    DeploymentData m_deploymentData;
-    QList<BuildTargetInfo> m_appTargets;
     QVariantMap m_pluginSettings;
 
     Kit *const m_kit;
     MacroExpander m_macroExpander;
+    BuildSystem *m_buildSystem = nullptr;
 
     ProjectConfigurationModel m_buildConfigurationModel;
     ProjectConfigurationModel m_deployConfigurationModel;
     ProjectConfigurationModel m_runConfigurationModel;
+
+    bool m_shuttingDown = false;
 };
 
 
@@ -124,33 +132,76 @@ Target::Target(Project *project, Kit *k, _constructor_tag) :
     QObject(project),
     d(std::make_unique<TargetPrivate>(this, k))
 {
+    // Note: nullptr is a valid state for the per-buildConfig systems.
+    d->m_buildSystem = project->createBuildSystem(this);
+
     QTC_CHECK(d->m_kit);
     connect(DeviceManager::instance(), &DeviceManager::updated, this, &Target::updateDeviceState);
-    connect(project, &Project::parsingFinished, this, [this](bool success) {
-        if (success && this->project() == SessionManager::startupProject()
-                && this == this->project()->activeTarget()) {
+
+    connect(this, &Target::parsingStarted, this, [this, project] {
+        project->anyParsingStarted(this);
+    });
+
+    connect(this, &Target::parsingFinished, this, [this, project](bool success) {
+        if (success && this == SessionManager::startupTarget())
             updateDefaultRunConfigurations();
-        }
+        // For testing.
+        emit SessionManager::instance()->projectFinishedParsing(project);
+        project->anyParsingFinished(this, success);
     }, Qt::QueuedConnection); // Must wait for run configs to change their enabled state.
 
     KitManager *km = KitManager::instance();
     connect(km, &KitManager::kitUpdated, this, &Target::handleKitUpdates);
     connect(km, &KitManager::kitRemoved, this, &Target::handleKitRemoval);
 
-    Utils::MacroExpander *expander = macroExpander();
-    expander->setDisplayName(tr("Target Settings"));
-    expander->setAccumulating(true);
+    d->m_macroExpander.setDisplayName(tr("Target Settings"));
+    d->m_macroExpander.setAccumulating(true);
 
-    expander->registerSubProvider([this] { return kit()->macroExpander(); });
+    d->m_macroExpander.registerSubProvider([this] { return kit()->macroExpander(); });
 
-    expander->registerVariable("sourceDir", tr("Source directory"),
+    d->m_macroExpander.registerVariable("sourceDir", tr("Source directory"),
             [project] { return project->projectDirectory().toUserOutput(); });
 
-    // Legacy support.
-    expander->registerVariable(Constants::VAR_CURRENTPROJECT_NAME,
+    // TODO: Remove in ~4.16.
+    d->m_macroExpander.registerVariable(Constants::VAR_CURRENTPROJECT_NAME,
             QCoreApplication::translate("ProjectExplorer", "Name of current project"),
             [project] { return project->displayName(); },
             false);
+    d->m_macroExpander.registerVariable("Project:Name",
+            QCoreApplication::translate("ProjectExplorer", "Name of current project"),
+            [project] { return project->displayName(); });
+
+    d->m_macroExpander.registerVariable("CurrentRun:Name",
+        tr("The currently active run configuration's name."),
+        [this]() -> QString {
+            if (RunConfiguration * const rc = activeRunConfiguration())
+                return rc->displayName();
+            return QString();
+        }, false);
+    d->m_macroExpander.registerFileVariables("CurrentRun:Executable",
+        tr("The currently active run configuration's executable (if applicable)."),
+        [this]() -> FilePath {
+            if (RunConfiguration * const rc = activeRunConfiguration())
+                return rc->commandLine().executable();
+            return FilePath();
+        }, false);
+    d->m_macroExpander.registerPrefix("CurrentRun:Env", tr("Variables in the current run environment."),
+                             [this](const QString &var) {
+        if (RunConfiguration * const rc = activeRunConfiguration()) {
+            if (const auto envAspect = rc->aspect<EnvironmentAspect>())
+                return envAspect->environment().expandedValueForKey(var);
+        }
+        return QString();
+    }, false);
+    d->m_macroExpander.registerVariable("CurrentRun:WorkingDir",
+                               tr("The currently active run configuration's working directory."),
+                               [this] {
+        if (RunConfiguration * const rc = activeRunConfiguration()) {
+            if (const auto wdAspect = rc->aspect<WorkingDirectoryAspect>())
+                return wdAspect->workingDirectory(&d->m_macroExpander).toString();
+        }
+        return QString();
+    }, false);
 }
 
 Target::~Target()
@@ -184,6 +235,16 @@ bool Target::isActive() const
     return project()->activeTarget() == this;
 }
 
+void Target::markAsShuttingDown()
+{
+    d->m_shuttingDown = true;
+}
+
+bool Target::isShuttingDown() const
+{
+    return d->m_shuttingDown;
+}
+
 Project *Target::project() const
 {
     return static_cast<Project *>(parent());
@@ -194,7 +255,49 @@ Kit *Target::kit() const
     return d->m_kit;
 }
 
-Core::Id Target::id() const
+BuildSystem *Target::buildSystem() const
+{
+    if (d->m_activeBuildConfiguration)
+        return d->m_activeBuildConfiguration->buildSystem();
+
+    return d->m_buildSystem;
+}
+
+BuildSystem *Target::fallbackBuildSystem() const
+{
+    return d->m_buildSystem;
+}
+
+DeploymentData Target::deploymentData() const
+{
+    const DeployConfiguration * const dc = activeDeployConfiguration();
+    if (dc && dc->usesCustomDeploymentData())
+        return dc->customDeploymentData();
+    return buildSystemDeploymentData();
+}
+
+DeploymentData Target::buildSystemDeploymentData() const
+{
+    QTC_ASSERT(buildSystem(), return {});
+    return buildSystem()->deploymentData();
+}
+
+BuildTargetInfo Target::buildTarget(const QString &buildKey) const
+{
+    QTC_ASSERT(buildSystem(), return {});
+    return buildSystem()->buildTarget(buildKey);
+}
+
+QString Target::activeBuildKey() const
+{
+    // Should not happen. If it does, return a buildKey that wont be found in
+    // the project tree, so that the project()->findNodeForBuildKey(buildKey)
+    // returns null.
+    QTC_ASSERT(d->m_activeRunConfiguration, return QString(QChar(0)));
+    return d->m_activeRunConfiguration->buildKey();
+}
+
+Utils::Id Target::id() const
 {
     return d->m_kit->id();
 }
@@ -209,6 +312,16 @@ QString Target::toolTip() const
     return d->m_kit->toHtml();
 }
 
+QString Target::displayNameKey()
+{
+    return QString("ProjectExplorer.ProjectConfiguration.DisplayName");
+}
+
+QString Target::deviceTypeKey()
+{
+    return QString("DeviceType");
+}
+
 void Target::addBuildConfiguration(BuildConfiguration *bc)
 {
     QTC_ASSERT(bc && !d->m_buildConfigurations.contains(bc), return);
@@ -228,7 +341,7 @@ void Target::addBuildConfiguration(BuildConfiguration *bc)
     // add it
     d->m_buildConfigurations.push_back(bc);
 
-    project()->addedProjectConfiguration(bc);
+    ProjectExplorerPlugin::targetSelector()->addedBuildConfiguration(bc);
     emit addedBuildConfiguration(bc);
     d->m_buildConfigurationModel.addProjectConfiguration(bc);
 
@@ -255,14 +368,14 @@ bool Target::removeBuildConfiguration(BuildConfiguration *bc)
     }
 
     emit removedBuildConfiguration(bc);
-    project()->removedProjectConfiguration(bc);
+    ProjectExplorerPlugin::targetSelector()->removedBuildConfiguration(bc);
     d->m_buildConfigurationModel.removeProjectConfiguration(bc);
 
     delete bc;
     return true;
 }
 
-QList<BuildConfiguration *> Target::buildConfigurations() const
+const QList<BuildConfiguration *> Target::buildConfigurations() const
 {
     return d->m_buildConfigurations;
 }
@@ -278,8 +391,8 @@ void Target::setActiveBuildConfiguration(BuildConfiguration *bc)
         (bc && d->m_buildConfigurations.contains(bc) &&
          bc != d->m_activeBuildConfiguration)) {
         d->m_activeBuildConfiguration = bc;
-        project()->activeBuildConfigurationChanged(d->m_activeBuildConfiguration);
         emit activeBuildConfigurationChanged(d->m_activeBuildConfiguration);
+        ProjectExplorerPlugin::updateActions();
     }
 }
 
@@ -297,7 +410,7 @@ void Target::addDeployConfiguration(DeployConfiguration *dc)
     // add it
     d->m_deployConfigurations.push_back(dc);
 
-    project()->addedProjectConfiguration(dc);
+    ProjectExplorerPlugin::targetSelector()->addedDeployConfiguration(dc);
     d->m_deployConfigurationModel.addProjectConfiguration(dc);
     emit addedDeployConfiguration(dc);
 
@@ -325,7 +438,7 @@ bool Target::removeDeployConfiguration(DeployConfiguration *dc)
                                                          SetActive::Cascade);
     }
 
-    project()->removedProjectConfiguration(dc);
+    ProjectExplorerPlugin::targetSelector()->removedDeployConfiguration(dc);
     d->m_deployConfigurationModel.removeProjectConfiguration(dc);
     emit removedDeployConfiguration(dc);
 
@@ -333,7 +446,7 @@ bool Target::removeDeployConfiguration(DeployConfiguration *dc)
     return true;
 }
 
-QList<DeployConfiguration *> Target::deployConfigurations() const
+const QList<DeployConfiguration *> Target::deployConfigurations() const
 {
     return d->m_deployConfigurations;
 }
@@ -354,50 +467,7 @@ void Target::setActiveDeployConfiguration(DeployConfiguration *dc)
     updateDeviceState();
 }
 
-void Target::setDeploymentData(const DeploymentData &deploymentData)
-{
-    if (d->m_deploymentData != deploymentData) {
-        d->m_deploymentData = deploymentData;
-        emit deploymentDataChanged();
-        emit applicationTargetsChanged();
-    }
-}
-
-DeploymentData Target::deploymentData() const
-{
-    return d->m_deploymentData;
-}
-
-void Target::setApplicationTargets(const QList<BuildTargetInfo> &appTargets)
-{
-    if (Utils::toSet(appTargets) != Utils::toSet(d->m_appTargets)) {
-        d->m_appTargets = appTargets;
-        emit applicationTargetsChanged();
-    }
-}
-
-const QList<BuildTargetInfo> Target::applicationTargets() const
-{
-    return d->m_appTargets;
-}
-
-BuildTargetInfo Target::buildTarget(const QString &buildKey) const
-{
-    return Utils::findOrDefault(d->m_appTargets, [&buildKey](const BuildTargetInfo &ti) {
-        return ti.buildKey == buildKey;
-    });
-}
-
-QList<ProjectConfiguration *> Target::projectConfigurations() const
-{
-    QList<ProjectConfiguration *> result;
-    result.append(Utils::static_container_cast<ProjectConfiguration *>(buildConfigurations()));
-    result.append(Utils::static_container_cast<ProjectConfiguration *>(deployConfigurations()));
-    result.append(Utils::static_container_cast<ProjectConfiguration *>(runConfigurations()));
-    return result;
-}
-
-QList<RunConfiguration *> Target::runConfigurations() const
+const QList<RunConfiguration *> Target::runConfigurations() const
 {
     return d->m_runConfigurations;
 }
@@ -419,7 +489,7 @@ void Target::addRunConfiguration(RunConfiguration *rc)
 
     d->m_runConfigurations.push_back(rc);
 
-    project()->addedProjectConfiguration(rc);
+    ProjectExplorerPlugin::targetSelector()->addedRunConfiguration(rc);
     d->m_runConfigurationModel.addProjectConfiguration(rc);
     emit addedRunConfiguration(rc);
 
@@ -441,7 +511,7 @@ void Target::removeRunConfiguration(RunConfiguration *rc)
     }
 
     emit removedRunConfiguration(rc);
-    project()->removedProjectConfiguration(rc);
+    ProjectExplorerPlugin::targetSelector()->removedRunConfiguration(rc);
     d->m_runConfigurationModel.removeProjectConfiguration(rc);
 
     delete rc;
@@ -454,20 +524,19 @@ RunConfiguration *Target::activeRunConfiguration() const
 
 void Target::setActiveRunConfiguration(RunConfiguration *rc)
 {
+    if (isShuttingDown())
+        return;
+
     if ((!rc && d->m_runConfigurations.isEmpty()) ||
         (rc && d->m_runConfigurations.contains(rc) &&
          rc != d->m_activeRunConfiguration)) {
         d->m_activeRunConfiguration = rc;
         emit activeRunConfigurationChanged(d->m_activeRunConfiguration);
+        ProjectExplorerPlugin::updateActions();
     }
     updateDeviceState();
 }
 
-bool Target::isEnabled() const
-{
-    return d->m_isEnabled;
-}
-
 QIcon Target::icon() const
 {
     return d->m_kit->icon();
@@ -496,15 +565,15 @@ QVariantMap Target::toMap() const
         return QVariantMap();
 
     QVariantMap map;
+    map.insert(displayNameKey(), displayName());
+    map.insert(deviceTypeKey(), DeviceTypeKitAspect::deviceTypeId(kit()).toSetting());
 
     {
         // FIXME: For compatibility within the 4.11 cycle, remove this block later.
         // This is only read by older versions of Creator, but even there not actively used.
         const char CONFIGURATION_ID_KEY[] = "ProjectExplorer.ProjectConfiguration.Id";
-        const char DISPLAY_NAME_KEY[] = "ProjectExplorer.ProjectConfiguration.DisplayName";
         const char DEFAULT_DISPLAY_NAME_KEY[] = "ProjectExplorer.ProjectConfiguration.DefaultDisplayName";
         map.insert(QLatin1String(CONFIGURATION_ID_KEY), id().toSetting());
-        map.insert(QLatin1String(DISPLAY_NAME_KEY), displayName());
         map.insert(QLatin1String(DEFAULT_DISPLAY_NAME_KEY), displayName());
     }
 
@@ -526,7 +595,8 @@ QVariantMap Target::toMap() const
     for (int i = 0; i < rcs.size(); ++i)
         map.insert(QString::fromLatin1(RC_KEY_PREFIX) + QString::number(i), rcs.at(i)->toMap());
 
-    map.insert(QLatin1String(PLUGIN_SETTINGS_KEY), d->m_pluginSettings);
+    if (!d->m_pluginSettings.isEmpty())
+        map.insert(QLatin1String(PLUGIN_SETTINGS_KEY), d->m_pluginSettings);
 
     return map;
 }
@@ -552,12 +622,12 @@ void Target::updateDefaultDeployConfigurations()
         return;
     }
 
-    QList<Core::Id> dcIds;
+    QList<Utils::Id> dcIds;
     foreach (DeployConfigurationFactory *dcFactory, dcFactories)
         dcIds.append(dcFactory->creationId());
 
     QList<DeployConfiguration *> dcList = deployConfigurations();
-    QList<Core::Id> toCreate = dcIds;
+    QList<Utils::Id> toCreate = dcIds;
 
     foreach (DeployConfiguration *dc, dcList) {
         if (dcIds.contains(dc->id()))
@@ -566,7 +636,7 @@ void Target::updateDefaultDeployConfigurations()
             removeDeployConfiguration(dc);
     }
 
-    foreach (Core::Id id, toCreate) {
+    foreach (Utils::Id id, toCreate) {
         foreach (DeployConfigurationFactory *dcFactory, dcFactories) {
             if (dcFactory->creationId() == id) {
                 DeployConfiguration *dc = dcFactory->create(this);
@@ -609,12 +679,12 @@ void Target::updateDefaultRunConfigurations()
         bool present = false;
         for (const RunConfigurationCreationInfo &item : creators) {
             QString buildKey = rc->buildKey();
-            if (item.id == rc->id() && item.buildKey == buildKey) {
+            if (item.factory->runConfigurationId() == rc->id() && item.buildKey == buildKey) {
                 existing.append(item);
                 present = true;
             }
         }
-        if (!present && project()->knowsAllBuildExecutables())
+        if (!present)
             toRemove.append(rc);
     }
     configuredCount -= toRemove.count();
@@ -627,7 +697,7 @@ void Target::updateDefaultRunConfigurations()
                 continue;
             bool exists = false;
             for (const RunConfigurationCreationInfo &ex : existing) {
-                if (ex.id == item.id && ex.buildKey == item.buildKey)
+                if (ex.factory == item.factory && ex.buildKey == item.buildKey)
                     exists = true;
             }
             if (exists)
@@ -636,7 +706,7 @@ void Target::updateDefaultRunConfigurations()
             RunConfiguration *rc = item.create(this);
             if (!rc)
                 continue;
-            QTC_CHECK(rc->id() == item.id);
+            QTC_CHECK(rc->id() == item.factory->runConfigurationId());
             if (!rc->isConfigured())
                 newUnconfigured << rc;
             else
@@ -720,9 +790,12 @@ void Target::setNamedSettings(const QString &name, const QVariant &value)
         d->m_pluginSettings.insert(name, value);
 }
 
-QVariant Target::additionalData(Core::Id id) const
+QVariant Target::additionalData(Utils::Id id) const
 {
-    return project()->additionalData(id, this);
+    if (const BuildSystem *bs = buildSystem())
+        return bs->additionalData(id);
+
+    return {};
 }
 
 MakeInstallCommand Target::makeInstallCommand(const QString &installRoot) const
@@ -784,15 +857,6 @@ void Target::updateDeviceState()
     setOverlayIcon(overlay);
 }
 
-void Target::setEnabled(bool enabled)
-{
-    if (enabled == d->m_isEnabled)
-        return;
-
-    d->m_isEnabled = enabled;
-    emit targetEnabled(d->m_isEnabled);
-}
-
 bool Target::fromMap(const QVariantMap &map)
 {
     QTC_ASSERT(d->m_kit == KitManager::kit(id()), return false);
@@ -841,7 +905,7 @@ bool Target::fromMap(const QVariantMap &map)
         QVariantMap valueMap = map.value(key).toMap();
         DeployConfiguration *dc = DeployConfigurationFactory::restore(this, valueMap);
         if (!dc) {
-            Core::Id id = idFromMap(valueMap);
+            Utils::Id id = idFromMap(valueMap);
             qWarning("No factory found to restore deployment configuration of id '%s'!",
                      id.isValid() ? qPrintable(id.toString()) : "UNKNOWN");
             continue;
@@ -871,7 +935,7 @@ bool Target::fromMap(const QVariantMap &map)
         RunConfiguration *rc = RunConfigurationFactory::restore(this, valueMap);
         if (!rc)
             continue;
-        const Core::Id theIdFromMap = ProjectExplorer::idFromMap(valueMap);
+        const Utils::Id theIdFromMap = ProjectExplorer::idFromMap(valueMap);
         if (!theIdFromMap.toString().contains("///::///")) { // Hack for cmake 4.10 -> 4.11
             QTC_CHECK(rc->id().withSuffix(rc->buildKey()) == theIdFromMap);
         }
