@@ -28,10 +28,11 @@
 
 #include <abstractview.h>
 #include <nodemetainfo.h>
-#include <qmldesignerplugin.h>
-#include <qmlobjectnode.h>
-#include <qmlmodelnodeproxy.h>
 #include <rewritingexception.h>
+#include <qmldesignerplugin.h>
+#include <qmlmodelnodeproxy.h>
+#include <qmlobjectnode.h>
+#include <qmltimeline.h>
 
 #include <coreplugin/messagebox.h>
 #include <utils/algorithm.h>
@@ -42,6 +43,7 @@
 #include <QFontDatabase>
 #include <QMessageBox>
 #include <QQmlContext>
+#include <QWindow>
 
 #include <coreplugin/icore.h>
 
@@ -88,16 +90,14 @@ QColor convertColorFromString(const QString &s)
 
 namespace QmlDesigner {
 
-PropertyEditorContextObject::PropertyEditorContextObject(QObject *parent) :
-    QObject(parent),
-    m_isBaseState(false),
-    m_selectionChanged(false),
-    m_backendValues(nullptr),
-    m_qmlComponent(nullptr),
-    m_qmlContext(nullptr)
-{
-
-}
+PropertyEditorContextObject::PropertyEditorContextObject(QObject *parent)
+    : QObject(parent)
+    , m_isBaseState(false)
+    , m_selectionChanged(false)
+    , m_backendValues(nullptr)
+    , m_qmlComponent(nullptr)
+    , m_qmlContext(nullptr)
+{}
 
 QString PropertyEditorContextObject::convertColorToString(const QVariant &color)
 {
@@ -179,6 +179,21 @@ void PropertyEditorContextObject::toogleExportAlias()
     }
 }
 
+void PropertyEditorContextObject::goIntoComponent()
+{
+    QTC_ASSERT(m_model && m_model->rewriterView(), return);
+
+    /* Ideally we should not missuse the rewriterView
+     * If we add more code here we have to forward the property editor view */
+    RewriterView *rewriterView = m_model->rewriterView();
+
+    QTC_ASSERT(!rewriterView->selectedModelNodes().isEmpty(), return);
+
+    const ModelNode selectedNode = rewriterView->selectedModelNodes().constFirst();
+
+    DocumentManager::goIntoComponent(selectedNode);
+}
+
 void PropertyEditorContextObject::changeTypeName(const QString &typeName)
 {
     QTC_ASSERT(m_model && m_model->rewriterView(), return);
@@ -247,7 +262,7 @@ void PropertyEditorContextObject::changeTypeName(const QString &typeName)
         if (!incompatibleProperties.empty()) {
             QString detailedText = QString("<b>Incompatible properties:</b><br>");
 
-            for (const auto &p : incompatibleProperties)
+            for (const auto &p : qAsConst(incompatibleProperties))
                 detailedText.append("- " + QString::fromUtf8(p) + "<br>");
 
             detailedText.chop(QString("<br>").size());
@@ -267,7 +282,7 @@ void PropertyEditorContextObject::changeTypeName(const QString &typeName)
             if (msgBox.exec() == QMessageBox::Cancel)
                 return;
 
-            for (auto p : incompatibleProperties)
+            for (const auto &p : qAsConst(incompatibleProperties))
                 selectedNode.removeProperty(p);
         }
 
@@ -286,6 +301,9 @@ void PropertyEditorContextObject::insertKeyframe(const QString &propertyName)
 {
     QTC_ASSERT(m_model && m_model->rewriterView(), return);
 
+    if (isBlocked(propertyName))
+        return;
+
     /* Ideally we should not missuse the rewriterView
      * If we add more code here we have to forward the property editor view */
     RewriterView *rewriterView = m_model->rewriterView();
@@ -294,9 +312,12 @@ void PropertyEditorContextObject::insertKeyframe(const QString &propertyName)
 
     ModelNode selectedNode = rewriterView->selectedModelNodes().constFirst();
 
-    rewriterView->emitCustomNotification("INSERT_KEYFRAME",
-                                         { selectedNode },
-                                         { propertyName });
+    QmlTimeline timeline = rewriterView->currentTimeline();
+
+    QTC_ASSERT(timeline.isValid(), return );
+    QTC_ASSERT(selectedNode.isValid(), return );
+
+    timeline.insertKeyframe(selectedNode, propertyName.toUtf8());
 }
 
 int PropertyEditorContextObject::majorVersion() const
@@ -454,7 +475,7 @@ void PropertyEditorContextObject::setIsBaseState(bool newIsBaseState)
 
 void PropertyEditorContextObject::setSelectionChanged(bool newSelectionChanged)
 {
-    if (newSelectionChanged ==  m_selectionChanged)
+    if (newSelectionChanged == m_selectionChanged)
         return;
 
     m_selectionChanged = newSelectionChanged;
@@ -463,7 +484,7 @@ void PropertyEditorContextObject::setSelectionChanged(bool newSelectionChanged)
 
 void PropertyEditorContextObject::setBackendValues(QQmlPropertyMap *newBackendValues)
 {
-    if (newBackendValues ==  m_backendValues)
+    if (newBackendValues == m_backendValues)
         return;
 
     m_backendValues = newBackendValues;
@@ -495,7 +516,9 @@ void PropertyEditorContextObject::hideCursor()
         return;
 
     QApplication::setOverrideCursor(QCursor(Qt::BlankCursor));
-    m_lastPos = QCursor::pos();
+
+    if (QWidget *w = QApplication::activeWindow())
+        m_lastPos = QCursor::pos(w->screen());
 }
 
 void PropertyEditorContextObject::restoreCursor()
@@ -503,8 +526,19 @@ void PropertyEditorContextObject::restoreCursor()
     if (!QApplication::overrideCursor())
         return;
 
-    QCursor::setPos(m_lastPos);
     QApplication::restoreOverrideCursor();
+
+    if (QWidget *w = QApplication::activeWindow())
+        QCursor::setPos(w->screen(), m_lastPos);
+}
+
+void PropertyEditorContextObject::holdCursorInPlace()
+{
+    if (!QApplication::overrideCursor())
+        return;
+
+    if (QWidget *w = QApplication::activeWindow())
+        QCursor::setPos(w->screen(), m_lastPos);
 }
 
 QStringList PropertyEditorContextObject::styleNamesForFamily(const QString &family)
@@ -524,6 +558,20 @@ QStringList PropertyEditorContextObject::allStatesForId(const QString &id)
       return {};
 }
 
+bool PropertyEditorContextObject::isBlocked(const QString &propName) const
+{
+    if (m_model && m_model->rewriterView()) {
+        const QList<ModelNode> nodes = m_model->rewriterView()->selectedModelNodes();
+        QScopedPointer<QmlObjectNode> objNode;
+        for (const auto &node : nodes) {
+            objNode.reset(QmlObjectNode::getQmlObjectNodeOfCorrectType(node));
+            if (objNode->isBlocked(propName.toUtf8()))
+                return true;
+        }
+    }
+    return false;
+}
+
 void EasingCurveEditor::registerDeclarativeType()
 {
      qmlRegisterType<EasingCurveEditor>("HelperWidgets", 2, 0, "EasingCurveEditor");
