@@ -36,24 +36,29 @@
 
 #include <app/app_version.h>
 
+#include <coreplugin/messagebox.h>
+#include <coreplugin/icore.h>
+
 #include <projectexplorer/kit.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/target.h>
 #include <projectexplorer/toolchain.h>
-#include <qmakeprojectmanager/qmakeproject.h>
-#include <qmakeprojectmanager/qmakenodes.h>
-#include <coreplugin/messagebox.h>
-#include <coreplugin/icore.h>
+
+#include <qmlprojectmanager/qmlmultilanguageaspect.h>
+
+#include <qmlprojectmanager/qmlproject.h>
 #include <qtsupport/baseqtversion.h>
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtsupportconstants.h>
-#include <coreplugin/icore.h>
 
 #include <utils/algorithm.h>
 #include <utils/environment.h>
 #include <utils/hostosinfo.h>
+#include <utils/qtcassert.h>
 #include <utils/temporarydirectory.h>
 
+#include <QApplication>
 #include <QProcess>
 #include <QCoreApplication>
 #include <QCryptographicHash>
@@ -64,8 +69,10 @@
 #include <QThread>
 #include <QSettings>
 
-static Q_LOGGING_CATEGORY(puppetStart, "qtc.puppet.start")
-static Q_LOGGING_CATEGORY(puppetBuild, "qtc.puppet.build")
+static Q_LOGGING_CATEGORY(puppetStart, "qtc.puppet.start", QtWarningMsg)
+static Q_LOGGING_CATEGORY(puppetBuild, "qtc.puppet.build", QtWarningMsg)
+
+using namespace ProjectExplorer;
 
 namespace QmlDesigner {
 
@@ -85,13 +92,13 @@ public:
     }
 };
 
-QHash<Core::Id, PuppetCreator::PuppetType> PuppetCreator::m_qml2PuppetForKitPuppetHash;
+QHash<Utils::Id, PuppetCreator::PuppetType> PuppetCreator::m_qml2PuppetForKitPuppetHash;
 
 QByteArray PuppetCreator::qtHash() const
 {
-    QtSupport::BaseQtVersion *currentQtVersion = QtSupport::QtKitInformation::qtVersion(m_kit);
+    QtSupport::BaseQtVersion *currentQtVersion = QtSupport::QtKitAspect::qtVersion(m_target->kit());
     if (currentQtVersion) {
-        return QCryptographicHash::hash(currentQtVersion->qmakeProperty("QT_INSTALL_DATA").toUtf8(),
+        return QCryptographicHash::hash(currentQtVersion->dataPath().toString().toUtf8(),
                                         QCryptographicHash::Sha1)
                 .toBase64(QByteArray::Base64UrlEncoding | QByteArray::OmitTrailingEquals);
     }
@@ -101,9 +108,9 @@ QByteArray PuppetCreator::qtHash() const
 
 QDateTime PuppetCreator::qtLastModified() const
 {
-    QtSupport::BaseQtVersion *currentQtVersion = QtSupport::QtKitInformation::qtVersion(m_kit);
+    QtSupport::BaseQtVersion *currentQtVersion = QtSupport::QtKitAspect::qtVersion(m_target->kit());
     if (currentQtVersion)
-        return QFileInfo(currentQtVersion->qmakeProperty("QT_INSTALL_LIBS")).lastModified();
+        return currentQtVersion->libraryPath().lastModified();
 
     return QDateTime();
 }
@@ -139,10 +146,10 @@ QDateTime PuppetCreator::puppetSourceLastModified() const
 bool PuppetCreator::useOnlyFallbackPuppet() const
 {
 #ifndef QMLDESIGNER_TEST
-    if (!m_kit || !m_kit->isValid())
+    if (!m_target || !m_target->kit()->isValid())
         qWarning() << "Invalid kit for QML puppet";
-    return m_designerSettings.value(DesignerSettingsKey::USE_ONLY_FALLBACK_PUPPET
-                                    ).toBool() || m_kit == 0 || !m_kit->isValid();
+    return m_designerSettings.value(DesignerSettingsKey::USE_DEFAULT_PUPPET).toBool()
+            || m_target == nullptr || !m_target->kit()->isValid();
 #else
     return true;
 #endif
@@ -151,8 +158,8 @@ bool PuppetCreator::useOnlyFallbackPuppet() const
 QString PuppetCreator::getStyleConfigFileName() const
 {
 #ifndef QMLDESIGNER_TEST
-    if (m_currentProject) {
-        for (const Utils::FileName &fileName : m_currentProject->files(ProjectExplorer::Project::SourceFiles)) {
+    if (m_target) {
+        for (const Utils::FilePath &fileName : m_target->project()->files(ProjectExplorer::Project::SourceFiles)) {
             if (fileName.fileName() == "qtquickcontrols2.conf")
                 return  fileName.toString();
         }
@@ -161,50 +168,67 @@ QString PuppetCreator::getStyleConfigFileName() const
     return QString();
 }
 
-PuppetCreator::PuppetCreator(ProjectExplorer::Kit *kit,
-                             ProjectExplorer::Project *project,
-                             const Model *model)
+bool PuppetCreator::usesVirtualKeyboard() const
+{
+#ifndef QMLDESIGNER_TEST
+    if (m_target) {
+        auto *qmlbuild = qobject_cast<QmlProjectManager::QmlBuildSystem *>(m_target->buildSystem());
 
-    : m_kit(kit)
+        const Utils::EnvironmentItem virtualKeyboard("QT_IM_MODULE", "qtvirtualkeyboard");
+        return qmlbuild && qmlbuild->environment().indexOf(virtualKeyboard);
+    }
+
+#endif
+    return false;
+}
+
+PuppetCreator::PuppetCreator(ProjectExplorer::Target *target, const Model *model)
+
+    : m_target(target)
     , m_availablePuppetType(FallbackPuppet)
     , m_model(model)
 #ifndef QMLDESIGNER_TEST
     , m_designerSettings(QmlDesignerPlugin::instance()->settings())
 #endif
-    , m_currentProject(project)
 {
 }
 
-QProcess *PuppetCreator::createPuppetProcess(const QString &puppetMode,
-                                             const QString &socketToken,
-                                             QObject *handlerObject,
-                                             const char *outputSlot,
-                                             const char *finishSlot) const
+QProcessUniquePointer PuppetCreator::createPuppetProcess(
+    const QString &puppetMode,
+    const QString &socketToken,
+    std::function<void()> processOutputCallback,
+    std::function<void(int, QProcess::ExitStatus)> processFinishCallback,
+    const QStringList &customOptions) const
 {
     return puppetProcess(qml2PuppetPath(m_availablePuppetType),
                          qmlPuppetDirectory(m_availablePuppetType),
                          puppetMode,
                          socketToken,
-                         handlerObject,
-                         outputSlot,
-                         finishSlot);
+                         processOutputCallback,
+                         processFinishCallback,
+                         customOptions);
 }
 
-
-QProcess *PuppetCreator::puppetProcess(const QString &puppetPath,
-                                       const QString &workingDirectory,
-                                       const QString &puppetMode,
-                                       const QString &socketToken,
-                                       QObject *handlerObject,
-                                       const char *outputSlot,
-                                       const char *finishSlot) const
+QProcessUniquePointer PuppetCreator::puppetProcess(
+    const QString &puppetPath,
+    const QString &workingDirectory,
+    const QString &puppetMode,
+    const QString &socketToken,
+    std::function<void()> processOutputCallback,
+    std::function<void(int, QProcess::ExitStatus)> processFinishCallback,
+    const QStringList &customOptions) const
 {
-    auto puppetProcess = new QProcess;
+    QProcessUniquePointer puppetProcess{new QProcess};
     puppetProcess->setObjectName(puppetMode);
     puppetProcess->setProcessEnvironment(processEnvironment());
 
-    QObject::connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit, puppetProcess, &QProcess::kill);
-    QObject::connect(puppetProcess, SIGNAL(finished(int,QProcess::ExitStatus)), handlerObject, finishSlot);
+    QObject::connect(QCoreApplication::instance(),
+                     &QCoreApplication::aboutToQuit,
+                     puppetProcess.get(),
+                     &QProcess::kill);
+    QObject::connect(puppetProcess.get(),
+                     static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished),
+                     processFinishCallback);
 
 #ifndef QMLDESIGNER_TEST
     QString forwardOutput = m_designerSettings.value(DesignerSettingsKey::
@@ -214,10 +238,30 @@ QProcess *PuppetCreator::puppetProcess(const QString &puppetPath,
 #endif
     if (forwardOutput == puppetMode || forwardOutput == "all") {
         puppetProcess->setProcessChannelMode(QProcess::MergedChannels);
-        QObject::connect(puppetProcess, SIGNAL(readyRead()), handlerObject, outputSlot);
+        QObject::connect(puppetProcess.get(), &QProcess::readyRead, processOutputCallback);
     }
     puppetProcess->setWorkingDirectory(workingDirectory);
-    puppetProcess->start(puppetPath, {socketToken, puppetMode, "-graphicssystem raster"});
+
+    bool forceFreeType = false;
+    if (Utils::HostOsInfo::isWindowsHost() && m_target) {
+        const QVariant customData = m_target->additionalData("CustomForceFreeType");
+
+        if (customData.isValid())
+            forceFreeType = customData.toBool();
+    }
+
+    QString forceFreeTypeOption;
+    if (forceFreeType)
+        forceFreeTypeOption = "-platform windows:fontengine=freetype";
+
+    if (puppetMode == "custom") {
+        QStringList args = customOptions;
+        args << "-graphicssystem raster";
+        args << forceFreeTypeOption;
+        puppetProcess->start(puppetPath, args);
+    } else {
+        puppetProcess->start(puppetPath, {socketToken, puppetMode, "-graphicssystem raster", forceFreeTypeOption });
+    }
 
 #ifndef QMLDESIGNER_TEST
     QString debugPuppet = m_designerSettings.value(DesignerSettingsKey::
@@ -247,7 +291,7 @@ static QString idealProcessCount()
 bool PuppetCreator::build(const QString &qmlPuppetProjectFilePath) const
 {
     PuppetBuildProgressDialog progressDialog;
-    progressDialog.setParent(Core::ICore::mainWindow());
+    progressDialog.setParent(Core::ICore::dialogParent());
 
     m_compileLog.clear();
 
@@ -277,7 +321,7 @@ bool PuppetCreator::build(const QString &qmlPuppetProjectFilePath) const
             qmakeArguments.append("CONFIG+=release");
 #endif
             qmakeArguments.append(qmlPuppetProjectFilePath);
-            buildSucceeded = startBuildProcess(buildDirectory.path(), qmakeCommand(), qmakeArguments, &progressDialog);
+            buildSucceeded = startBuildProcess(buildDirectory.path().path(), qmakeCommand(), qmakeArguments, &progressDialog);
             if (buildSucceeded) {
                 progressDialog.show();
                 QString buildingCommand = buildCommand();
@@ -286,7 +330,7 @@ bool PuppetCreator::build(const QString &qmlPuppetProjectFilePath) const
                     buildArguments.append("-j");
                     buildArguments.append(idealProcessCount());
                 }
-                buildSucceeded = startBuildProcess(buildDirectory.path(), buildingCommand, buildArguments, &progressDialog);
+                buildSucceeded = startBuildProcess(buildDirectory.path().path(), buildingCommand, buildArguments, &progressDialog);
             }
 
             if (!buildSucceeded) {
@@ -322,6 +366,19 @@ static void warnAboutInvalidKit()
                                                                        ).arg(Core::Constants::IDE_DISPLAY_NAME));
 }
 
+static Utils::FilePath pathForBinPuppet(ProjectExplorer::Target *target)
+{
+    if (!target || !target->kit())
+        return {};
+
+    QtSupport::BaseQtVersion *currentQtVersion = QtSupport::QtKitAspect::qtVersion(target->kit());
+
+    if (currentQtVersion)
+        return currentQtVersion->binPath() / Utils::HostOsInfo::withExecutableSuffix("qml2puppet");
+
+    return {};
+}
+
 void PuppetCreator::createQml2PuppetExecutableIfMissing()
 {
     m_availablePuppetType = FallbackPuppet;
@@ -329,26 +386,29 @@ void PuppetCreator::createQml2PuppetExecutableIfMissing()
     if (!useOnlyFallbackPuppet()) {
         // check if there was an already failing try to get the UserSpacePuppet
         // -> imagine as result a FallbackPuppet and nothing will happen again
-        if (m_qml2PuppetForKitPuppetHash.value(m_kit->id(), UserSpacePuppet) == UserSpacePuppet ) {
+        if (m_qml2PuppetForKitPuppetHash.value(m_target->id(), UserSpacePuppet) == UserSpacePuppet ) {
             if (checkPuppetIsReady(qml2PuppetPath(UserSpacePuppet))) {
                 m_availablePuppetType = UserSpacePuppet;
             } else {
-                if (m_kit->isValid()) {
+                if (m_target->kit()->isValid()) {
                     bool buildSucceeded = build(qml2PuppetProjectFile());
                     if (buildSucceeded)
                         m_availablePuppetType = UserSpacePuppet;
                 } else {
                     warnAboutInvalidKit();
                 }
-                m_qml2PuppetForKitPuppetHash.insert(m_kit->id(), m_availablePuppetType);
+                m_qml2PuppetForKitPuppetHash.insert(m_target->id(), m_availablePuppetType);
             }
         }
+    } else if (m_target && m_target->kit() && m_target->kit()->isValid()) {
+        if (pathForBinPuppet(m_target).isExecutableFile())
+            m_availablePuppetType = BinPathPuppet;
     }
 }
 
 QString PuppetCreator::defaultPuppetToplevelBuildDirectory()
 {
-    return Core::ICore::userResourcePath() + "/qmlpuppet/";
+    return Core::ICore::userResourcePath("qmlpuppet/").toString();
 }
 
 QString PuppetCreator::qmlPuppetToplevelBuildDirectory() const
@@ -370,26 +430,34 @@ QString PuppetCreator::qmlPuppetDirectory(PuppetType puppetType) const
         return qmlPuppetToplevelBuildDirectory() + '/' + QCoreApplication::applicationVersion()
                 + '/' + QString::fromLatin1(qtHash());
 
-    return qmlPuppetFallbackDirectory();
+#ifndef QMLDESIGNER_TEST
+    if (puppetType == BinPathPuppet)
+        return pathForBinPuppet(m_target).toFileInfo().absoluteDir().canonicalPath();
+
+    return qmlPuppetFallbackDirectory(m_designerSettings);
+#else
+    return QString();
+#endif
 }
 
 QString PuppetCreator::defaultPuppetFallbackDirectory()
 {
     if (Utils::HostOsInfo::isMacHost())
-        return Core::ICore::libexecPath() + "/qmldesigner";
+        return Core::ICore::libexecPath("qmldesigner").toString();
     else
-        return Core::ICore::libexecPath();
+        return Core::ICore::libexecPath().toString();
 }
 
-QString PuppetCreator::qmlPuppetFallbackDirectory() const
+QString PuppetCreator::qmlPuppetFallbackDirectory(const DesignerSettings &settings)
 {
 #ifndef QMLDESIGNER_TEST
-    QString puppetFallbackDirectory = m_designerSettings.value(
-        DesignerSettingsKey::PUPPET_FALLBACK_DIRECTORY).toString();
-    if (puppetFallbackDirectory.isEmpty())
+    QString puppetFallbackDirectory = settings.value(
+        DesignerSettingsKey::PUPPET_DEFAULT_DIRECTORY).toString();
+    if (puppetFallbackDirectory.isEmpty() || !QFileInfo::exists(puppetFallbackDirectory))
         return defaultPuppetFallbackDirectory();
     return puppetFallbackDirectory;
 #else
+    Q_UNUSED(settings)
     return QString();
 #endif
 }
@@ -403,7 +471,7 @@ static void filterOutQtBaseImportPath(QStringList *stringList)
 {
     Utils::erase(*stringList, [](const QString &string) {
         QDir dir(string);
-        return dir.dirName() == "qml" && !dir.entryInfoList(QStringList("QtQuick.2"), QDir::Dirs).isEmpty();
+        return dir.dirName() == "qml" && !dir.entryInfoList(QStringList("QtTest"), QDir::Dirs).isEmpty();
     });
 }
 
@@ -411,19 +479,27 @@ QProcessEnvironment PuppetCreator::processEnvironment() const
 {
     static const QString pathSep = Utils::HostOsInfo::pathListSeparator();
     Utils::Environment environment = Utils::Environment::systemEnvironment();
-    if (!useOnlyFallbackPuppet())
-        m_kit->addToEnvironment(environment);
-    const QtSupport::BaseQtVersion *qt = QtSupport::QtKitInformation::qtVersion(m_kit);
-    if (QTC_GUARD(qt)) { // Kits without a Qt version should not have a puppet!
-        // Update PATH to include QT_HOST_BINS
-        const Utils::FileName qtBinPath = qt->binPath();
-        environment.prependOrSetPath(qtBinPath.toString());
+    if (QTC_GUARD(m_target)) {
+        if (!useOnlyFallbackPuppet() || m_availablePuppetType == BinPathPuppet) {
+            m_target->kit()->addToBuildEnvironment(environment);
+            const QtSupport::BaseQtVersion *qt = QtSupport::QtKitAspect::qtVersion(m_target->kit());
+            if (QTC_GUARD(qt)) { // Kits without a Qt version should not have a puppet!
+                // Update PATH to include QT_HOST_BINS
+                const Utils::FilePath qtBinPath = qt->hostBinPath();
+                environment.prependOrSetPath(qtBinPath.toString());
+            }
+        }
     }
     environment.set("QML_BAD_GUI_RENDER_LOOP", "true");
-    environment.set("QML_USE_MOCKUPS", "true");
     environment.set("QML_PUPPET_MODE", "true");
     environment.set("QML_DISABLE_DISK_CACHE", "true");
-    if (!environment.hasKey("QT_SCREEN_SCALE_FACTORS"))
+    environment.set("QMLPUPPET_RENDER_EFFECTS", "true");
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    if (!environment.hasKey("QT_SCREEN_SCALE_FACTORS") && !environment.hasKey("QT_SCALE_FACTOR")
+            && QApplication::testAttribute(Qt::AA_EnableHighDpiScaling))
+#else
+    if (!environment.hasKey("QT_SCREEN_SCALE_FACTORS") && !environment.hasKey("QT_SCALE_FACTOR"))
+#endif
         environment.set("QT_AUTO_SCREEN_SCALE_FACTOR", "1");
 
 #ifndef QMLDESIGNER_TEST
@@ -432,7 +508,7 @@ QProcessEnvironment PuppetCreator::processEnvironment() const
 #else
     const QString controlsStyle;
 #endif
-    if (!controlsStyle.isEmpty() && controlsStyle != "Default") {
+    if (!controlsStyle.isEmpty()) {
         environment.set("QT_QUICK_CONTROLS_STYLE", controlsStyle);
         environment.set("QT_LABS_CONTROLS_STYLE", controlsStyle);
     }
@@ -443,18 +519,24 @@ QProcessEnvironment PuppetCreator::processEnvironment() const
 
     const QString styleConfigFileName = getStyleConfigFileName();
 
-    /* QT_QUICK_CONTROLS_CONF is not supported for Qt Version < 5.8.1,
-     * but we can manually at least set the correct style. */
-    if (!styleConfigFileName.isEmpty()) {
-        QSettings infiFile(styleConfigFileName, QSettings::IniFormat);
-        environment.set("QT_QUICK_CONTROLS_STYLE", infiFile.value("Controls/Style", "Default").toString());
-    }
-
     if (!m_qrcMapping.isEmpty()) {
         environment.set("QMLDESIGNER_RC_PATHS", m_qrcMapping);
     }
+
+    if (usesVirtualKeyboard()) {
+        environment.set("QT_IM_MODULE", "qtvirtualkeyboard");
+        environment.set("QT_VIRTUALKEYBOARD_DESKTOP_DISABLE", "1");
+    }
+
 #ifndef QMLDESIGNER_TEST
-    QmlDesignerPlugin::instance()->viewManager().nodeInstanceView()->emitCustomNotification("PuppetStatus", {}, {QVariant(m_qrcMapping)});
+    // set env var if QtQuick3D import exists
+    QmlDesigner::Import import = QmlDesigner::Import::createLibraryImport("QtQuick3D", "1.0");
+    if (m_model->hasImport(import, true, true))
+        environment.set("QMLDESIGNER_QUICK3D_MODE", "true");
+    import = QmlDesigner::Import::createLibraryImport("QtCharts", "2.0");
+    if (m_model->hasImport(import, true, true))
+        environment.set("QMLDESIGNER_FORCE_QAPPLICATION", "true");
+    environment.set("QT_QUICK3D_DISABLE_PARTICLE_SYSTEMS", "1");
 #endif
 
     QStringList importPaths = m_model->importPaths();
@@ -466,47 +548,54 @@ QProcessEnvironment PuppetCreator::processEnvironment() const
     if (!styleConfigFileName.isEmpty())
         environment.appendOrSet("QT_QUICK_CONTROLS_CONF", styleConfigFileName);
 
-    if (m_currentProject) {
-        auto qmakeProject = qobject_cast<QmakeProjectManager::QmakeProject *>(m_currentProject);
-        if (qmakeProject) {
-            QStringList designerImports = qmakeProject->rootProjectNode()->variableValue(QmakeProjectManager::Variable::QmlDesignerImportPath);
-            importPaths.append(designerImports);
+    QStringList customFileSelectors;
+
+    if (m_target) {
+        QStringList designerImports = m_target->additionalData("QmlDesignerImportPath").toStringList();
+        importPaths.append(designerImports);
+
+        customFileSelectors = m_target->additionalData("CustomFileSelectorsData").toStringList();
+
+        if (auto multiLanguageAspect = QmlProjectManager::QmlMultiLanguageAspect::current(m_target)) {
+            if (!multiLanguageAspect->databaseFilePath().isEmpty())
+                environment.set("QT_MULTILANGUAGE_DATABASE", multiLanguageAspect->databaseFilePath().toString());
         }
     }
 
+    customFileSelectors.append("DesignMode");
+
     if (m_availablePuppetType == FallbackPuppet)
         importPaths.prepend(QLibraryInfo::location(QLibraryInfo::Qml2ImportsPath));
 
     environment.appendOrSet("QML2_IMPORT_PATH", importPaths.join(pathSep), pathSep);
 
+    if (!customFileSelectors.isEmpty())
+        environment.appendOrSet("QML_FILE_SELECTORS", customFileSelectors.join(","), pathSep);
+
     qCInfo(puppetStart) << Q_FUNC_INFO;
     qCInfo(puppetStart) << "Puppet qrc mapping" << m_qrcMapping;
     qCInfo(puppetStart) << "Puppet import paths:" << importPaths;
     qCInfo(puppetStart) << "Puppet environment:" << environment.toStringList();
+    qCInfo(puppetStart) << "Puppet selectors:" << customFileSelectors;
 
     return environment.toProcessEnvironment();
 }
 
 QString PuppetCreator::buildCommand() const
 {
-    Utils::Environment environment = Utils::Environment::systemEnvironment();
-    m_kit->addToEnvironment(environment);
+    const Utils::Environment environment = m_target->kit()->buildEnvironment();
 
-    ProjectExplorer::ToolChain *toolChain
-            = ProjectExplorer::ToolChainKitInformation::toolChain(m_kit,
-                                                                  ProjectExplorer::Constants::CXX_LANGUAGE_ID);
-
-    if (toolChain)
-        return toolChain->makeCommand(environment);
+    if (ToolChain *toolChain = ToolChainKitAspect::cxxToolChain(m_target->kit()))
+        return toolChain->makeCommand(environment).toString();
 
     return QString();
 }
 
 QString PuppetCreator::qmakeCommand() const
 {
-    QtSupport::BaseQtVersion *currentQtVersion = QtSupport::QtKitInformation::qtVersion(m_kit);
+    QtSupport::BaseQtVersion *currentQtVersion = QtSupport::QtKitAspect::qtVersion(m_target->kit());
     if (currentQtVersion)
-        return currentQtVersion->qmakeCommand().toString();
+        return currentQtVersion->qmakeFilePath().toString();
 
     return QString();
 }
@@ -564,7 +653,7 @@ bool PuppetCreator::startBuildProcess(const QString &buildDirectoryPath,
 
 QString PuppetCreator::puppetSourceDirectoryPath()
 {
-    return Core::ICore::resourcePath() + "/qml/qmlpuppet";
+    return Core::ICore::resourcePath("qml/qmlpuppet").toString();
 }
 
 QString PuppetCreator::qml2PuppetProjectFile()
@@ -591,7 +680,7 @@ static bool nonEarlyQt5Version(const QtSupport::QtVersionNumber &currentQtVersio
 
 bool PuppetCreator::qtIsSupported() const
 {
-    QtSupport::BaseQtVersion *currentQtVersion = QtSupport::QtKitInformation::qtVersion(m_kit);
+    QtSupport::BaseQtVersion *currentQtVersion = QtSupport::QtKitAspect::qtVersion(m_target->kit());
 
     return currentQtVersion
             && currentQtVersion->isValid()
