@@ -43,9 +43,15 @@
 #include <QPlainTextEdit>
 #include <QFileInfo>
 #include <QDir>
+#include <QImageReader>
 
+#include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 
+#ifndef QMLDESIGNER_TEST
+#include <designermcumanager.h>
+#endif
+
 namespace QmlDesigner {
 
 bool QmlItemNode::isItemOrWindow(const ModelNode &modelNode)
@@ -73,46 +79,104 @@ QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view,
     return QmlItemNode(createQmlObjectNode(view, itemLibraryEntry, position, parentQmlItemNode));
 }
 
-QmlItemNode QmlItemNode::createQmlItemNodeFromImage(AbstractView *view, const QString &imageName, const QPointF &position, QmlItemNode parentQmlItemNode)
+QmlItemNode QmlItemNode::createQmlItemNodeFromImage(AbstractView *view, const QString &imageName, const QPointF &position, QmlItemNode parentQmlItemNode, bool executeInTransaction)
 {
     if (!parentQmlItemNode.isValid())
         parentQmlItemNode = QmlItemNode(view->rootModelNode());
 
     NodeAbstractProperty parentProperty = parentQmlItemNode.defaultNodeAbstractProperty();
 
-    return QmlItemNode::createQmlItemNodeFromImage(view, imageName, position, parentProperty);
+    return QmlItemNode::createQmlItemNodeFromImage(view, imageName, position, parentProperty, executeInTransaction);
 }
 
-QmlItemNode QmlItemNode::createQmlItemNodeFromImage(AbstractView *view, const QString &imageName, const QPointF &position, NodeAbstractProperty parentproperty)
+QmlItemNode QmlItemNode::createQmlItemNodeFromImage(AbstractView *view, const QString &imageName, const QPointF &position, NodeAbstractProperty parentproperty, bool executeInTransaction)
 {
     QmlItemNode newQmlItemNode;
 
-    if (parentproperty.isValid() && view->model()->hasNodeMetaInfo("QtQuick.Image")) {
-        view->executeInTransaction("QmlItemNode::createQmlItemNodeFromImage", [=, &newQmlItemNode, &parentproperty](){
-            NodeMetaInfo metaInfo = view->model()->metaInfo("QtQuick.Image");
-            QList<QPair<PropertyName, QVariant> > propertyPairList;
-            propertyPairList.append({PropertyName("x"), QVariant(qRound(position.x()))});
-            propertyPairList.append({PropertyName("y"), QVariant(qRound(position.y()))});
+    auto doCreateQmlItemNodeFromImage = [=, &newQmlItemNode, &parentproperty]() {
+        NodeMetaInfo metaInfo = view->model()->metaInfo("QtQuick.Image");
+        QList<QPair<PropertyName, QVariant> > propertyPairList;
+        propertyPairList.append({PropertyName("x"), QVariant(qRound(position.x()))});
+        propertyPairList.append({PropertyName("y"), QVariant(qRound(position.y()))});
 
-            QString relativeImageName = imageName;
+        QString relativeImageName = imageName;
 
-            //use relative path
-            if (QFileInfo::exists(view->model()->fileUrl().toLocalFile())) {
-                QDir fileDir(QFileInfo(view->model()->fileUrl().toLocalFile()).absolutePath());
-                relativeImageName = fileDir.relativeFilePath(imageName);
-                propertyPairList.append({PropertyName("source"), QVariant(relativeImageName)});
-            }
+        //use relative path
+        if (QFileInfo::exists(view->model()->fileUrl().toLocalFile())) {
+            QDir fileDir(QFileInfo(view->model()->fileUrl().toLocalFile()).absolutePath());
+            relativeImageName = fileDir.relativeFilePath(imageName);
+            propertyPairList.append({PropertyName("source"), QVariant(relativeImageName)});
+        }
 
-            newQmlItemNode = QmlItemNode(view->createModelNode("QtQuick.Image", metaInfo.majorVersion(), metaInfo.minorVersion(), propertyPairList));
-            parentproperty.reparentHere(newQmlItemNode);
+        TypeName type("QtQuick.Image");
+        QImageReader reader(imageName);
+        if (reader.supportsAnimation())
+            type = "QtQuick.AnimatedImage";
 
-            newQmlItemNode.setId(view->generateNewId(QLatin1String("image")));
+        newQmlItemNode = QmlItemNode(view->createModelNode(type, metaInfo.majorVersion(), metaInfo.minorVersion(), propertyPairList));
+        parentproperty.reparentHere(newQmlItemNode);
 
-            newQmlItemNode.modelNode().variantProperty("fillMode").setEnumeration("Image.PreserveAspectFit");
+        QFileInfo fi(relativeImageName);
+        newQmlItemNode.setId(view->generateNewId(fi.baseName(), "image"));
 
-            Q_ASSERT(newQmlItemNode.isValid());
-        });
-    }
+        newQmlItemNode.modelNode().variantProperty("fillMode").setEnumeration("Image.PreserveAspectFit");
+
+        Q_ASSERT(newQmlItemNode.isValid());
+    };
+
+    if (executeInTransaction)
+        view->executeInTransaction("QmlItemNode::createQmlItemNodeFromImage", doCreateQmlItemNodeFromImage);
+    else
+        doCreateQmlItemNodeFromImage();
+
+    return newQmlItemNode;
+}
+
+QmlItemNode QmlItemNode::createQmlItemNodeFromFont(AbstractView *view,
+                                                   const QString &fontFamily,
+                                                   const QPointF &position,
+                                                   QmlItemNode parentQmlItemNode,
+                                                   bool executeInTransaction)
+{
+    if (!parentQmlItemNode.isValid())
+        parentQmlItemNode = QmlItemNode(view->rootModelNode());
+
+    NodeAbstractProperty parentProperty = parentQmlItemNode.defaultNodeAbstractProperty();
+
+    return QmlItemNode::createQmlItemNodeFromFont(view, fontFamily, position,
+                                                  parentProperty, executeInTransaction);
+}
+
+QmlItemNode QmlItemNode::createQmlItemNodeFromFont(AbstractView *view,
+                                                   const QString &fontFamily,
+                                                   const QPointF &position,
+                                                   NodeAbstractProperty parentproperty,
+                                                   bool executeInTransaction)
+{
+    QmlItemNode newQmlItemNode;
+
+    auto doCreateQmlItemNodeFromFont = [=, &newQmlItemNode, &parentproperty]() {
+        NodeMetaInfo metaInfo = view->model()->metaInfo("QtQuick.Text");
+        QList<QPair<PropertyName, QVariant>> propertyPairList;
+        propertyPairList.append({PropertyName("x"), QVariant(qRound(position.x()))});
+        propertyPairList.append({PropertyName("y"), QVariant(qRound(position.y()))});
+        propertyPairList.append({PropertyName("font.family"), QVariant(fontFamily)});
+        propertyPairList.append({PropertyName("font.pointSize"), 20});
+        propertyPairList.append({PropertyName("text"), QVariant(fontFamily)});
+
+        newQmlItemNode = QmlItemNode(view->createModelNode("QtQuick.Text", metaInfo.majorVersion(),
+                                                           metaInfo.minorVersion(), propertyPairList));
+        parentproperty.reparentHere(newQmlItemNode);
+
+        newQmlItemNode.setId(view->generateNewId("text", "text"));
+
+        Q_ASSERT(newQmlItemNode.isValid());
+    };
+
+    if (executeInTransaction)
+        view->executeInTransaction("QmlItemNode::createQmlItemNodeFromImage", doCreateQmlItemNodeFromFont);
+    else
+        doCreateQmlItemNodeFromFont();
 
     return newQmlItemNode;
 }
@@ -282,6 +346,41 @@ bool QmlItemNode::modelIsResizable() const
             && !modelIsInLayout();
 }
 
+static bool isMcuRotationAllowed(QString itemName, bool hasChildren)
+{
+#ifndef QMLDESIGNER_TEST
+    const QString propName = "rotation";
+    const DesignerMcuManager &manager = DesignerMcuManager::instance();
+    if (manager.isMCUProject()) {
+        if (manager.allowedItemProperties().contains(itemName)) {
+            const DesignerMcuManager::ItemProperties properties =
+                    manager.allowedItemProperties().value(itemName);
+            if (properties.properties.contains(propName)) {
+                if (hasChildren)
+                    return properties.allowChildren;
+                return true;
+            }
+        }
+
+        if (manager.bannedItems().contains(itemName))
+            return false;
+
+        if (manager.bannedProperties().contains(propName))
+            return false;
+    }
+#endif
+
+    return true;
+}
+
+bool QmlItemNode::modelIsRotatable() const
+{
+    return !modelNode().hasBindingProperty("rotation")
+            && itemIsResizable(modelNode())
+            && !modelIsInLayout()
+            && isMcuRotationAllowed(QString::fromUtf8(modelNode().type()), hasChildren());
+}
+
 bool QmlItemNode::modelIsInLayout() const
 {
     if (modelNode().hasParentProperty()) {
@@ -531,6 +630,30 @@ void QmlItemNode::setSize(const QSizeF &size)
         setVariantProperty("height", qRound(size.height()));
 }
 
+void QmlItemNode::setRotation(const qreal &angle)
+{
+    if (!hasBindingProperty("rotation"))
+        setVariantProperty("rotation", angle);
+}
+
+qreal QmlItemNode::rotation() const
+{
+    if (hasProperty("rotation") && !hasBindingProperty("rotation")) {
+        return modelNode().variantProperty("rotation").value().toReal();
+    }
+
+    return 0.0;
+}
+
+QVariant QmlItemNode::transformOrigin()
+{
+    if (hasProperty("transformOrigin")) {
+        return modelNode().variantProperty("transformOrigin").value();
+    }
+
+    return {};
+}
+
 bool QmlFlowItemNode::isValid() const
 {
     return isValidQmlFlowItemNode(modelNode());
@@ -545,7 +668,7 @@ bool QmlFlowItemNode::isValidQmlFlowItemNode(const ModelNode &modelNode)
 QList<QmlFlowActionAreaNode> QmlFlowItemNode::flowActionAreas() const
 {
     QList<QmlFlowActionAreaNode> list;
-    for (const ModelNode &node : allDirectSubModelNodes())
+    for (const ModelNode node : allDirectSubModelNodes())
         if (QmlFlowActionAreaNode::isValidQmlFlowActionAreaNode(node))
             list.append(node);
     return list;
@@ -600,6 +723,7 @@ void QmlFlowActionAreaNode::assignTargetFlowItem(const QmlFlowTargetNode &flowIt
 
 QmlFlowItemNode QmlFlowActionAreaNode::flowItemParent() const
 {
+    QTC_ASSERT(modelNode().hasParentProperty(), return QmlFlowItemNode({}));
     return modelNode().parentProperty().parentModelNode();
 }
 
@@ -613,32 +737,6 @@ void QmlFlowActionAreaNode::destroyTarget()
     }
 }
 
-ModelNode QmlFlowActionAreaNode::decisionNodeForTransition(const ModelNode &transition) const
-{
-    ModelNode target = targetTransition();
-
-    if (target.isValid() && target.hasMetaInfo() && QmlVisualNode::isFlowTransition(target)) {
-
-        ModelNode finalTarget = target.bindingProperty("to").resolveToModelNode();
-
-        if (finalTarget.isValid() && finalTarget.hasMetaInfo() && QmlVisualNode::isFlowDecision(finalTarget)) {
-            if (finalTarget.hasBindingProperty("targets")
-                    && finalTarget.bindingProperty("targets").resolveToModelNodeList().contains(transition))
-                return finalTarget;
-        }
-        QmlFlowViewNode flowView(view()->rootModelNode());
-        if (flowView.isValid()) {
-            for (const ModelNode target : flowView.decicions()) {
-                if (target.hasBindingProperty("targets")
-                        && target.bindingProperty("targets").resolveToModelNodeList().contains(transition))
-                    return target;
-            }
-        }
-    }
-
-    return {};
-}
-
 bool QmlFlowViewNode::isValid() const
 {
     return isValidQmlFlowViewNode(modelNode());
@@ -653,7 +751,7 @@ bool QmlFlowViewNode::isValidQmlFlowViewNode(const ModelNode &modelNode)
 QList<QmlFlowItemNode> QmlFlowViewNode::flowItems() const
 {
     QList<QmlFlowItemNode> list;
-    for (const ModelNode &node : allDirectSubModelNodes())
+    for (const ModelNode node : allDirectSubModelNodes())
         if (QmlFlowItemNode::isValidQmlFlowItemNode(node)
                 || QmlVisualNode::isFlowDecision(node)
                 || QmlVisualNode::isFlowWildcard(node))
@@ -664,9 +762,7 @@ QList<QmlFlowItemNode> QmlFlowViewNode::flowItems() const
 
 ModelNode QmlFlowViewNode::addTransition(const QmlFlowTargetNode &from, const QmlFlowTargetNode &to)
 {
-    ModelNode transition = view()->createModelNode("FlowView.FlowTransition", 1, 0);
-
-    nodeListProperty("flowTransitions").reparentHere(transition);
+    ModelNode transition = createTransition();
 
     QmlFlowTargetNode f = from;
     QmlFlowTargetNode t = to;
@@ -678,17 +774,15 @@ ModelNode QmlFlowViewNode::addTransition(const QmlFlowTargetNode &from, const Qm
     return transition;
 }
 
-const QList<ModelNode> QmlFlowViewNode::transitions() const
+QList<ModelNode> QmlFlowViewNode::transitions() const
 {
     if (modelNode().nodeListProperty("flowTransitions").isValid())
         return modelNode().nodeListProperty("flowTransitions").toModelNodeList();
 
     return {};
-
-
 }
 
-const QList<ModelNode> QmlFlowViewNode::wildcards() const
+QList<ModelNode> QmlFlowViewNode::wildcards() const
 {
     if (modelNode().nodeListProperty("flowWildcards").isValid())
         return modelNode().nodeListProperty("flowWildcards").toModelNodeList();
@@ -696,7 +790,7 @@ const QList<ModelNode> QmlFlowViewNode::wildcards() const
     return {};
 }
 
-const QList<ModelNode> QmlFlowViewNode::decicions() const
+QList<ModelNode> QmlFlowViewNode::decicions() const
 {
     if (modelNode().nodeListProperty("flowDecisions").isValid())
         return modelNode().nodeListProperty("flowDecisions").toModelNodeList();
@@ -706,13 +800,12 @@ const QList<ModelNode> QmlFlowViewNode::decicions() const
 
 QList<ModelNode> QmlFlowViewNode::transitionsForTarget(const ModelNode &modelNode)
 {
-    QList<ModelNode> list;
-    for (const ModelNode &transition : transitions()) {
-        if (transition.hasBindingProperty("to")
-                && transition.bindingProperty("to").resolveToModelNode() == modelNode)
-            list.append(transition);
-    }
-    return list;
+    return transitionsForProperty("to", modelNode);
+}
+
+QList<ModelNode> QmlFlowViewNode::transitionsForSource(const ModelNode &modelNode)
+{
+    return transitionsForProperty("from", modelNode);
 }
 
 void QmlFlowViewNode::removeDanglingTransitions()
@@ -732,6 +825,9 @@ void QmlFlowTargetNode::assignTargetItem(const QmlFlowTargetNode &node)
 {
     if (QmlFlowActionAreaNode::isValidQmlFlowActionAreaNode(modelNode())) {
         QmlFlowActionAreaNode(modelNode()).assignTargetFlowItem(node);
+
+    } else if (isFlowItem()) {
+        flowView().addTransition(modelNode(), node);
     } else if (isFlowWildcard()) {
         destroyTargets();
         ModelNode transition = flowView().addTransition(ModelNode(),
@@ -782,12 +878,38 @@ QmlFlowViewNode QmlFlowTargetNode::flowView() const
     return view()->rootModelNode();
 }
 
+ModelNode QmlFlowItemNode::decisionNodeForTransition(const ModelNode &transition)
+{
+    ModelNode target = transition;
+
+    if (target.isValid() && target.hasMetaInfo() && QmlVisualNode::isFlowTransition(target)) {
+
+        ModelNode finalTarget = target.bindingProperty("to").resolveToModelNode();
+
+        if (finalTarget.isValid() && finalTarget.hasMetaInfo() && QmlVisualNode::isFlowDecision(finalTarget)) {
+            if (finalTarget.hasBindingProperty("targets")
+                    && finalTarget.bindingProperty("targets").resolveToModelNodeList().contains(transition))
+                return finalTarget;
+        }
+        QmlFlowViewNode flowView(transition.view()->rootModelNode());
+        if (flowView.isValid()) {
+            for (const ModelNode &target : flowView.decicions()) {
+                if (target.hasBindingProperty("targets")
+                        && target.bindingProperty("targets").resolveToModelNodeList().contains(transition))
+                    return target;
+            }
+        }
+    }
+
+    return {};
+}
+
 ModelNode QmlFlowTargetNode::findSourceForDecisionNode() const
 {
     if (!isFlowDecision())
         return {};
 
-    for (const ModelNode transition : flowView().transitionsForTarget(modelNode())) {
+    for (const ModelNode &transition : flowView().transitionsForTarget(modelNode())) {
         if (transition.hasBindingProperty("from")) {
             const ModelNode source = transition.bindingProperty("from").resolveToModelNode();
             if (source.isValid()) {
@@ -830,4 +952,79 @@ void QmlFlowViewNode::removeAllTransitions()
         removeProperty("flowTransitions");
 }
 
+void QmlFlowViewNode::setStartFlowItem(const QmlFlowItemNode &flowItem)
+{
+    QTC_ASSERT(flowItem.isValid(), return);
+    QmlFlowItemNode item = flowItem;
+
+    ModelNode transition;
+
+    for (const ModelNode &node : transitionsForSource(modelNode()))
+        transition = node;
+    if (!transition.isValid())
+        transition = createTransition();
+
+    transition.bindingProperty("from").setExpression(modelNode().validId());
+    transition.bindingProperty("to").setExpression(item.validId());
+}
+
+ModelNode QmlFlowViewNode::createTransition()
+{
+    ModelNode transition = view()->createModelNode("FlowView.FlowTransition", 1, 0);
+    nodeListProperty("flowTransitions").reparentHere(transition);
+
+    return transition;
+}
+
+QList<ModelNode> QmlFlowViewNode::transitionsForProperty(const PropertyName &propertyName,
+                                                         const ModelNode &modelNode)
+{
+    QList<ModelNode> list;
+    for (const ModelNode &transition : transitions()) {
+        if (transition.hasBindingProperty(propertyName)
+                && transition.bindingProperty(propertyName).resolveToModelNode() == modelNode)
+            list.append(transition);
+    }
+    return list;
+}
+
+PropertyNameList QmlFlowViewNode::st_mouseSignals = { "clicked", "doubleClicked", "pressAndHold",
+                                                      "pressed", "released", "wheel" };
+
+QList<QmlConnections> QmlFlowViewNode::getAssociatedConnections(const ModelNode &node)
+{
+    if (!node.isValid())
+        return {};
+
+    AbstractView *view = node.view();
+
+    return Utils::transform<QList<QmlConnections>>(Utils::filtered(view->allModelNodes(),
+                                                                   [&node](const ModelNode &n) {
+        const QmlConnections connection(n);
+        if (!connection.isValid())
+            return false;
+
+        const QList<SignalHandlerProperty> &signalProperties = connection.signalProperties();
+        for (const SignalHandlerProperty &property : signalProperties) {
+            auto signalWithoutPrefix = SignalHandlerProperty::prefixRemoved(property.name());
+            const QStringList sourceComponents = property.source().split(".");
+            QString sourceId;
+            QString sourceProperty;
+            if (sourceComponents.size() > 1) {
+                sourceId = sourceComponents[0];
+                sourceProperty = sourceComponents[1];
+            }
+
+            if (st_mouseSignals.contains(signalWithoutPrefix)
+                && sourceId == node.id()
+                && sourceProperty == "trigger()")
+                return true;
+        }
+
+        return false;
+    }), [](const ModelNode &n) {
+        return QmlConnections(n);
+    });
+}
+
 } //QmlDesigner
