@@ -43,14 +43,13 @@
 #include <qmljs/parser/qmljsast_p.h>
 #include <qmljs/parser/qmljsastfwd_p.h>
 #include <qmljs/qmljsutils.h>
-#include <qmljs/qmljsqrcparser.h>
 #include <texteditor/texteditor.h>
 #include <utils/executeondestruction.h>
+#include <utils/qrcparser.h>
 #include <utils/tooltip/tooltip.h>
 
 #include <QDir>
 #include <QList>
-#include <QStringRef>
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
 
@@ -59,13 +58,12 @@ using namespace QmlJS;
 using namespace TextEditor;
 
 namespace QmlJSEditor {
-namespace Internal {
 
 namespace {
 
     QString textAt(const Document::Ptr doc,
-                   const AST::SourceLocation &from,
-                   const AST::SourceLocation &to)
+                   const SourceLocation &from,
+                   const SourceLocation &to)
     {
         return doc->source().mid(from.offset, to.end() - from.begin());
     }
@@ -133,7 +131,7 @@ static inline QString getModuleName(const ScopeChain &scopeChain, const Document
             const QString name = relativeDir.replace(QLatin1Char('/'), QLatin1Char('.'));
             return name;
         } else if (importInfo.isValid() && importInfo.type() == ImportType::QrcDirectory) {
-            QString path = QrcParser::normalizedQrcDirectoryPath(importInfo.path());
+            QString path = Utils::QrcParser::normalizedQrcDirectoryPath(importInfo.path());
             path = path.mid(1, path.size() - ((path.size() > 1) ? 2 : 1));
             const QString name = path.replace(QLatin1Char('/'), QLatin1Char('.'));
             return name;
@@ -174,7 +172,7 @@ bool QmlJSHoverHandler::setQmlTypeHelp(const ScopeChain &scopeChain, const Docum
     QRegularExpressionMatch m = version.match(moduleName);
     if (m.hasMatch()) {
         QMap<QString, QUrl> filteredUrlMap;
-        QStringRef maj = m.capturedRef(2);
+        const QString maj = m.captured(2);
         for (const HelpItem::Link &link : links) {
             QString urlModuleName = link.second.path().split('/')[1];
             if (urlModuleName.contains(maj))
@@ -517,6 +515,5 @@ bool QmlJSHoverHandler::setQmlHelpItem(const ScopeChain &scopeChain,
     return false;
 }
 
-} // namespace Internal
 } // namespace QmlJSEditor
 
