@@ -58,7 +58,7 @@ void SshKeyDeployer::deployPublicKey(const SshConnectionParameters &sshParams,
     cleanup();
 
     Utils::FileReader reader;
-    if (!reader.fetch(keyFilePath)) {
+    if (!reader.fetch(Utils::FilePath::fromString(keyFilePath))) {
         emit error(tr("Public key error: %1").arg(reader.errorString()));
         return;
     }
@@ -67,9 +67,10 @@ void SshKeyDeployer::deployPublicKey(const SshConnectionParameters &sshParams,
             this, &SshKeyDeployer::handleConnectionFailure);
     connect(&d->deployProcess, &SshRemoteProcessRunner::processClosed,
             this, &SshKeyDeployer::handleKeyUploadFinished);
-    const QByteArray command = "test -d .ssh "
-        "|| mkdir .ssh && chmod 0700 .ssh && echo '"
-        + reader.data() + "' >> .ssh/authorized_keys && chmod 0600 .ssh/authorized_keys";
+    const QString command = "test -d .ssh "
+        "|| mkdir -p ~/.ssh && chmod 0700 .ssh && echo '"
+            + QString::fromLocal8Bit(reader.data())
+            + "' >> .ssh/authorized_keys && chmod 0600 .ssh/authorized_keys";
     d->deployProcess.run(command, sshParams);
 }
 
