@@ -72,12 +72,13 @@ void ValgrindToolRunner::start()
     m_progress.reportStarted();
 
 #if VALGRIND_DEBUG_OUTPUT
-    emit outputReceived(tr("Valgrind options: %1").arg(toolArguments().join(' ')), DebugFormat);
-    emit outputReceived(tr("Working directory: %1").arg(runnable().workingDirectory), DebugFormat);
-    emit outputReceived(tr("Command line arguments: %1").arg(runnable().debuggeeArgs), DebugFormat);
+    emit outputReceived(tr("Valgrind options: %1").arg(toolArguments().join(' ')), LogMessageFormat);
+    emit outputReceived(tr("Working directory: %1").arg(runnable().workingDirectory), LogMessageFormat);
+    emit outputReceived(tr("Command line arguments: %1").arg(runnable().debuggeeArgs), LogMessageFormat);
 #endif
 
-    CommandLine valgrind{m_settings.valgrindExecutable()};
+    CommandLine valgrind{m_settings.valgrindExecutable.value()};
+    valgrind.addArgs(m_settings.valgrindArguments.value(), CommandLine::Raw);
     valgrind.addArgs(genericToolArguments());
     valgrind.addArgs(toolArguments());
 
@@ -123,7 +124,7 @@ QStringList ValgrindToolRunner::genericToolArguments() const
 {
     QString smcCheckValue;
 
-    switch (m_settings.selfModifyingCodeDetection()) {
+    switch (m_settings.selfModifyingCodeDetection.value()) {
     case ValgrindBaseSettings::DetectSmcNo:
         smcCheckValue = "none";
         break;
@@ -149,7 +150,7 @@ void ValgrindToolRunner::handleProgressCanceled()
 
 void ValgrindToolRunner::handleProgressFinished()
 {
-    QApplication::alert(ICore::mainWindow(), 3000);
+    QApplication::alert(ICore::dialogParent(), 3000);
 }
 
 void ValgrindToolRunner::runnerFinished()
@@ -174,7 +175,7 @@ void ValgrindToolRunner::receiveProcessOutput(const QString &output, OutputForma
 void ValgrindToolRunner::receiveProcessError(const QString &message, QProcess::ProcessError error)
 {
     if (error == QProcess::FailedToStart) {
-        const QString valgrind = m_settings.valgrindExecutable();
+        const QString valgrind = m_settings.valgrindExecutable.value();
         if (!valgrind.isEmpty())
             appendMessage(tr("Error: \"%1\" could not be started: %2").arg(valgrind, message), ErrorMessageFormat);
         else
