@@ -34,10 +34,6 @@
 # include <qmutex.h>
 #endif
 
-#ifndef QT_NO_TEXTCODEC
-QT_FORWARD_DECLARE_CLASS(QTextCodec)
-#endif
-
 #ifdef PROEVALUATOR_DUAL_VFS
 # ifndef PROEVALUATOR_CUMULATIVE
 #  error PROEVALUATOR_DUAL_VFS requires PROEVALUATOR_CUMULATIVE
@@ -72,10 +68,13 @@ public:
     Q_DECLARE_FLAGS(VfsFlags, VfsFlag)
 
     QMakeVfs();
+    ~QMakeVfs();
+
+    static void ref();
+    static void deref();
 
     int idForFileName(const QString &fn, VfsFlags flags);
     QString fileNameForId(int id);
-    static void clearIds();
     bool writeFile(int id, QIODevice::OpenMode mode, VfsFlags flags, const QString &contents, QString *errStr);
     ReadResult readFile(int id, QString *contents, QString *errStr);
     bool exists(const QString &fn, QMakeVfs::VfsFlags flags);
@@ -85,14 +84,11 @@ public:
     void invalidateContents();
 #endif
 
-#ifndef QT_NO_TEXTCODEC
-    void setTextCodec(const QTextCodec *textCodec);
-#endif
-
 private:
 #ifdef PROEVALUATOR_THREAD_SAFE
     static QMutex s_mutex;
 #endif
+    static int s_refCount;
     static QAtomicInt s_fileIdCounter;
     // Qt Creator's ProFile cache is a singleton to maximize its cross-project
     // effectiveness (shared prf files from QtVersions).
@@ -121,9 +117,6 @@ private:
     QString m_magicMissing;
     QString m_magicExisting;
 #endif
-#ifndef QT_NO_TEXTCODEC
-    const QTextCodec *m_textCodec;
-#endif
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QMakeVfs::VfsFlags)
